/*===========================================================================*/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*===========================================================================*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "rssmb.h"
#include "rssmb_utility.h"

#define IEEE_ERROR_BITS         0x3C    /* AND mask for ESR checking */
#define BUFFER_SIZE             2048UL    /* File I/O buffer size */

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssmb_invalidViBooleanRange (ViSession instrSession, ViBoolean val)
{
    rssmb_instrRange    instrPtr;
    ViStatus rssmb_status = VI_SUCCESS;
    rssmb_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    if ((rssmb_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;

}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssmb_invalidViInt32Range (ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max)
{
    rssmb_instrRange    instrPtr;
    ViStatus rssmb_status = VI_SUCCESS;
    rssmb_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    if ((rssmb_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssmb_invalidViReal64Range (ViSession instrSession, ViReal64 val, ViReal64 min, ViReal64 max)
{
    rssmb_instrRange    instrPtr;
    ViStatus rssmb_status = VI_SUCCESS;
    rssmb_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr);

    if ((rssmb_status == VI_SUCCESS) && (instrPtr->rangeChecking == VI_TRUE) )
        return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
    else
        return VI_FALSE;
}
/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rssmb_checkStatus (ViSession instrSession)
{
    ViStatus            rssmb_status = VI_SUCCESS;
    rssmb_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> errorChecking != VI_FALSE)
        {
            ViInt32 esrByte = 0;

            /*--------------- Read the Standard Event Register ------------------*/
            CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

            /*---------------- Check if any error or message bit was asserted ---*/
            rssmb_status = ((esrByte & IEEE_ERROR_BITS) != 0) ? RSSMB_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }
    }

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rssmb_waitOPC (ViSession instrSession, ViPString command,
                ViPString instrResp, ViInt32 timeout)
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViUInt16    response        = 0;
    ViInt32     counter         = 0;
    ViInt32     oldTimeout      = 0;
    ViUInt16    intf       = VI_INTF_GPIB;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial intf */
            CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) timeout));
            if ((rssmb_status = viQueryf (instrSession, "%s\n", "%[^\r\n]", command, instrResp)) < 0)
            {
                (void) viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout);
                return rssmb_status;
            }
            CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, (ViAttrState) oldTimeout));
        break;
        default: /* Default case */
            CHECKERR (viPrintf (instrSession, "%s\n", command));

            while ((response & 0x10) == 0) /* (queries MAV in STB) */
                {

                CHECKERR (viReadSTB (instrSession, &response));

                (void) rssmb_delay (instrSession, 0.005);
                counter+= 5;
                if (counter >= timeout)
                    return VI_ERROR_TMO;
                }

            CHECKERR (viScanf (instrSession, "%[^\r\n]", instrResp));
        break;
        }

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rssmb_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    size_t      length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc(length + 1);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;

    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    length++;
    GOCHECKERR (viWrite (instrSession, (ViPBuf) cmd, (ViUInt32) length, VI_NULL));
    GOCHECKERR (rssmb_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssmb_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViString parameterName)
{
    ViChar      *wrt_buf;
    ViStatus    rssmb_status = VI_SUCCESS;

    if ((max - min) > 0)
        if (rssmb_invalidViInt32Range (instrSession, value, min, max) == VI_TRUE)
	    {
	       rssmb_instrRange   instrPtr;
	       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	       return VI_ERROR_INSTR_PARAMETER;
	    }

    wrt_buf = (ViChar*) malloc (strlen (cmd) + 20);
    
    sprintf (wrt_buf, "%s %ld", cmd, value);

    rssmb_status = rssmb_sendCmd (instrSession, wrt_buf);
    
    free (wrt_buf);
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssmb_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rssmb_checkStatus (instrSession));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssmb_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViString parameterName)
{
    return rssmb_setViString (instrSession,
                              cmd,
                              (value == VI_TRUE) ? "ON" : "OFF",
                              parameterName);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rssmb_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[4];

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    CHECKERR (rssmb_getViString (instrSession,
                                cmd,
                                buffer, 4,
                                0));

    *value = (ViBoolean) ((strncmp (buffer, "1", 1) == 0) || (strncmp (buffer,"ON",2) == 0));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssmb_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssmb_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
	    {
	       rssmb_instrRange   instrPtr;
	       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	       return VI_ERROR_INSTR_PARAMETER;
	    }

    sprintf(buffer,"%s %.12lG", cmd, value);

    return rssmb_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rssmb_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    CHECKERR(viPrintf(instrSession, "%s\n",cmd));
    CHECKERR(viScanf (instrSession, "%le",value));
    CHECKERR(rssmb_checkStatus (instrSession));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 with Units                                         */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rssmb_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViString parameterName)
{
    ViChar      buffer[BUFFER_SIZE];
    ViStatus    rssmb_status = VI_SUCCESS;

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rssmb_invalidViReal64Range (instrSession, value, min, max) == VI_TRUE)
	    {
	       rssmb_instrRange   instrPtr;
	       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	       return VI_ERROR_INSTR_PARAMETER;
	    }

    sprintf(buffer,"%s %.12f %s",cmd, value, units);

    return rssmb_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rssmb_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rssmb_checkStatus(instrSession));

ERROR_LABEL:

    free(wrt_buf);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rssmb_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViString parameterName)
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0;
    size_t      length          = 0;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    memcpy (pbuffer, cmd, length);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free (pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssmb_readDataUnknownLength (instrSession, &pbuffer, &count));
    
    if (pbuffer == NULL)
        return rssmb_status;
    
    GOCHECKERR (rssmb_checkStatus (instrSession));

    if (count > buffer_size)
    {
        rssmb_status = RSSMB_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, (size_t) count);
        value[count] = '\0';
    }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
        {
            rssmb_status = VI_SUCCESS;
            memcpy (value, pbuffer, (size_t) (p2buffer - pbuffer));
            value[p2buffer - pbuffer] = '\0';
        }

ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rssmb_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rssmb_checkStatus(instrSession));

ERROR_LABEL:

    free (wrt_buf);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssmb_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViString parameterName)
{
    ViStatus rssmb_status = VI_SUCCESS;

    CHECKERR (rssmb_checkIndexInList (instrSession, value, list, parameterName));

    return rssmb_setViString (instrSession, cmd, list[value], parameterName);
}

/*===========================================================================*/
/* Function: Get Enum                                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rssmb_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";
    
    CHECKERR( rssmb_getViString(instrSession, cmd, buffer, BUFFER_SIZE, parameterName));
    
    CHECKERR( rssmb_getIndexFromString (buffer, list, value));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rssmb_checkIndexInList (ViSession instrSession,
								 ViInt32 index,
                                 ViString list[],
                                 ViString parameterName)

{
    ViInt32     i=0;
    ViStatus    rssmb_status = VI_SUCCESS;

    if (index < 0)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    while (list[i++] != VI_NULL);
    if (index >= i)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rssmb_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = (ViInt32) i;

    return (list[i]) ? rssmb_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Get Catalog                                                     */
/*===========================================================================*/
ViStatus rssmb_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    size_t      count  = 0,
                length = 0;
    ViUInt32    unknown_count = 0;

    if (array == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free(pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rssmb_readDataUnknownLength (instrSession, &pbuffer, &unknown_count));
    GOCHECKERR (rssmb_checkStatus (instrSession));

    if (count > (size_t) arraySize)
        {
        rssmb_status = RSSMB_WARNING_SMALL_BUFFER;
        count = (size_t)arraySize;
        memcpy (array, pbuffer, count);
        array[count-1] = '\0';
        }
    else
        {
        if ((p2buffer = strchr (pbuffer, ';')) != VI_NULL)
            {
            rssmb_status = VI_SUCCESS;
            memcpy (array, pbuffer, (size_t)(p2buffer - pbuffer));
            array[p2buffer - pbuffer] = '\0';
            }
        else
            {
            memcpy (array, pbuffer, count);
            array[count-1] = '\0';
            }
        }
ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rssmb_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssmb_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atof(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssmb_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get ViInt32 Array                                               */
/*===========================================================================*/
ViStatus rssmb_getViInt32Array (ViSession instrSession,
                            ViString cmd,
                            ViInt32 *value,
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViUInt32    local_count = 0;
    ViInt32     i = 0;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    GOCHECKERR (viPrintf(instrSession, "%s\n",cmd));
    GOCHECKERR (rssmb_readDataUnknownLength(instrSession, &pbuffer, &local_count));
    p2buf = pbuffer;

    p2buf = strtok(pbuffer, ",");

    while ((p2buf != NULL) && (i < count))
    {
        value[i++] = atol(p2buf);
        p2buf = strtok(NULL, ",");
    };

    GOCHECKERR (rssmb_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intf                                                 */
/*===========================================================================*/
ViStatus rssmb_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rssmb_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = NULL, *p2buffer = NULL;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc(actSize, (size_t) sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
    {
        if ((rssmb_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
        {
            free(buffer);
            *outBuffer = VI_NULL;
            return rssmb_status;
        }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
    } while (rssmb_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count != NULL) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get ViUInt32                                                    */
/* Purpose:  This function gets command with unsigned integer parameter.     */
/*===========================================================================*/
ViStatus rssmb_getViUInt32 (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 *value,
                           ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%u", value));
    CHECKERR( rssmb_checkStatus (instrSession));

    return rssmb_status;
}

/*****************************************************************************
 * Function: Read Trace Data
 * Purpose:  This function reads out trace data from the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_ReadTraceData(
    ViSession   instrSession,
    ViString    cmd,
    ViInt32*    noofPoints,
    ViReal64    _VI_FAR traceData[]
)
{
    ViUInt32 retCnt = 0;
    ViChar buffer[BUFFER_SIZE];
    ViByte* exBuf;
    ViReal32 *temp;
    ViInt32 lenLength, i = 0;
    ViStatus rssmb_status = VI_SUCCESS;
    ViUInt16 interface = VI_INTF_GPIB;
    rssmb_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    /* Interface specific operations */
    switch (interface)
        {
        case VI_INTF_ASRL: /* Serial Interface. */
            sprintf (buffer, ":FORMAT ASCII;*CLS;%s\n", cmd);

            CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32)strlen (buffer), &retCnt));
            CHECKERR (rssmb_checkBeforeRead (instrSession));
            CHECKERR (viScanf (instrSession, "%,#Lf", noofPoints, traceData));
        break;
        case VI_INTF_GPIB: default:  /* GPIB Interface. Default case */
            sprintf (buffer, ":FORM REAL,32;*CLS;%s\n", cmd);

            if ((rssmb_status = viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer),
                &retCnt)) < 0)
                return rssmb_status;
            if ((rssmb_status = rssmb_checkBeforeRead (instrSession)) < 0)
                return rssmb_status;

            if ((rssmb_status = viRead (instrSession, (ViBuf) buffer, 2, &retCnt)) < 0)
                return rssmb_status;
            if (sscanf (buffer, "#%1d", &lenLength) != 1) {
                rssmb_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rssmb_status;
            }

            if ((rssmb_status = viRead (instrSession, (ViBuf) buffer, lenLength,
                &retCnt)) < 0)
                return rssmb_status;
            if (sscanf (buffer, "%ld", noofPoints) != 1) {
                rssmb_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
                return rssmb_status;
            }

            /* Allocate memory space */
            if ((exBuf = malloc (*noofPoints + 1)) == NULL)
                return VI_ERROR_ALLOC;
            if ((rssmb_status = viRead (instrSession, (ViBuf) exBuf, *noofPoints + 1,
                &retCnt)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                if (exBuf) free (exBuf);
                return rssmb_status;
                }

            /* Convert binary data into array of ViReal64 types. */
            *noofPoints = (ViInt32)(*noofPoints/4);
            /* Binary data byte order (MOTOROLA or INTEL). */
            rssmb_SWAP_32(exBuf, *noofPoints);
            /* Pointer to data */
            temp = (ViReal32*)(exBuf);
            for (i = 0; i < *noofPoints; i++)
                *traceData++ = (ViReal64)*temp++;

            if (exBuf) /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
        break;
        }

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < 0)
        return rssmb_status;

    return rssmb_status;
}
/*===========================================================================*/
/* Function: Swap 32bit Integer                                              */
/* Purpose:  Byte order for binary transfers depends on MOTOROLA or INTEL    */
/*           formats.                                                        */
/*===========================================================================*/
void rssmb_SWAP_32(unsigned char *data, long points)
{
    /* MOTOROLA (byte order 0123) or INTEL (byte order 3210). */
    unsigned int    word = 0x1122;
    unsigned char   *p2word = (unsigned char*)&word;

    if (*p2word == 0x11) /* MOTOROLA format */
    {
       unsigned char    src[4];
       long             *p2data = (long*)data;
       int              idx;

       for (idx = 0; idx < points; idx++, p2data++)
          {
              *((long *)src) = *(p2data);
              ((unsigned char *)(p2data))[0] =  ((unsigned char*)(src))[3];
              ((unsigned char *)(p2data))[1] =  ((unsigned char*)(src))[2];
              ((unsigned char *)(p2data))[2] =  ((unsigned char*)(src))[1];
              ((unsigned char *)(p2data))[3] =  ((unsigned char*)(src))[0];
          }
    }
}

/*===========================================================================*/
/* Function: Check STB Before Read                                           */
/* Purpose:  This is a utility function which checks Error/Event Queue bit of*/
/*           status byte. Call this function before read result of query     */
/*           command. When error occurs, this function returns error code    */
/*           RSZVB_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE. Prior to calling    */
/*           this function is necessary to clear STB.                        */
/*===========================================================================*/
ViStatus rssmb_checkBeforeRead (ViSession instrSession)
{
    ViStatus rssmb_status = VI_SUCCESS;
    ViUInt16 response = 0;
    ViInt32 counter = 0, timeout = 0;
    ViUInt16 interface = VI_INTF_GPIB;
    rssmb_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));

    if (instrPtr -> errorChecking) /* Is Error Checking Enabled? */
        {
        /* Interface specific operations */
        switch (interface)
            {
            case VI_INTF_ASRL: /* Serial interface */

                /* Not supported */

            break;
            default: /* Default case */

                CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &timeout));

#if 0 /* Removed from the build */

                /* Result of reading STB must be zero. Initial settings */
                do {
                    CHECKERR (rssmb_status = viReadSTB (instrSession, &response));

                    rssmb_delay (0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait until STB is non zero value. */
                } while (!response);
#endif
                /* Check STB for errors. */
                do {
                    CHECKERR (rssmb_status = viReadSTB (instrSession, &response));

                    rssmb_delay (instrSession, 0.005);
                    counter += 5;
                    if (counter >= timeout)
                        return VI_ERROR_TMO;
                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
                } while (!(response & 0x14));

                if (response & 0x4)
                {
                    ViInt32 error;
                    CHECKERR(viQueryf(instrSession, "*ESR?\n", "%ld", &error));
                    if (error == 4)
                        CHECKERR(viQueryf(instrSession, ":SYST:ERR?\n", "%*[^\r\n]"));
                }
                /* (check Error/Event Queue bit in STB) */
                return (response & 0x4) ?  RSSMB_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE : rssmb_status;
            }
        }

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Read Data To File                                               */
/* Purpose:  This function read data from the instrument and                 */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus rssmb_readDataToFile (ViSession instrSession, ViString cmd,
                           ViString destination)
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "%s\n", cmd);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssmb_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssmb_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssmb_status;
            }

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssmb_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < 0)
        return rssmb_status;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Check Instrument                                                */
/* Purpose:  This function checks presence of selected instrument. According */
/*           to generateErrorIfPresent value it either generates error or    */
/*           returns success if instrumet is present.                        */
/*===========================================================================*/
ViStatus rssmb_checkInstrument (ViSession instrSession, ViInt32 instrument, ViBoolean generateErrorIfPresent)
{
    ViStatus            rssmb_status = VI_SUCCESS;
    rssmb_instrRange    instrPtr;
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if ((instrPtr->errorChecking != 0) && (instrPtr->optionChecking != 0))
    {
        if (generateErrorIfPresent != VI_FALSE)
        {
            if (instrPtr->instrVer == instrument)
                rssmb_status = RSSMB_ERROR_INSTRUMENT_NSUP_MODEL;
        }
        else
        {
            if (instrPtr->instrVer != instrument)
                rssmb_status = RSSMB_ERROR_INSTRUMENT_NSUP_MODEL;
        }
    }

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 Array                                              */
/*===========================================================================*/
ViStatus rssmb_setViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value[],
                            ViInt32  count,
                            ViString parameterName)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar*     pbuffer = VI_NULL;
    ViChar*     p2buf;
    ViInt32     i = 0;

    if (value == VI_NULL)
    {
       rssmb_instrRange   instrPtr;
       CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
       strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
       return VI_ERROR_INSTR_PARAMETER;
    }

    pbuffer = (ViChar*) malloc (count * 15 + strlen (cmd) + 200);
	
	p2buf = pbuffer + sprintf (pbuffer, "%s ", cmd);
    
	while ((p2buf != NULL) && (i < count))
        p2buf += sprintf (p2buf, "%.12lG,", value[i++]);
	
	*--p2buf = '\n';
	
	GOCHECKERR (viWrite (instrSession, (ViBuf) pbuffer, (ViUInt32) strlen (pbuffer), NULL));

	GOCHECKERR (rssmb_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rssmb_status;
}
ViStatus rssmb_ParameterError (ViSession io, ViString parameterName)
{
	ViStatus    rssmb_status = VI_SUCCESS;
	rssmb_instrRange   instrPtr;
	CHECKERR (viGetAttribute (io, VI_ATTR_USER_DATA, &instrPtr));
	strncpy (instrPtr->parameterErrorMessage, parameterName, 255);
	return VI_ERROR_INSTR_PARAMETER;
}
