/*= R&S SMB Signal Generator ================================================
 * LabWindows/CVI 8.0 VXIpnp Instrument Driver
 * Original Release: July 2007
 * By: Petr Kominek, Jiri Kominek
 *
 * Should you have any technical questions please contact the hotline of
 * Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 * e-mail: CustomerSupport@rohde-schwarz.com
 *
 *   see file rssmb_vxi.chm
 *
*===========================================================================*/
#ifdef _WIN32
    #ifdef _CVI_
        #include <userint.h>
    #else
        #include <windows.h>
    #endif
#endif
#include <math.h>
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "rssmb.h"
#include "rssmb_utility.h"
#include <limits.h>
#include <locale.h>  

/*= Macros ==================================================================*/

#define CHECKERR_CU(fCal) if (rssmb_status = (fCal), rssmb_status < VI_SUCCESS) \
    return rssmb_initCleanUp (rmSession, instrSession, rssmb_status); else
#define CHECKVIINT32(io, arg, min, max) if ( rssmb_invalidViInt32Range (io, arg,min,max)) return rssmb_ParameterError(io,#arg)
#define CHECKVIREAL64(io, arg, min, max) if ( rssmb_invalidViReal64Range (io, arg,min,max)) return rssmb_ParameterError(io,#arg)

#define SETPARAMENUM(io,cmd,value,list) rssmb_setEnum (io,cmd,value,list,#value)
#define GETPARAMENUM(io,cmd,value,list) rssmb_getEnum (io,cmd,value,list,#value)

#define SETPARAMBOOL(io,cmd,value) rssmb_setViBoolean (io,cmd,value,#value)
#define GETPARAMBOOL(io,cmd,value) rssmb_getViBoolean (io,cmd,value,#value)

#define SETPARAMINT32(io,cmd,value,min,max) rssmb_setViInt32 (io,cmd,value,min,max,#value)
#define GETPARAMINT32(io,cmd,value) rssmb_getViInt32 (io,cmd,value,#value)
#define GETPARAMINT32ARRAY(io,cmd,value,count) rssmb_getViInt32Array (io,cmd,value,count,#value)
#define GETPARAMUINT32(io,cmd,value) rssmb_getViUInt32 (io,cmd,value,#value)

#define SETPARAMREAL64(io,cmd,value,min,max) rssmb_setViReal64 (io,cmd,value,min,max,#value)
#define GETPARAMREAL64(io,cmd,value) rssmb_getViReal64 (io,cmd,value,#value)
#define SETPARAMREAL64ARRAY(io,cmd,value,count) rssmb_setViReal64Array (io,cmd,value,count,#value)
#define GETPARAMREAL64ARRAY(io,cmd,value,count) rssmb_getViReal64Array (io,cmd,value,count,#value)
#define SETPARAMREAL64UNITS(io,cmd,value,units,min,max) rssmb_setViReal64Units (io,cmd,value,units,min,max,#value)

#define SETPARAMSTRINGQUOTED(io,cmd,quotes,value) rssmb_setViStringQuoted (io,cmd,quotes,value,#value)
#define SETPARAMSTRING(io,cmd,value) rssmb_setViString (io,cmd,value,#value)
#define GETPARAMSTRING(io,cmd,value,buf_size) rssmb_getViString (io,cmd,value,buf_size,#value)

/*= DEFINES =================================================================*/
#define RSSMB_REVISION          "Rev 2.21.0, 07/2016, CVI 2015" /* Instrument driver revision */
#define BUFFER_SIZE             2048L    /* File I/O buffer size */

#define RSSMB_DEFAULT_TIMEOUT   10000   /* Default Timeout value in ms (10 sec) */

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString triggerRFSourceArr[] = {"AUTO","SING","EXT","EAUT", VI_NULL};
static ViString onRecallArr[] = {"INCL","EXCL", VI_NULL};
static ViString externReferenceFreqArr[] = {"10MHZ", VI_NULL};
static ViString attenuatorModeArr[] = {"AUTO","FIX", VI_NULL};
static ViString powerOnStateArr[] = {"OFF","UNCH", VI_NULL};
static ViString alcStateArr[] = {"OFF","ON","AUTO", VI_NULL};
static ViString shapeArr[] = {"SINE","SQU", VI_NULL};
static ViString amExtCouplingArr[] = {"AC","DC", VI_NULL};
static ViString polarityArr[] = {"NORM","INV", VI_NULL};
static ViString pulmSourceArr[] = {"INT","EXT", VI_NULL};
static ViString pulmTriggerModeArr[] = {"AUTO","EXT","EGAT", VI_NULL};
static ViString sourceArr[] = {"INT","EXT","INT,EXT", VI_NULL};
static ViString fmModeArr[] = {"NORM","LNO","HDEV", VI_NULL};
static ViString pmModeArr[] = {"NORM","LNO","HDEV", VI_NULL};
static ViString fmExtCouplingArr[] = {"AC","DC", VI_NULL};
static ViString slopeArr[] = {"POS","NEG", VI_NULL};
static ViString externalInputImpedanceArr[] = {"G50","G10K","G600","HIGH",VI_NULL};
static ViString testCommandsArr[] = {"SSYN","SSYNEX","IQOP3","IQOP6","SATT3",
                                     "SATT6","BBINS","DIRECTLAST", VI_NULL};
static ViString spacingArr[] = {"LIN","LOG", VI_NULL};
static ViString regGroupArr[] = {"OPER","QUES", VI_NULL};
static ViString regTypeArr[] = {"COND","EVEN","ENAB","NTR","PTR", VI_NULL};
static ViString lfSweepModeArr[] = {"AUTO","MAN","STEP", VI_NULL};
static ViString frequencyModeArr[] = {"CW","SWE","LIST", VI_NULL};
static ViString lfOutputModeArr[] = {"CW","SWE", VI_NULL};
static ViString variationArr[] = {"DEC","USER",VI_NULL};
static ViString listModeArr[] = {"AUTO","STEP", VI_NULL};
static ViString listTriggerSourceArr[] = {"AUTO","SING","EXT","HOP","DHOP", VI_NULL};
static ViString powerUnitArr[] = {"DBM","DBUV","WATT", VI_NULL};
static ViString powerSourceArr[] = {"A","USER", VI_NULL};
static ViString powerFilterModeArr[] = {"AUTO","USER",VI_NULL};
static ViString dexcListModeArr[] = {"IMP","EXP", VI_NULL};
static ViString dexcExtensionArr[] = {"TXT","CSV", VI_NULL};
static ViString dexcSeparatorArr[] = {"TAB","SEM","COMM","SPAC", VI_NULL};
static ViString dexcDecimalSeparatorArr[] = {"DOT","COMM", VI_NULL};
static ViString keyboardArr[] = {"US","DE", VI_NULL};
static ViString ipAddressModeArr[] = {"STAT","STAT","AUTO", VI_NULL};
static ViString sweepShapeArr[] = {"SAWT","TRI", VI_NULL};
static ViString powerSenseFilterModeArr[] = {"AUTO","USER","NSR", VI_NULL};
static ViString ariBKCodeArr[] = {"A","B","C","D","E","F", VI_NULL};
static ViString ariTypeArr[] = {"OFF","DK","BK","BKDK", VI_NULL};
static ViString audioModeArr[] = {"LEFT","RIGH","REL","REML","RNEL", VI_NULL};
static ViString audioPreemphasisArr[] = {"0.00005","0.000075", VI_NULL};
static ViString rdsDataSetArr[] = {"","DS1","DS2","DS3","DS4","DS5", VI_NULL};
static ViString stereoInputImpedanceArr[] = {"100000","600", VI_NULL};
static ViString stereoSourceArr[] = {"OFF","LREX","SPEX","LFG", VI_NULL};
static ViString angleUnitsArr[] = {"DEG","RAD",VI_NULL};
static ViString powerUnitsArr[] = {"V","DBM", VI_NULL};
static ViString velocityUnitsArr[] = {"MPS","KMH","MPH", VI_NULL};
static ViString imageFormatArr[] = {"BMP","JPG","XPM","PNG", VI_NULL};
static ViString instrVersionList[] = {"SMB","SMC","UNKNOWN", VI_NULL};
static ViString lfGeneratorSourceArr[] = {"","LF1","LF2", VI_NULL};
static ViString calLevelAdjDataArr[] = {"FACT","CUST", VI_NULL};
static ViString hcopImageSizeArr[] = {"320,240","640,480","800,600","1024,768", VI_NULL};
static ViString hcopDeviceArr[] = {"FILE", VI_NULL};
static ViString dmemExternalInputImpedance[] = {"G50","G10K", VI_NULL};
static ViString rfModeArr[] = {"NORM","LOWN","LOWD", VI_NULL};
static ViString attenuatorModeArr2[] = {"UNCH","FATT", VI_NULL};
static ViString synchronisationBandwidthArr[] = {"NARR","WIDE", VI_NULL};
static ViString searchModeArr[] = {"NORM","MIN", VI_NULL};
static ViString priorityArr[] = {"AVER","PEAK", VI_NULL};

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus rssmb_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus rssmb_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssmb_SetSweepMode (ViSession instrSession, ViInt32 mode);
ViStatus rssmb_GetSweepMode (ViSession instrSession, ViPInt32 sweepMode);
ViStatus rssmb_SetFrequencySweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceRFFreqSweep);
ViStatus rssmb_GetFrequencySweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceRFFreqSweep);
ViStatus rssmb_SetFrequencyMode (ViSession instrSession, ViInt32 frequencyMode);
ViStatus rssmb_GetFrequencyMode (ViSession instrSession, ViPInt32 frequencyMode);
ViStatus rssmb_SetSweepLevelMode (ViSession instrSession, ViInt32 sweepLevelMode);
ViStatus rssmb_GetSweepLevelMode (ViSession instrSession, ViPInt32 sweepLevelMode);
ViStatus rssmb_SetLevelSweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceRFLevelSweep);
ViStatus rssmb_GetLevelSweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceRFLevelSweep);
ViStatus rssmb_SetLevelMode (ViSession instrSession, ViInt32 RFLevelMode);
ViStatus rssmb_GetLevelMode (ViSession instrSession, ViPInt32 RFLevelMode);
ViStatus rssmb_SetSweepLFMode (ViSession instrSession, ViInt32 LFSweepMode);
ViStatus rssmb_GetSweepLFMode (ViSession instrSession, ViPInt32 LFSweepMode);
ViStatus rssmb_SetLFSweepTriggerSource (ViSession instrSession, ViInt32 triggerSourceLFSweep);
ViStatus rssmb_GetLFSweepTriggerSource (ViSession instrSession, ViPInt32 triggerSourceLFSweep);
ViStatus rssmb_SetLFMode (ViSession instrSession, ViInt32 LFOutputMode);
ViStatus rssmb_GetLFMode (ViSession instrSession, ViPInt32 LFOutputMode);
ViStatus rssmb_SetModeLIST (ViSession instrSession, ViInt32 listMode);
ViStatus rssmb_SetLISTTriggerSource (ViSession instrSession,ViInt32 triggerSource);

/*****************************************************************************
 * Function: Initialize
 * Purpose:  This function performs the following initialization actions:
 *           
 *           - Opens a session to the Default Resource Manager resource and a 
 *           session to the specified device using the interface and address 
 *           specified in the Resource_Name control.
 *           
 *           - Performs an identification query on the Instrument.
 *           
 *           - Resets the instrument to a known state.
 *           
 *           - Sends initialization commands to the instrument that set any 
 *           necessary programmatic variables such as Headers Off, Short 
 *           Command form, and Data Transfer Binary to the state necessary for 
 *           the operation of the instrument driver.
 *           
 *           - Returns an Instrument Handle which is used to differentiate 
 *           between different sessions of this instrument driver.
 *           
 *           - Each time this function is invoked a Unique Session is opened.  
 *           It is possible to have more than one session open for the same 
 *           resource.
 *           
 *****************************************************************************/

ViStatus _VI_FUNC rssmb_init(
    ViRsrc  resourceName,
    ViBoolean   idQuery,
    ViBoolean   resetDevice,
    ViSession*  instrSession
)
{
    ViStatus            rssmb_status = VI_SUCCESS;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE] = "";
    rssmb_instrRange    instrPtr = NULL;

    /* --- Set standard "C" locale --- */
    {
       char *locale;

       locale = setlocale(LC_ALL, "C");
    }
    
    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rssmb_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
    {
        (void)viClose (rmSession);
        return rssmb_status;
    }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */

    if (instrPtr == NULL)
        instrPtr = calloc (1, sizeof (struct rssmb_statusDataRanges));
        

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    //With remote control via Ethernet, remote control is not automatically
    //set by means of a command. The instrument must be explicitly set to
    //the REMOTE state, e.g. by sending the intf command &GTR
    CHECKERR (viPrintf (*instrSession, "&GTR\n"));

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE,
                                 RSSMB_DEFAULT_TIMEOUT));
    CHECKERR( viSetBuf (*instrSession, (ViUInt16) (VI_READ_BUF|VI_WRITE_BUF), 4000));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));

    /* -------------------------------------------------------------------- */
    /* Drawback of Agilent VISA is patched by adding extra termination      */
    /* character "\n" to each I/O operation                                 */
    /*                                                                      */
    /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE     */
    /* -------------------------------------------------------------------- */

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));

    // read/reset ESR which may still indicate device dependent error from 
    // previous handling!
    (void) rssmb_checkStatus(*instrSession);
    
    /*- Identification Query ------------------------------------------------*/
    if (idQuery == VI_TRUE)
    {
        int i = 0;
        CHECKERR( viQueryf (*instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]",
                               buffer));

        /* Check if instrument is supported or not */
        while (instrVersionList[i] != NULL)
        {
            if (strncmp (buffer, instrVersionList[i], strlen (instrVersionList[i])) == 0)
                break;
            else i++;
        }
        if (instrVersionList[i] == NULL)
            return rssmb_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
        else
            instrPtr->instrVer = i;
    }

    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
        CHECKERR( rssmb_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR( rssmb_defaultInstrSetup (*instrSession));

    return rssmb_status;
}


/*============================================================================
 * Function: rssmb_TriggerSourceAllSweeps
 * Purpose:  This function sets the trigger source for all sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerSourceAllSweeps (ViSession instrSession,
                                                ViInt32 triggerAllSweeps)
{
    return SETPARAMENUM (instrSession,
                          "TRIG:SWE:SOUR",
                          triggerAllSweeps,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: Set All RF Outputs State
 * Purpose:  This function switches on or off all RF output signals of the
 *           instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAllRFOutputsState (ViSession instrSession,
                                              ViBoolean RFOutputStates)
{
    return SETPARAMBOOL (instrSession,
                               "OUTP:ALL:STAT",
                               RFOutputStates);
}

/*============================================================================
 * Function: rssmb_SetOutputState
 * Purpose:  This function activates and deactivates the RF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetOutputState(
    ViSession instrSession,
    ViBoolean outputState
)
{
    return SETPARAMBOOL (instrSession,
                               "OUTP:STAT",
                               outputState);
}

/*============================================================================
 * Function: rssmb_GetOutputState
 * Purpose:  This function returns state of the RF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetOutputState (ViSession  instrSession,
                                        ViBoolean* outputState)
{
    return GETPARAMBOOL (instrSession,
                               "OUTP:STAT?",
                               outputState);
}

/*============================================================================
 * Function: rssmb_SetRFFrequencyonRecall
 * Purpose:  This function determines whether the current frequency setting
 *           is retained or whether the stored frequency setting is adopted
 *           when an instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetRFFrequencyonRecall (ViSession instrSession,
                                                ViInt32   frequencyOnRecall)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:FREQ:RCL",
                          frequencyOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssmb_GetRFFrequencyonRecall
 * Purpose:  This function returns whether the current frequency setting is
 *           retained or whether the stored frequency setting is adopted when 
 *           an instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetRFFrequencyonRecall (ViSession instrSession,
                                                ViInt32*  frequencyOnRecall)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:FREQ:RCL?",
                          frequencyOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssmb_SetRFLevelonRecall
 * Purpose:  This function determines whether the current level is retained
 *           or whether the stored level setting is adopted when an
 *           instrument configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetRFLevelonRecall (ViSession instrSession,
                                            ViInt32   levelOnRecall)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:RCL",
                          levelOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssmb_GetRFLevelonRecall
 * Purpose:  This function returns whether the current level is retained or
 *           whether the stored level setting is adopted when an instrument
 *           configuration is loaded.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetRFLevelonRecall (ViSession instrSession,
                                            ViInt32*  levelOnRecall)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:RCL?",
                          levelOnRecall,
                          onRecallArr);
}

/*============================================================================
 * Function: rssmb_OvervoltageProtectionClear
 * Purpose:  This function resets the protective circuit after it has been
 *           tripped. The state of the output is again determined by
 *           OUTPut:STATe.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_OvervoltageProtectionClear (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, ":OUTP:PROT:CLE");
}

/*============================================================================
 * Function: rssmb_OvervoltageProtectionTripped
 * Purpose:  This function queries the state of the protective circuit.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_OvervoltageProtectionTripped (ViSession instrSession,
                                                      ViInt32*  stateProtective)
{
    return GETPARAMINT32 (instrSession,
                             "OUTP:PROT:TRIP?",
                             stateProtective);
}

/*============================================================================
 * Function: Get Output Impedance
 * Purpose:  This function queries the impedance of the RF output. This 
 *           permits converting the output level between units V and W. 
 *           The impedances cannot be changed.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetOutputImpedance (ViSession instrSession,
                                            ViInt32 *impedance)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:IMP?",
                          impedance,
                          dmemExternalInputImpedance);
}

/*****************************************************************************
 * Function: Set RF Mode
 * Purpose:  This function sets the RF level mode.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:LMODe NORMal | LOWNoise | LOWDistortion
 *           
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetRFMode(
	ViSession	instrSession,
	ViInt32	rfMode
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:LMOD",
                          rfMode,
                          rfModeArr);
}

/*****************************************************************************
 * Function: Get RF Mode
 * Purpose:  This function returns the RF level mode.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:LMODe?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetRFMode(
	ViSession	instrSession,
	ViInt32*	rfMode
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:LMOD?",
                          rfMode,
                          rfModeArr);
}

/*****************************************************************************
 * Function: Set RF Attenuator Mode
 * Purpose:  This function sets the attenuator mode, when the RF signal is 
 *           switched off.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:ATTenuation:RFOFf:MODE UNCHanged | FATTenuated
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetRFAttMode(
	ViSession	instrSession,
	ViInt32	mode
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:ATT:RFOF:MODE",
                          mode,
                          attenuatorModeArr2);
}

/*****************************************************************************
 * Function: Get RF Attenuator Mode
 * Purpose:  This function returns the attenuator mode, when the RF signal is 
 *           switched off.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:ATTenuation:RFOFf:MODE?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetRFAttMode(
	ViSession	instrSession,
	ViInt32*	mode
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:ATT:RFOF:MODE?",
                          mode,
                          attenuatorModeArr2);
}

/*****************************************************************************
 * Function: Set RF Level
 * Purpose:  This function sets the RF level of the RF output connector.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:POWer 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetRFLevel(
	ViSession	instrSession,
	ViReal64	rfLevel
)
{
    return SETPARAMREAL64 (instrSession,
                          	  "SOUR:POW:POW",
                          	  rfLevel,
                          	  0.0, 0.0);
}

/*****************************************************************************
 * Function: Get RF Level
 * Purpose:  This function returns the RF level of the RF output connector.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:POWer:POWer?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetRFLevel(
	ViSession	instrSession,
	ViReal64*	rfLevel
)
{
    return GETPARAMREAL64 (instrSession,
                          	  "SOUR:POW:POW?",
                          	  rfLevel);
}

/*============================================================================
 * Function: rssmb_confRFFreq
 * Purpose:  This function selects Fixed frequency mode and sets the
 *           frequency and frequency offset of the RF output signal for CW
 *           mode (:SOURce:FREQuency:MODE CW).
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confRFFreq (ViSession instrSession, ViReal64 frequency,
                                    ViReal64 offset, ViBoolean variationActive,
                                    ViReal64 variationStep)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetRFFrequency (instrSession, frequency));
    
    CHECKERR (rssmb_SetRFFrequencyOffset (instrSession, offset));
    CHECKERR (rssmb_SetRFVariationActive (instrSession, variationActive));
    if (variationActive == VI_TRUE)
    {
        CHECKERR (rssmb_SetRFVariationStep (instrSession, variationStep));
    }

    RETURN;
}


/*============================================================================
 * Function: rssmb_SetRFFrequency
 * Purpose:  This function sets the frequency of the RF output signal for CW
 *           mode (:SOURce:FREQuency:MODE CW). In Sweep mode
 *           (:SOURce:FREQuency:MODE SWEep), this value is linked to the
 *           current sweep frequency.
 *
 *           In addition to a numerical value, it is also possible to specify
 *           UP and DOWN. The frequency is then increased or decreased by the
 *           value which is set under [SOURce<1|2>:]FREQuency:STEP.
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFFrequency (ViSession instrSession,
                                        ViReal64  frequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ",
                              frequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetRFFrequency
 * Purpose:  This function returns the frequency of the RF output signal for
             CW mode (:SOURce:FREQuency:MODE CW). In Sweep mode
 *           (:SOURce:FREQuency:MODE SWEep), this value is linked to the
 *           current sweep frequency.
 *
 *           In addition to a numerical value, it is also possible to specify
 *           UP and DOWN. The frequency is then increased or decreased by the
 *           value which is set under [SOURce<1|2>:]FREQuency:STEP.
 *
 *           As with the FREQ value entered in the display, the OFFSet value
 *           is also taken into consideration with this function. The
 *           specified value range is therefore only effective if OFFSet is
 *           set to 0. The value range for other OFFset values can be
 *           calculated using the following formula:
 *
 *           300 kHz + OFFSet ... RFmax + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFFrequency (ViSession instrSession,
                                        ViReal64* frequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ?",
                              frequency);
}

/*============================================================================
 * Function: rssmb_SetRFFrequencyOffset
 * Purpose:  This function sets the frequency offset of a downstream
 *           instrument, e.g. a mixer (see section RF Frequency and Phase -
 *           Frequency - Phase). If a frequency offset is entered, the
 *           frequency entered with SOURce:FREQuency:... no longer
 *           corresponds to the RF output frequency. The following
 *           correlation applies:
 *
 *           SOURce:FREQuency:... = RF output frequency  +
 *           SOURce:FREQuency:OFFSet.
 *
 *           Entering an offset does not change the RF output frequency, but
 *           rather the query value of SOURce:FREQuency:...
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFFrequencyOffset (ViSession instrSession,
                                              ViReal64  offset)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:OFFS",
                              offset,
                              -50e9,50e9);
}

/*============================================================================
 * Function: rssmb_GetRFFrequencyOffset
 * Purpose:  This function returns the frequency offset of a downstream
 *           instrument, e.g. a mixer (see section RF Frequency and Phase -
 *           Frequency - Phase). If a frequency offset is entered, the
 *           frequency entered with SOURce:FREQuency:... no longer
 *           corresponds to the RF output frequency. The following
 *           correlation applies:
 *
 *           SOURce:FREQuency:... = RF output frequency  +
 *           SOURce:FREQuency:OFFSet.
 *
 *           Entering an offset does not change the RF output frequency, but
 *           rather the query value of SOURce:FREQuency:...
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFFrequencyOffset (ViSession instrSession,
                                              ViReal64* offset)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:OFFS?",
                              offset);
}

/*============================================================================
 * Function: rssmb_SetRFVariationActive
 * Purpose:  This function activates (USER) or deactivates (DECimal) the
 *           user-defined step width used when varying the frequency value
 *           with the frequency values UP/DOWN. This function is linked to
 *           the function Variation Active for manual control, i.e. the
 *           function also activates/deactives the user-defined step width
 *           used when varying the frequency value with the rotary knob.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFVariationActive (ViSession instrSession,
                                              ViBoolean variationActive)
{
    return SETPARAMENUM (instrSession,
                           "SOUR:FREQ:STEP:MODE",
                           (ViInt32) variationActive,
                           variationArr);
}

/*============================================================================
 * Function: rssmb_GetRFVariationActive
 * Purpose:  This function queries the frequency step mode. This function is
 *           linked to the function Variation Active for manual control, i.e.
 *           the function also activates/deactives the user-defined step
 *           width used when varying the frequency value with the rotary
 *           knob.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFVariationActive (ViSession  instrSession,
                                              ViBoolean* variationActive)
{
    ViStatus    rssmb_status;
    ViInt32     variation = 0;
    
    CHECKERR (GETPARAMENUM (instrSession,
                             "SOUR:FREQ:STEP:MODE?",
                             &variation,
                             variationArr));
    if (variationActive != NULL)
        *variationActive = (ViBoolean)((variation == 1)? 1 : 0);

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_SetRFVariationStep
 * Purpose:  This function sets the step width for the frequency setting if
 *           the frequency values UP/DOWN are used and variation mode
 *           :SOUR:FREQ:STEP:MODE USER is selected. This function is linked
 *           to  Variation Step for manual control, i.e. the command also
 *           sets the step width of the rotary knob for Variation Active on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFVariationStep (ViSession instrSession,
                                            ViReal64  variationStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STEP",
                              variationStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetRFVariationStep
 * Purpose:  This function returns the step width of the frequency setting if
 *           the frequency values UP/DOWN are used and variation mode
 *           :SOUR:FREQ:STEP:MODE USER is selected. This function is linked
 *           to  Variation Step for manual control, i.e. the command also
 *           sets the step width of the rotary knob for Variation Active on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFVariationStep (ViSession instrSession,
                                            ViReal64* variationStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STEP?",
                              variationStep);
}

/*============================================================================
 * Function: rssmb_SetRFDeltaPhase
 * Purpose:  This function specifies the phase variation relative to the
 *           current phase. The variation can be specified in RADians.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFDeltaPhase (ViSession instrSession,
                                         ViReal64  deltaPhase)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:PHAS",
                              deltaPhase,
                              -359.9,359.9);
}

/*============================================================================
 * Function: rssmb_GetRFDeltaPhase
 * Purpose:  This function returns the phase variation relative to the
 *           current phase. The variation can be specified in RADians.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFDeltaPhase (ViSession instrSession,
                                         ViReal64* deltaPhase)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:PHAS?",
                              deltaPhase);
}

/*============================================================================
 * Function: rssmb_ResetRFDeltaPhaseDisplay
 * Purpose:  This function adopts the phase set with SOURce:PHASe:ADJust as
 *           the current phase.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_ResetRFDeltaPhaseDisplay (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "SOUR:PHAS:REF");
}

/*============================================================================
 * Function: rssmb_confRosc
 * Purpose:  This function configures reference oscillator for external or
 *           internal reference.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_confRosc (ViSession instrSession, ViInt32 ROSCSource,
                                  ViInt32 externalReferenceFreq,
                                  ViBoolean adjustmentActive,
                                  ViInt32 adjustmentFrequency)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetROSCSource (instrSession, ROSCSource));
    CHECKERR (rssmb_SetROSCExternalReferenceFrequency (instrSession, externalReferenceFreq));
    CHECKERR (rssmb_status = rssmb_SetROSCAdjustmentActive (instrSession, adjustmentActive));    
    CHECKERR (rssmb_status = rssmb_SetROSCAdjustmentFrequency (instrSession, adjustmentFrequency));
    RETURN;
}

/*============================================================================
 * Function: rssmb_SetROSCSource
 * Purpose:  This function selects between internal and external reference
 *           frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetROSCSource (ViSession instrSession,
                                       ViInt32   roscSource)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:SOUR",
                          roscSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_GetROSCSource
 * Purpose:  This function returns reference frequency source.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetROSCSource (ViSession instrSession,
                                       ViInt32*  roscSource)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:SOUR?",
                          roscSource,
                          sourceArr);
}

/*============================================================================
 * Function: Set ROSC RF Output
 * Purpose:  This function determines if the RF output is switched off in
 *           case of a missing external reference signal for selection
 *           external source. If enabled, this setting ensures that no
 *           improper RF signal due to the missing external reference signal
 *           is output and used for measurements. In addition to the error
 *           message "Ext Ref missing", message "RF output deactivated' is
 *           generated
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetROSCRFOutput (ViSession instrSession,
                                         ViBoolean RFOutput)
{
	return SETPARAMBOOL (instrSession,
		                       "SOUR:ROSC:EXT:RFOF",
							   RFOutput);
}

/*============================================================================
 * Function: Get ROSC RF Output
 * Purpose:  This function returns whether the RF output is switched off in
 *           case of a missing external reference signal for selection
 *           external source.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetROSCRFOutput (ViSession instrSession,
                                         ViBoolean *RFOutput)
{
	return GETPARAMBOOL (instrSession,
		                       "SOUR:ROSC:EXT:RFOF?",
							   RFOutput);
}

/*============================================================================
 * Function: rssmb_SetROSCExternalReferenceFrequency
 * Purpose:  This function informs the instrument of the frequency of the
 *           external reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetROSCExternalReferenceFrequency (ViSession instrSession,
                                                           ViInt32   externReferenceFreq)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ",
                          externReferenceFreq,
                          externReferenceFreqArr);
}

/*============================================================================
 * Function: rssmb_GetROSCExternalReferenceFrequency
 * Purpose:  This function returns frequency of the external reference.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetROSCExternalReferenceFrequency (ViSession instrSession,
                                                           ViInt32*  externReferenceFreq)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:FREQ?",
                          externReferenceFreq,
                          externReferenceFreqArr);
}

/*****************************************************************************
 * Function: Set ROSC External Synchronisation Bandwidth
 * Purpose:  This function sets the synchronization bandwidth for an external 
 *           reference signal.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:ROSCillator:EXTernal:SBANdwidth NARRow | WIDE
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetROSCExternalSynchronisationBandwidth(
	ViSession	instrSession,
	ViInt32	synchronisationBandwidth
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:SBAN",
                          synchronisationBandwidth,
                          synchronisationBandwidthArr);
}

/*****************************************************************************
 * Function: Get ROSC External Synchronisation Bandwidth
 * Purpose:  This function returns the synchronization bandwidth for an 
 *           external reference signal.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:ROSCillator:EXTernal:SBANdwidth?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetROSCExternalSynchronisationBandwidth(
	ViSession	instrSession,
	ViInt32*	synchonisationBandwidth
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:ROSC:EXT:SBAN?",
                          synchonisationBandwidth,
                          synchronisationBandwidthArr);
}


/*============================================================================
 * Function: rssmb_SetROSCAdjustmentActive
 * Purpose:  This function determines whether the calibrated (OFF) or a
 *           user-defined (ON) adjustment value is used for fine adjustment
 *           of the frequency. With STATe ON, the instrument is no longer in
 *           the calibrated state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetROSCAdjustmentActive (ViSession instrSession,
                                                 ViBoolean adjustmentActive)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:ROSC:ADJ",
                               adjustmentActive);
}

/*============================================================================
 * Function: rssmb_GetROSCAdjustmentActive
 * Purpose:  This function determines whether the calibrated (OFF) or a
 *           user-defined (ON) adjustment value is used for fine adjustment
 *           of the frequency. With STATe ON, the instrument is no longer in
 *           the calibrated state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetROSCAdjustmentActive (ViSession  instrSession,
                                                 ViBoolean* adjustmentActive)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:ROSC:ADJ?",
                               adjustmentActive);
}

/*============================================================================
 * Function: rssmb_SetROSCAdjustmentFrequency
 * Purpose:  This function specifies the frequency correction value
 *           (adjustment value). A detailed definition is given in  section
 *           RF Reference Frequency - Reference Oscillator.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetROSCAdjustmentFrequency (ViSession instrSession,
                                                    ViInt32   adjustmentFreq)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:ROSC:ADJ:VAL",
                             adjustmentFreq,
                             0,4095);
}

/*============================================================================
 * Function: rssmb_GetROSCAdjustmentFrequency
 * Purpose:  This function returns the frequency correction value (adjustment
 *           value). A detailed definition is given in  section  RF Reference
 *           Frequency - Reference Oscillator.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetROSCAdjustmentFrequency (ViSession  instrSession,
                                                    ViInt32*   adjustmentFreq)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:ROSC:ADJ:VAL?",
                             adjustmentFreq);
}

/*============================================================================
 * Function: rssmb_confRFLevel
 * Purpose:  This function sets all parameters of fixed RF level and changes
 *           the mode to fixed RF level.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confRFLevel (ViSession instrSession, ViReal64 amplitude,
                                     ViReal64 offset, ViReal64 limit,
                                     ViBoolean variationActive,
                                     ViReal64 variationStep, ViInt32 attenuatorMode,
                                     ViInt32 RFLevel_onRecall)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetRFAmplitude (instrSession, amplitude));

    CHECKERR (rssmb_SetRFLevelLimit (instrSession, limit));
    CHECKERR (rssmb_SetRFLevelOffset (instrSession, offset));
    if (rssmb_checkInstrument (instrSession, SMB, VI_TRUE) != VI_SUCCESS)
    {
        CHECKERR (rssmb_SetRFAttenuatorMode (instrSession, attenuatorMode));
	}

    CHECKERR (rssmb_SetRFVariationActive (instrSession, variationActive));
    if (variationActive == VI_TRUE)
    {
        CHECKERR (rssmb_SetRFVariationStep (instrSession, variationStep));
	}

    CHECKERR (rssmb_SetRFLevelonRecall (instrSession, RFLevel_onRecall));
    RETURN;
}

/*============================================================================
 * Function: rssmb_SetRFAmplitude
 * Purpose:  This function sets the RF output level in CW mode.
 *
 *           As with the Level value entered in the header, the OFFSet value
 *           is also taken into consideration with this command.
 *
 *           The specified value range is therefore only effective if
 *           :SOURce:POWer:OFFSet is set to 0. The value range for other
 *           OFFSet values can be calculated using the following formula:
 *
 *           Minimum level + OFFSet ... Maximum level + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFAmplitude (ViSession instrSession,
                                        ViReal64  amplitude)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW",
                              amplitude,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssmb_GetRFAmplitude
 * Purpose:  This function returns the RF output level in CW mode.
 *
 *           As with the Level value entered in the header, the OFFSet value
 *           is also taken into consideration with this function.
 *
 *           The specified value range is therefore only effective if
 *           :SOURce:POWer:OFFSet is set to 0. The value range for other
 *           OFFSet values can be calculated using the following formula:
 *
 *           Minimum level + OFFSet ... Maximum level + OFFSet
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFAmplitude (ViSession instrSession,
                                        ViReal64* amplitude)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW?",
                              amplitude);
}

/*============================================================================
 * Function: rssmb_SetRFLevelLimit
 * Purpose:  This function limits the maximum RF output level in CW and SWEEP
 *           mode. It does not influence the LEVEL display or the response to
 *           the POW? query command.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFLevelLimit (ViSession instrSession,
                                         ViReal64  limit)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:LIM",
                              limit,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssmb_GetRFLevelLimit
 * Purpose:  This function returns the RF output level limits in CW and SWEEP
 *           mode.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFLevelLimit (ViSession instrSession,
                                         ViReal64* limit)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:LIM?",
                              limit);
}

/*============================================================================
 * Function: rssmb_SetRFLevelOffset
 * Purpose:  This function specifies the constant level offset of a
 *           downstream attenuator/amplifier. If a level offset is entered,
 *           the level entered with :POWer no longer corresponds to the RF
 *           output level. The following correlation applies:
 *
 *           POWer = RF output level + POWer:OFFSet.
 *
 *           Entering a level offset does not change the RF output level, but
 *           rather the query value of :POWer.
 *
 *           Only dB is permitted as the unit here. The linear units (V, W,
 *           etc.) are not permitted.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFLevelOffset (ViSession instrSession,
                                          ViReal64  offset)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:OFFS",
                              offset,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssmb_GetRFLevelOffset
 * Purpose:  This function returns the constant level offset of a downstream
 *           attenuator/amplifier. If a level offset is entered, the level
 *           entered with :POWer no longer corresponds to the RF output
 *           level. The following correlation applies:
 *
 *           POWer = RF output level + POWer:OFFSet.
 *
 *           Entering a level offset does not change the RF output level, but
 *           rather the query value of :POWer.
 *
 *           Only dB is permitted as the unit here. The linear units (V, W,
 *           etc.) are not permitted.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFLevelOffset (ViSession instrSession,
                                          ViReal64* offset)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:OFFS?",
                              offset);
}

/*============================================================================
 * Function: rssmb_SetRFAttenuatorMode
 * Purpose:  This function switches the mode of the attenuator at the RF
 *           output (Attenuator MODe).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32   attenuatorMode)
{
    return SETPARAMENUM (instrSession,
                          "OUTP:AMOD",
                          attenuatorMode,
                          attenuatorModeArr);
}

/*============================================================================
 * Function: rssmb_GetRFAttenuatorMode
 * Purpose:  This function returns the mode of the attenuator at the RF
 *           output (Attenuator MODe).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetRFAttenuatorMode (ViSession instrSession,
                                             ViInt32*  attenuatorMode)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:AMOD?",
                          attenuatorMode,
                          attenuatorModeArr);
}

/*****************************************************************************
 * Function: Set RF Attenuator Power Search Mode
 * Purpose:  This function sets the RF output during the power search.
 *           
 *           Remote-control command(s):
 *           OUTPut:ALC:SEARch:MODE NORMal | MINimum
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetRFAttenuatorPowerSearchMode(
	ViSession	instrSession,
	ViInt32	searchMode
)
{
    return SETPARAMENUM (instrSession,
                          "OUTP:ALC:SEAR:MODE",
                          searchMode,
                          searchModeArr);
}

/*****************************************************************************
 * Function: Get RF Attenuator Power Search Mode
 * Purpose:  This function returns the RF output during the power search.
 *           
 *           Remote-control command(s):
 *           OUTPut:ALC:SEARch:MODE?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetRFAttenuatorPowerSearchMode(
	ViSession	instrSession,
	ViInt32*	searchMode
)
{
    return GETPARAMENUM (instrSession,
                          "OUTP:ALC:SEAR:MODE?",
                          searchMode,
                          searchModeArr);
}

/*============================================================================
 * Function: rssmb_GetRFAttenuatorLowerRange
 * Purpose:  This function queries the minimum level which can be set without
 *           the attenuator being adjusted (Attenuator FIXed).
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFAttenuatorLowerRange (ViSession instrSession,
                                                   ViReal64* attenuatorLowRange)
{
    return GETPARAMREAL64 (instrSession,
                              "OUTP:AFIX:RANG:LOW?",
                              attenuatorLowRange);
}

/*============================================================================
 * Function: rssmb_GetRFAttenuatorUpperRange
 * Purpose:  This function queries the maximum level which can be set without
 *           the attenuator being adjusted (Attenuator FIXed).
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFAttenuatorUpperRange (ViSession instrSession,
                                                   ViReal64* attenuatorUppRange)
{
    return GETPARAMREAL64 (instrSession,
                              "OUTP:AFIX:RANG:UPP?",
                              attenuatorUppRange);
}

/*============================================================================
 * Function: rssmb_SetRFLevelVariationActive
 * Purpose:  This function activates (USER) or deactivates (DECimal) the
 *           user-defined step width used when varying the level value with
 *           the level values UP/DOWN. This function is linked to setting
 *           Variation Active for manual control, i.e. the command also
 *           activates/deactives the user-defined step width used when
 *           varying the level value with the rotary knob.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetRFLevelVariationActive (ViSession instrSession,
                                                   ViBoolean variationActive)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:STEP:MODE",
                          (ViInt32) variationActive,
                          variationArr);
}

/*============================================================================
 * Function: rssmb_GetRFLevelVariationActive
 * Purpose:  This function returns level step mode. This function is linked
 *           to setting Variation Active for manual control, i.e. the
 *           function also activates/deactives the user-defined step width
 *           used when varying the level value with the rotary knob.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetRFLevelVariationActive (ViSession  instrSession,
                                                   ViBoolean* variationActive)
{
    ViStatus    rssmb_status;
    ViInt32     variation = 0;
    
    CHECKERR (GETPARAMENUM (instrSession,
                             "SOUR:POW:STEP:MODE?",
                             &variation,
                             variationArr));
    if (variationActive != NULL)
        *variationActive = (ViBoolean)((variation == 1)? 1 : 0);

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_SetRFLevelVariationStep
 * Purpose:  This function sets the step width for the level setting if UP
 *           and DOWN are used as the level values and variation mode
 *           :SOUR:POW:STEP:MODE USER is selected. This function is linked to
 *           setting Variation Step for manual control, i.e. the function
 *           also sets the step width of the rotary knob for Variation Active
 *           on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFLevelVariationStep (ViSession instrSession,
                                                 ViReal64  variationStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STEP",
                              variationStep,
                              0.0,100.0);
}

/*============================================================================
 * Function: rssmb_GetRFLevelVariationStep
 * Purpose:  This function returns the step width for the level setting if UP
 *           and DOWN are used as the level values and variation mode
 *           :SOUR:POW:STEP:MODE USER is selected. This function is linked to
 *           setting Variation Step for manual control, i.e. the function
 *           also sets the step width of the rotary knob for Variation Active
 *           on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFLevelVariationStep (ViSession instrSession,
                                                 ViReal64* variationStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STEP?",
                              variationStep);
}

/*============================================================================
 * Function: rssmb_SetRFPowerOnState
 * Purpose:  This function selects the state which the RF output assumes when
 *           the instrument is switched on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetRFPowerOnState (ViSession instrSession,
                                           ViInt32   powerOnState)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "OUTP:PON",
                          powerOnState,
                          powerOnStateArr);
}

/*============================================================================
 * Function: rssmb_GetRFPowerOnState
 * Purpose:  This function returns the state which the RF output assumes when
 *           the instrument is switched on.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_GetRFPowerOnState (ViSession instrSession,
                                           ViInt32*  powerOnState)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "OUTP:PON?",
                          powerOnState,
                          powerOnStateArr);
}

/*============================================================================
 * Function: rssmb_SetALCState
 * Purpose:  This function activates/deactivates automatic level control.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_SetALCState (ViSession instrSession,
                                     ViInt32   alcState)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:POW:ALC",
                          alcState,
                          alcStateArr);
}

/*============================================================================
 * Function: rssmb_GetALCState
 * Purpose:  This function state of the automatic level control.
 *===========================================================================*/
 
ViStatus _VI_FUNC rssmb_GetALCState (ViSession instrSession,
                                     ViInt32*  alcState)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:POW:ALC?",
                          alcState,
                          alcStateArr);
}

/*============================================================================
 * Function: rssmb_ALCSearchOnce
 * Purpose:  This function briefly activates level control for correction
 *           purposes.
 *===========================================================================*/

ViStatus _VI_FUNC rssmb_ALCSearchOnce (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "SOUR:POW:ALC:SONC");
    
}

/*============================================================================
 * Function: rssmb_SetUserCorrectionDataState
 * Purpose:  This function activates/deactivates level correction. Level
 *           correction is performed using the table which has been selected
 *           with the command CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetUserCorrectionDataState (ViSession instrSession,
                                                    ViBoolean correctionState)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:CORR",
                               correctionState);
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataState
 * Purpose:  This function queries state of the level correction. Level
 *           correction is performed using the table which has been selected
 *           with the command CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataState (ViSession instrSession,
                                                    ViBoolean *correctionState)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:CORR?",
                               correctionState);
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataCatalog
 * Purpose:  This function requests a list of user correction tables. The
 *           individual lists are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataCatalog (ViSession instrSession,
                                                      ViString directoryPath, 
                                                      ViInt32 arraySize,
                                                      ViChar _VI_FAR catalogList[])
{
    ViChar buffer[BUFFER_SIZE];

    if (!directoryPath)
        strncpy (buffer, "CORR:CSET:CAT?;*OPC?", (size_t)BUFFER_SIZE);
    else
        sprintf (buffer, "CORR:CSET:CAT? '%s';*OPC?", directoryPath);

    return rssmb_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             catalogList, "catalogList");
}

/*============================================================================
 * Function: rssmb_SetUserCorrectionDataTable
 * Purpose:  This function selects the table for user correction. Level
 *           correction must also be activated with the command
 *           SOURce<1|2>:CORRection:CSET:STATe ON.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetUserCorrectionDataTable (ViSession instrSession,
                                                    ViString correctionTable)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:CSET",
                                    "'",
                                    correctionTable);
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataTable
 * Purpose:  This function returns the table name for user correction. Level
 *           correction must also be activated with the command
 *           SOURce<1|2>:CORRection:CSET:STATe ON.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataTable (ViSession instrSession,
                                                    ViChar _VI_FAR correctionTable[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:CSET?",
                              correctionTable,
                              128);
}

/*============================================================================
 * Function: rssmb_SetUserCorrectionDataFrequencyValues
 * Purpose:  This function transfers the frequency data to the table selected
 *           with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetUserCorrectionDataFrequencyValues (ViSession instrSession,
                                                              ViInt32 arraySize,
                                                              ViReal64 _VI_FAR frequencyValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus rssmb_status;

    CHECKNULL (frequencyValues);
    p2buf = pbuffer = (ViChar*) malloc ((size_t) arraySize * 20 + 20);

    p2buf += sprintf (pbuffer, "CORR:CSET:DATA:FREQ ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", frequencyValues[i]);

    *--p2buf = '\0';

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataFrequencyValues
 * Purpose:  This function queries the frequency data from the table selected
 *           with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataFrequencyValues (ViSession instrSession,
                                                              ViInt32 arraySize,
                                                              ViReal64 _VI_FAR frequencyValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   "CORR:CSET:DATA:FREQ?",
                                   frequencyValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataNumberofFrequencyValues
 * Purpose:  This function queries the number of frequency values in the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataNumberofFrequencyValues (ViSession instrSession,
                                                                      ViInt32 *number_ofFrequencyValues)
{
    return GETPARAMINT32 (instrSession,
                             "CORR:CSET:DATA:FREQ:POIN?",
                             number_ofFrequencyValues);
}

/*============================================================================
 * Function: rssmb_SetUserCorrectionDataLevelValues
 * Purpose:  This function transfers the level correction data to the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetUserCorrectionDataLevelValues (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus rssmb_status;

    CHECKNULL (levelValues);
    p2buf = pbuffer = malloc ((size_t) arraySize * 20 + 20);

    p2buf += sprintf (pbuffer, "CORR:CSET:DATA:POW ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", levelValues[i]);

    *--p2buf = '\0';

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataLevelValues
 * Purpose:  This function queries the level correction data of the table
 *           selected with :CORRection:CSET:SELect.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataLevelValues (ViSession instrSession,
                                                          ViInt32 arraySize,
                                                          ViReal64 _VI_FAR levelValues[])
{
    return GETPARAMREAL64ARRAY (instrSession,
                                   "CORR:CSET:DATA:POW?",
                                   levelValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssmb_GetUserCorrectionDataNumberofLevelValues
 * Purpose:  This function queries the number of level values in the selected
 *           table.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetUserCorrectionDataNumberofLevelValues (ViSession instrSession,
                                                                  ViInt32 *number_ofLevelValues)
{
    return GETPARAMINT32 (instrSession,
                             "CORR:CSET:DATA:POW:POIN?",
                             number_ofLevelValues);
}

/*============================================================================
 * Function: rssmb_UserCorrectionDataDeleteTable
 * Purpose:  This function deletes the specified table.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_UserCorrectionDataDeleteTable (ViSession instrSession,
                                                       ViString tableName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "CORR:CSET:DEL",
                                    "'",
                                    tableName);
}

/*****************************************************************************
 * Function: User Correction Fill Data
 * Purpose:  This function activates the filling of the user correction list 
 *           with level values acquired by the power sensor. The command 
 *           activates the filling of the user correction list with level 
 *           values acquired by the selected power sensor. All level correction 
 *           values for the given frequency values are entered in the selected 
 *           list.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_UserCorrectionFillData(
    ViSession   instrSession,
    ViInt32 sensor
)
{
    ViChar   buffer[BUFFER_SIZE] = "";
    
    CHECKVIINT32(instrSession, sensor, 1, 3);
    sprintf (buffer, "SOUR:CORR:CSET:DATA:SENS%ld:POW:SONC\n", sensor);
    return rssmb_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get User Correction Value
 * Purpose:  This function queries the current value for user correction. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetUserCorrectionValue(
    ViSession   instrSession,
    ViReal64*   userCorrectionValue
)
{
    return GETPARAMREAL64 (instrSession,
                              "CORR:VAL?",
                              userCorrectionValue);
}

/*****************************************************************************
 * Function: Set CORRection Import Export List Mode
 * Purpose:  This function sets the dexchange list mode.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportListMode(
    ViSession   instrSession,
    ViInt32 listMode
)
{
    INIT;

    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*****************************************************************************
 * Function: Get CORRection Import Export List Mode
 * Purpose:  This function returns the dexchange list mode.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportListMode(
    ViSession   instrSession,
    ViInt32*    listMode
)
{
    INIT;

    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*****************************************************************************
 * Function: Set CORRection Import Export List File
 * Purpose:  This function calls the File Manager for selecting the list mode 
 *           list to be exported (source) into an ASCII file or the destination 
 *           for the ASCII file to be imported (destination) in. 
 *           
 *           The list mode files are stored with the fixed file extensions 
 *           *.lsw in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR. A path can also be specified in command 
 *           SOUR:LIST:DEXC:SEL, in which case the files are stored or loaded 
 *           in the specified directory.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportListFile(
    ViSession   instrSession,
    ViString    sourceFile
)
{
    INIT;

    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*****************************************************************************
 * Function: Get CORRection Import Export List File
 * Purpose:  This function calls the File Manager for querying the list mode 
 *           list to be exported (source) into an ASCII file or the destination 
 *           for the ASCII file to be imported (destination) in.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportListFile(
    ViSession   instrSession,
    ViChar _VI_FAR sourceFile[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*****************************************************************************
 * Function: CORRection Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           When import is selected, the ASCII file is imported as list mode 
 *           list.
 *           
 *           When export is selected, the list mode list is exported into the 
 *           selected ASCII file.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_CORRectionImportExportExecute(
    ViSession   instrSession
)
{
    INIT;

    return rssmb_sendCmd (instrSession, "SOUR:CORR:DEXC:EXEC");
}

/*****************************************************************************
 * Function: Set CORRection Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII file 
 *           to be imported into a list mode list (source) or the ASCII file 
 *           the list mode list is exported (destination) in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:CORR:DEXC:AFIL:SEL, in which case the files are stored or 
 *           loaded in the specified directory.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportASCIIFile(
    ViSession   instrSession,
    ViString    asciiSourceFile
)
{
    INIT;

    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:CORR:DEXC:AFIL:SEL",
                                    "'",
                                    asciiSourceFile);
}

/*****************************************************************************
 * Function: Get CORRection Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII file 
 *           to be imported into a list mode list (source) or the ASCII file 
 *           the list mode list is exported (destination) in.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportASCIIFile(
    ViSession   instrSession,
    ViChar _VI_FAR asciiSourceFile[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              "SOUR:CORR:DEXC:AFIL:SEL?",
                              asciiSourceFile,
                              128);
}

/*****************************************************************************
 * Function: Set CORRection Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to be 
 *           imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32 extension
)
{
    INIT;

    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*****************************************************************************
 * Function: Get CORRection Import Export ASCII Extension
 * Purpose:  This function queries the selected file extension of the ASCII 
 *           file to be imported or exported. Selection TXT (text file) or CSV 
 *           (Excel file) is available. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32*    extension
)
{
    INIT;

    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*****************************************************************************
 * Function: Set CORRection Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32 separator
)
{
    INIT;

    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*****************************************************************************
 * Function: Get CORRection Import Export ASCII Column Separator
 * Purpose:  This function queries the selected separator between the 
 *           frequency and level column of the ASCII table. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32*    separator
)
{
    INIT;

    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*****************************************************************************
 * Function: Set CORRection Import Export ASCII Column Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetCORRectionImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32 decimalSeparator
)
{
    INIT;

    return SETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*****************************************************************************
 * Function: Get CORRection Import Export ASCII Column Decimal Separator
 * Purpose:  This function displays the selected decimal separator used in the 
 *           ASCII data - '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetCORRectionImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32*    decimalSeparator
)
{
    INIT;

    return GETPARAMENUM (instrSession,
                          "SOUR:CORR:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*****************************************************************************
 * Function: CORRection Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of list mode data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:CORR:DEXC:AFIL:CAT?, in which case the files in the specified 
 *           directory are read.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_CORRectionImportExportCatalog(
    ViSession   instrSession,
    ViString    directoryPath,
    ViInt32 arraySize,
    ViChar _VI_FAR    catalogList[]
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    
    if (!directoryPath)
        sprintf (buffer, ":CORR:DEXC:AFIL:CAT?;*OPC?");
    else
        sprintf (buffer, ":CORR:DEXC:AFIL:CAT? '%s';*OPC?", directoryPath);

    rssmb_status = rssmb_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList,"catalogList");

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_confFrequencySweep
 * Purpose:  This function sets the frequency parameters of sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confFrequencySweep (ViSession instrSession,
                                            ViInt32 mode,
                                            ViInt32 frequencySetting,
                                            ViReal64 centerFrequency,
                                            ViReal64 span,
                                            ViReal64 startFrequency,
                                            ViReal64 stopFrequency,
                                            ViInt32 spacing, ViReal64 step,
                                            ViReal64 dwellTime)
{
    ViStatus    rssmb_status;

    CHECKVIINT32(instrSession, frequencySetting, 0, 1);

    CHECKERR (rssmb_SetFrequencySweepMode (instrSession, mode));

    if (frequencySetting == RSSMB_FREQ_CENTER_SPAN)
    {
        CHECKERR (rssmb_SetFrequencySweepCenterFreq (instrSession, centerFrequency));        
        CHECKERR (rssmb_SetFrequencySweepSpan (instrSession, span));    }
    else
    {
        CHECKERR (rssmb_SetFrequencySweepStartFreq (instrSession, startFrequency));
		CHECKERR (rssmb_SetFrequencySweepStopFreq (instrSession, stopFrequency));
	}

    CHECKERR (rssmb_SetFrequencySweepSpacing (instrSession, spacing));
    if (spacing == RSSMB_FREQ_SWEEP_SPACING_LIN)
    {
        CHECKERR (rssmb_SetFrequencySweepStepLin (instrSession, step));
	}
    else
    {
        CHECKERR (rssmb_SetFrequencySweepStepLog (instrSession, step));
    }

    CHECKERR (rssmb_SetFrequencySweepDwellTime (instrSession, dwellTime));

    RETURN;
}

/*============================================================================
 * Function: rssmb_FrequencySweepExecute
 * Purpose:  This function triggers a sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_FrequencySweepExecute (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "SOUR:SWE:FREQ:EXEC");
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepMode (ViSession instrSession,
                                               ViInt32 mode)
{
    INIT; 
    
    switch (mode)
    {
    case RSSMB_SWEEP_MODE_AUTO:
        CHECKERR (rssmb_SetSweepMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetFrequencySweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_SINGLE:
        CHECKERR (rssmb_SetSweepMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetFrequencySweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        CHECKERR (rssmb_FrequencySweepExecute (instrSession));
        break;
    case RSSMB_SWEEP_MODE_STEP:
        CHECKERR (rssmb_SetSweepMode (instrSession, RSSMB_SWEEP_LOW_MODE_MAN));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssmb_SetSweepMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetFrequencySweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssmb_SetSweepMode (instrSession, RSSMB_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssmb_SetFrequencySweepSpacing (instrSession, RSSMB_FREQ_SWEEP_SPACING_LIN));
        CHECKERR (rssmb_SetFrequencySweepStepLin (instrSession, 1.0e6));
        CHECKERR (rssmb_SetFrequencySweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        break;
    default:
        rssmb_status = rssmb_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepStartFreq
 * Purpose:  This function sets the start frequency for the sweep mode. STARt
 *           can be greater than STOP.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStartFreq (ViSession instrSession,
                                                    ViReal64 startFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STAR",
                              startFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepStartFreq
 * Purpose:  This function returns the start frequency of the sweep mode.
 *           STARt can be greater than STOP.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStartFreq (ViSession instrSession,
                                                    ViReal64 *startFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STAR?",
                              startFrequency);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepStopFreq
 * Purpose:  This function sets the stop frequency for the sweep mode. STOP
 *           can be less than STARt.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStopFreq (ViSession instrSession,
                                                   ViReal64 stopFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STOP",
                              stopFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepStopFreq
 * Purpose:  This function returns the stop frequency of the sweep mode. STOP
 *           can be less than STARt.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStopFreq (ViSession instrSession,
                                                   ViReal64 *stopFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:STOP?",
                              stopFrequency);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepCenterFreq
 * Purpose:  This function sets the center frequency of the sweep. This
 *           setting in combination with the span setting
 *           ([:SOURce]:FREQuency:SPAN) defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepCenterFreq (ViSession instrSession,
                                                     ViReal64 centerFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:CENT",
                              centerFrequency,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepCenterFreq
 * Purpose:  This function returns the center frequency of the sweep. This
 *           setting in combination with the span setting
 *           ([:SOURce]:FREQuency:SPAN) defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepCenterFreq (ViSession instrSession,
                                                     ViReal64 *centerFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:CENT?",
                              centerFrequency);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepSpan
 * Purpose:  This function specifies the span for the sweep. This setting in
 *           combination with the center frequency setting ([:SOUR]:FREQ:CENT)
 *           defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepSpan (ViSession instrSession, 
                                               ViReal64 span)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:SPAN",
                              span,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepSpan
 * Purpose:  This function returns the span of the sweep. This setting in
 *           combination with the center frequency setting ([:SOUR]:FREQ:CENT)
 *           defines the sweep range.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepSpan (ViSession instrSession, 
                                               ViReal64 *span)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:SPAN?",
                              span);
}

/*===========================================================================*/
/* Function: Set Frequency Sweep Shape
/* Purpose:  This function sets the cycle mode for a sweep sequence (shape).
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepShape(
    ViSession   instrSession,
    ViInt32 shape
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:FREQ:SHAP",
                          shape,
                          sweepShapeArr);
}

/*===========================================================================*/
/* Function: Get Frequency Sweep Shape
/* Purpose:  This function returns the cycle mode for a sweep sequence 
/*           (shape).
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepShape(
    ViSession   instrSession,
    ViInt32*    shape
)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:FREQ:SHAP?",
                          shape,
                          sweepShapeArr);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepSpacing
 * Purpose:  This function selects linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepSpacing (ViSession instrSession,
                                                  ViInt32 spacing)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:SPAC",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepSpacing
 * Purpose:  This function returns linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepSpacing (ViSession instrSession,
                                                  ViInt32 *spacing)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:SPAC?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepStepLin
 * Purpose:  This function sets the step width for linear sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStepLin (ViSession instrSession,
                                                  ViReal64 stepLin)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP",
                              stepLin,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepStepLin
 * Purpose:  This function returns the step width for linear sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStepLin (ViSession instrSession,
                                                  ViReal64 *stepLin)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP?",
                              stepLin);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepStepLog
 * Purpose:  This function specifies the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStepLog (ViSession instrSession,
                                                  ViReal64 stepLog)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP:LOG",
                              stepLog,
                              0.01, 100.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepStepLog
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStepLog (ViSession instrSession,
                                                  ViReal64 *stepLog)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:STEP:LOG?",
                              stepLog);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepDwellTime
 * Purpose:  This function sets the time taken for each frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepDwellTime (ViSession instrSession,
                                                    ViReal64 dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:DWEL",
                              dwellTime,
                              3.0e-3, 10.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepDwellTime
 * Purpose:  This function returns the time taken for each frequency step of
 *           the sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepDwellTime (ViSession instrSession,
                                                    ViReal64 *dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:DWEL?",
                              dwellTime);
}


/*============================================================================
 * Function: rssmb_SetFrequencySweepManualStep
 * Purpose:  In Sweep mode (:SOUR:FREQ:MODE SWE) this function sets the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:SWE:MODE MAN). Here only frequency values between the
 *           settings [:SOUR]:FREQ:STAR and ...:STOP are permitted. Each sweep
 *           step is triggered by a separate :SOUR:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepManualStep (ViSession instrSession,
                                                     ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepManualStep
 * Purpose:  In Sweep mode (:SOUR:FREQ:MODE SWE) this function returns the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:SWE:MODE MAN). Here only frequency values between the
 *           settings [:SOUR]:FREQ:STAR and ...:STOP are permitted. Each sweep
 *           step is triggered by a separate :SOUR:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepManualStep (ViSession instrSession,
                                                     ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:FREQ:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepPoints
 * Purpose:  This function sets the number of steps in an RF sweep. This
 *           function is linked to the command :SWEep[:FREQuency]:STEP as
 *           follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepPoints (ViSession instrSession,
                                                 ViInt32 frequencySweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:SWE:POIN",
                             frequencySweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssmb_GetFrequencySweepPoints
 * Purpose:  This function returns the number of steps in an RF sweep. This
 *           function is linked to the command :SWEep[:FREQuency]:STEP as
 *           follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepPoints (ViSession instrSession,
                                                 ViInt32 *frequencySweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:SWE:POIN?",
                             frequencySweepPoints);
}

/*============================================================================
 * Function: Set Frequency Sweep Voltage Out
 * Purpose:  This function activates the output of a sweep voltage ramp at
 *           the LF connector. The voltage range is set with functions:
 *           rssmb_SetFrequencySweepStartVoltage and
 *           rssmb_SetFrequencySweepStopVoltage.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepVoltageOut (ViSession instrSession,
                                                     ViBoolean voltageOut)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:SWE:FREQ:LFC",
                               voltageOut);
}

/*============================================================================
 * Function: Set Frequency Sweep Voltage Out
 * Purpose:  This function returns the state of the output of a sweep voltage 
 *           ramp at the LF connector. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepVoltageOut (ViSession instrSession,
                                                     ViBoolean *voltageOut)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:SWE:FREQ:LFC?",
                               voltageOut);
}

/*============================================================================
 * Function: Set Frequency Sweep Start Voltage
 * Purpose:  This function sets the start voltage for the sweep voltage ramp.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStartVoltage (ViSession instrSession,
                                                       ViReal64 startVoltage)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:FREQ:OVOL:STAR",
                              startVoltage,
                              -3.0, 3.0);
}

/*============================================================================
 * Function: Get Frequency Sweep Start Voltage
 * Purpose:  This function returns the start voltage for the sweep voltage ramp.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStartVoltage (ViSession instrSession,
                                                       ViReal64 *startVoltage)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:FREQ:OVOL:STAR?",
                              startVoltage);
}

/*============================================================================
 * Function: Set Frequency Sweep Stop Voltage
 * Purpose:  This function sets the stop voltage for the sweep voltage ramp.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFrequencySweepStopVoltage (ViSession instrSession,
                                                      ViReal64 stopVoltage)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:FREQ:OVOL:STOP",
                              stopVoltage,
                              -3.0, 3.0);
}

/*============================================================================
 * Function: Get Frequency Sweep Stop Voltage
 * Purpose:  This function returns the stop voltage for the sweep voltage ramp.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFrequencySweepStopVoltage (ViSession instrSession,
                                                      ViReal64 *stopVoltage)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:FREQ:OVOL:STOP?",
                              stopVoltage);
}

/*****************************************************************************
 * Function: Set Frequency Sweep Multiplier
 * Purpose:  This function sets the value for the multiplication factor of a 
 *           subsequent downstream instrument. 
 *           
 *           Remote-control command(s):
 *           [:SOURce]:FREQuency:MULTiplier
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetFrequencySweepMultiplier(
	ViSession	instrSession,
	ViInt32	multiplier
)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:FREQ:MULT",
                             multiplier,
                             1, 10000);
}

/*****************************************************************************
 * Function: Get Frequency Sweep Multiplier
 * Purpose:  This function returns the value for the multiplication factor of 
 *           a subsequent downstream instrument. 
 *           
 *           Remote-control command(s):
 *           [:SOURce]:FREQuency:MULTiplier?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetFrequencySweepMultiplier(
	ViSession	instrSession,
	ViInt32*	multiplier
)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:FREQ:MULT?",
                             multiplier);
}

/*============================================================================
 * Function: rssmb_confRFLevelSweep
 * Purpose:  This function sets the RF sweep parameters and changes the mode
 *           of RF level to Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confRFLevelSweep (ViSession instrSession,
                                          ViInt32 levelSweepMode,
                                          ViReal64 startLevel,
                                          ViReal64 stopLevel, ViReal64 stepLin,
                                          ViReal64 dwellTime)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetLevelSweepMode (instrSession, levelSweepMode));
    CHECKERR (rssmb_SetLevelSweepStartLevel (instrSession, startLevel));
    CHECKERR (rssmb_SetLevelSweepStopLevel (instrSession, stopLevel));
    CHECKERR (rssmb_SetLevelSweepStep (instrSession, stepLin));
    CHECKERR (rssmb_SetLevelSweepDwell (instrSession, dwellTime));
    RETURN;
}

/*============================================================================
 * Function: rssmb_LevelSweepExecute
 * Purpose:  This function triggers a sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LevelSweepExecute (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "SOUR:SWE:POW:EXEC");
}

/*============================================================================
 * Function: rssmb_SetLevelSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepMode (ViSession instrSession,
                                           ViInt32 mode)
{
    INIT;
    
    switch (mode)
    {
    case RSSMB_SWEEP_MODE_AUTO:
        CHECKERR (rssmb_SetSweepLevelMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLevelSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssmb_SetLevelMode (instrSession, RSSMB_POWER_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_SINGLE:
        CHECKERR (rssmb_SetSweepLevelMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLevelSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssmb_SetLevelMode (instrSession, RSSMB_POWER_MODE_SWEEP));
        CHECKERR (rssmb_LevelSweepExecute (instrSession));
        break;
    case RSSMB_SWEEP_MODE_STEP:
        CHECKERR (rssmb_SetSweepLevelMode (instrSession, RSSMB_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssmb_SetLevelSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssmb_SetLevelMode (instrSession, RSSMB_POWER_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssmb_SetSweepLevelMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLevelSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetLevelMode (instrSession, RSSMB_POWER_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssmb_SetSweepLevelMode (instrSession, RSSMB_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssmb_SetLevelSweepStep (instrSession, 0.5));
        CHECKERR (rssmb_SetLevelSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetLevelMode (instrSession, RSSMB_POWER_MODE_SWEEP));
        break;
    default:
        rssmb_status = rssmb_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssmb_SetLevelSweepStartLevel
 * Purpose:  This function sets the RF start level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepStartLevel (ViSession instrSession,
                                                 ViReal64 startLevel)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STAR",
                              startLevel,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepStartLevel
 * Purpose:  This function returns the RF start level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepStartLevel (ViSession instrSession,
                                                 ViReal64 *startLevel)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STAR?",
                              startLevel);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepStopLevel
 * Purpose:  This function sets the stop level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepStopLevel (ViSession instrSession,
                                                ViReal64 stopLevel)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:STOP",
                              stopLevel,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepStopLevel
 * Purpose:  This function returns the stop level in Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepStopLevel (ViSession instrSession, 
                                                ViReal64 *stopLevel)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:STOP?",
                              stopLevel);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepStep
 * Purpose:  This function sets the step width factor for logarithmic sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepStep (ViSession instrSession,
                                           ViReal64 step)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:STEP",
                              step,
                              0.1, 20.0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepStep
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepStep (ViSession instrSession, 
                                           ViReal64 *step)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:STEP?",
                              step);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepDwell
 * Purpose:  This function sets the time taken for each level step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepDwell (ViSession instrSession, 
                                            ViReal64 dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:DWEL",
                              dwellTime,
                              2E-3, 10.0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepDwell
 * Purpose:  This function returns the time taken for each level step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepDwell (ViSession instrSession,
                                            ViReal64 *dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:SWE:POW:DWEL?",
                              dwellTime);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:POW:MODE SWE) this function sets the level
 *           for the next sweep step in the Step sweep mode (:SOUR:SWE:POW:MODE
 *           MAN). Here only level values between the settings [:SOUR]:POW:STAR
 *           and ...:STOP are permitted. Each sweep step is triggered by a
 *           separate :SOUR:POW:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepManualStep (ViSession instrSession, 
                                                 ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "SOUR:POW:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepManualStep
 * Purpose:  This function returns the level of the next sweep step in the
 *           Step sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepManualStep (ViSession instrSession, 
                                                 ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "SOUR:POW:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepPoints
 * Purpose:  This function sets the number of steps in a level sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLevelSweepPoints (ViSession instrSession, 
                                             ViInt32 levelSweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "SOUR:SWE:POW:POIN",
                             levelSweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssmb_GetLevelSweepPoints
 * Purpose:  This function returns the number of steps in a level sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepPoints (ViSession instrSession, 
                                             ViInt32 *levelSweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "SOUR:SWE:POW:POIN?",
                             levelSweepPoints);
}

/*****************************************************************************
 * Function: Set Level Sweep Shape
 * Purpose:  This function selects the cycle mode for a sweep sequence 
 *           (shape).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetLevelSweepShape(
    ViSession   instrSession,
    ViInt32 shape
)
{
    return SETPARAMENUM (instrSession,
                          "SWE:POW:SHAP",
                          shape,
                          sweepShapeArr);
}

/*****************************************************************************
 * Function: Get Level Sweep Shape
 * Purpose:  This function queries the cycle mode for a sweep sequence 
 *           (shape).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetLevelSweepShape(
    ViSession   instrSession,
    ViInt32*    shape
)
{
    return GETPARAMENUM (instrSession,
                          "SWE:POW:SHAP?",
                          shape,
                          sweepShapeArr);
}

/*****************************************************************************
 * Function: Reset All Sweeps
 * Purpose:  This function resets all active sweeps to the starting point.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_ResetAllSweeps(
    ViSession   instrSession
)
{
    return rssmb_sendCmd (instrSession, "SWE:RES");
}

/*============================================================================
 * Function: rssmb_confLFFreq
 * Purpose:  This function sets parameters of LF Output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confLFFreqSweep (ViSession instrSession,
                                         ViInt32 mode, ViReal64 startFreq,
                                         ViReal64 stopFreq, ViInt32 spacing,
                                         ViReal64 step, ViReal64 dwellTime)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetLFSweepStartFreq (instrSession, startFreq));
	CHECKERR (rssmb_SetLFSweepStopFreq (instrSession, stopFreq));
    CHECKERR (rssmb_SetLFSweepSpacing (instrSession, spacing));
    if (spacing == RSSMB_LFOUTPUT_SWEEP_FREQUENCY_SPACING_LINEAR)
    {
        CHECKERR (rssmb_SetLFSweepStepLin (instrSession, step));
	}
    else
    {
        CHECKERR (rssmb_SetLFSweepStepLog (instrSession, step));
	}

    CHECKERR (rssmb_SetLFSweepDwellTime (instrSession, dwellTime));
    return rssmb_SetLFSweepMode (instrSession, mode);
}

/*============================================================================
 * Function: rssmb_LFOutputLFSweepTrigger
 * Purpose:  This function immediately starts an LF sweep. The MODE setting
 *           determines which sweep is executed, e.g.
 *           :SOURce:LFOutput:SWEep:FREQuency:MODE STEP. This function
 *           corresponds to the manual-control command EXECUTE SINGLE SWEEP.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LFSweepTrigger(ViSession    instrSession)
{
    return rssmb_sendCmd (instrSession, "LFO:SWE:EXEC");
}

/*============================================================================
 * Function: rssmb_SetLFSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepMode (ViSession instrSession, ViInt32 mode)
{
    ViStatus    rssmb_status;

    switch (mode)
    {
    case RSSMB_SWEEP_MODE_AUTO:
        CHECKERR (rssmb_SetSweepLFMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLFSweepTriggerSource (instrSession,
                                                        RSSMB_TRIG_SOURCE_SWEEP_AUTO));
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_SINGLE:
        CHECKERR (rssmb_SetSweepLFMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLFSweepTriggerSource (instrSession, RSSMB_TRIG_SOURCE_SWEEP_SINGLE));
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        CHECKERR (rssmb_LFSweepTrigger (instrSession));
        break;
    case RSSMB_SWEEP_MODE_STEP:
        CHECKERR (rssmb_SetSweepLFMode (instrSession, RSSMB_SWEEP_LOW_MODE_MAN));
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_SWEEP_LOW_MODE_AUTO));
        CHECKERR (rssmb_SetLFSweepTriggerSource (instrSession,RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    case RSSMB_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssmb_SetSweepLFMode (instrSession, RSSMB_SWEEP_LOW_MODE_STEP));
        CHECKERR (rssmb_SetLFSweepTriggerSource (instrSession,
                                                        RSSMB_TRIG_SOURCE_SWEEP_EXTERNAL));
        CHECKERR (rssmb_SetLFMode (instrSession, RSSMB_LFOUTPUT_FREQUENCY_MODE_SWEEP));
        break;
    default:
        rssmb_status = rssmb_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssmb_SetLFSweepStartFreq
 * Purpose:  This function sets the start frequency for the LF Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepStartFreq (ViSession instrSession,
                                             ViReal64  startFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STAR",
                              startFrequency,
                              0.1, 10.0e6);
}

/*============================================================================
 * Function: rssmb_GetLFSweepStartFreq
 * Purpose:  This function Returns the start frequency of the LF Sweep mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepStartFreq(ViSession  instrSession,
                                            ViReal64*  startFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STAR?",
                              startFrequency);
}

/*============================================================================
 * Function: rssmb_SetLFSweepStopFreq
 * Purpose:  This function sets the stop frequency for the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepStopFreq(ViSession   instrSession,
                                           ViReal64    stopFrequency)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STOP",
                              stopFrequency,
                              0.1, 10.0e6);
}

/*============================================================================
 * Function: rssmb_GetLFSweepStopFreq
 * Purpose:  This function returns the stop frequency of the LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepStopFreq(ViSession   instrSession,
                                           ViReal64*   stopFrequency)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:STOP?",
                              stopFrequency);
}

/*============================================================================
 * Function: rssmb_SetLFSweepSpacing
 * Purpose:  This function selects linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepSpacing(ViSession instrSession,
                                          ViInt32 spacing)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:SPAC",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssmb_GetLFSweepSpacing
 * Purpose:  This function returns linear or logarithmic sweep spacing.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepSpacing(ViSession instrSession,
                                          ViInt32*    spacing)
{
    return GETPARAMENUM (instrSession,
                          "LFO:SWE:SPAC?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: rssmb_SetLFSweepStepLin
 * Purpose:  This function sets the step width for the linear sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepStepLin(ViSession   instrSession,
                                          ViReal64    stepLin)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP",
                              stepLin,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetLFSweepStepLin
 * Purpose:  This function returns the step width for the linear sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepStepLin(ViSession   instrSession,
                                          ViReal64*   stepLin
)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP?",
                              stepLin);
}

/*============================================================================
 * Function: rssmb_SetLFSweepStepLog
 * Purpose:  This function specifies the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepStepLog(ViSession  instrSession,
                                          ViReal64    stepLog)
{
    return SETPARAMREAL64UNITS(instrSession,
                                   "LFO:SWE:STEP:LOG",
                                   stepLog,
                                   "PCT",
                                   0.01, 50.0);
}

/*============================================================================
 * Function: rssmb_GetLFSweepStepLog
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepStepLog(ViSession  instrSession,
                                          ViReal64*   stepLog)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:STEP:LOG?",
                              stepLog);
}

/*============================================================================
 * Function: rssmb_SetLFSweepDwellTime
 * Purpose:  This function sets the dwell time for each frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepDwellTime(ViSession    instrSession,
                                            ViReal64     dwellTime)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:SWE:DWEL",
                              dwellTime,
                              2.0e-3, 10.0);
}

/*============================================================================
 * Function: rssmb_GetLFSweepDwellTime
 * Purpose:  This function returns the dwell time of frequency step of the
 *           sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepDwellTime(ViSession    instrSession,
                                            ViReal64*    dwellTime)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:SWE:DWEL?",
                              dwellTime);
}

/*============================================================================
 * Function: rssmb_SetLFSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:LFO:FREQ:MODE SWE) this function sets the
 *           frequency for the next sweep step in the Step sweep mode
 *           (:SOUR:LFO:SWE:MODE MAN). Here only frequency values between the
 *           settings :SOUR:LFO:FREQ:STAR and ...:STOP are permitted. Each
 *           sweep step is triggered by a separate :SOUR:LFO:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepManualStep (ViSession instrSession,
                                              ViReal64 manualStep)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ:MAN",
                              manualStep,
                              0.0, 0.0);
}

/*============================================================================
 * Function: rssmb_GetLFSweepManualStep
 * Purpose:  In Sweep mode (:SOUR:LFO:FREQ:MODE SWE) this function returns the
 *           frequency of the next sweep step in the Step sweep mode
 *           (:SOUR:LFO:SWE:MODE MAN). Here only frequency values between the
 *           settings :SOUR:LFO:FREQ:STAR and ...:STOP are permitted. Each
 *           sweep step is triggered by a separate :SOUR:LFO:FREQ:MAN command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepManualStep (ViSession instrSession,
                                              ViReal64 *manualStep)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ:MAN?",
                              manualStep);
}

/*============================================================================
 * Function: rssmb_SetLFSweepPoints
 * Purpose:  This function sets the number of steps in an LF sweep. This
 *           function is linked to the command :LFOutput:SWEep[:FREQuency]:STEP
 *           as follows:
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFSweepPoints(ViSession  instrSession,
                                         ViInt32 lfSweepPoints)
{
    return SETPARAMINT32 (instrSession,
                             "LFO:SWE:POIN",
                             lfSweepPoints,
                             0, 0);
}

/*============================================================================
 * Function: rssmb_GetLFSweepPoints
 * Purpose:  This function returns the number of steps in an LF sweep.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFSweepPoints(ViSession  instrSession,
                                         ViInt32*    lfSweepPoints)
{
    return GETPARAMINT32 (instrSession,
                             "LFO:SWE:POIN?",
                             lfSweepPoints);
}

/*****************************************************************************
 * Function: Set LF Sweep Shape
 * Purpose:  This function selects the cycle mode for a sweep sequence 
 *           (shape).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetLFSweepShape(
    ViSession   instrSession,
    ViInt32 shape
)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:SHAP",
                          shape,
                          sweepShapeArr);
}

/*****************************************************************************
 * Function: Get LF Sweep Shape
 * Purpose:  This function queries the cycle mode for a sweep sequence 
 *           (shape).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetLFSweepShape(
    ViSession   instrSession,
    ViInt32*    shape
)
{
    return GETPARAMENUM (instrSession,
                          "LFO:SWE:SHAP?",
                          shape,
                          sweepShapeArr);
}

/*****************************************************************************
 * Function: Set LF Sweep Source *N
 * Purpose:  This function selects the source for the LFsweep.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetLFSweepSource(
    ViSession   instrSession,
    ViInt32 source
)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:FREQ:SOUR",
                          source,
                          lfGeneratorSourceArr);
}

/*****************************************************************************
 * Function: Get LF Sweep Source
 * Purpose:  This function queries the source for the LFsweep.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetLFSweepSource(
    ViSession   instrSession,
    ViInt32*    source
)
{
    return GETPARAMENUM (instrSession,
                          "LFO:SWE:FREQ:SOUR?",
                          source,
                          lfGeneratorSourceArr);
}

/*============================================================================
 * Function: rssmb_confLFFreq
 * Purpose:  This function sets parameters of LF Output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confLFFreq (ViSession instrSession,
                                    ViBoolean state,
                                    ViReal64 LFGenFrequency,
                                    ViReal64 LFGenVoltage)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetLFGenFrequency (instrSession, LFGenFrequency));
    CHECKERR (rssmb_SetLFGenVoltage (instrSession, LFGenVoltage));
	
    return rssmb_SetLFOutputState (instrSession, state);
}

/*============================================================================
 * Function: rssmb_SetLFOutputState
 * Purpose:  This function activates/deactivates the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFOutputState(ViSession instrSession,
                                         ViBoolean lfOutputState)
{
    return SETPARAMBOOL (instrSession,
                               "LFO",
                               lfOutputState);
}

/*============================================================================
 * Function: rssmb_GetLFOutputState
 * Purpose:  This function returns state the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFOutputState(ViSession instrSession,
                                         ViBoolean*    lfOutputState)
{
    return GETPARAMBOOL (instrSession,
                               "LFO?",
                               lfOutputState);
}

/*============================================================================
 * Function: rssmb_SetLFGenVoltage
 * Purpose:  This function sets the output voltage of the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFGenVoltage (ViSession instrSession,
                                         ViReal64  voltage)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:VOLT",
                              voltage,
                              0.0, 4.0);
}

/*============================================================================
 * Function: rssmb_GetLFGenVoltage
 * Purpose:  This function returns the output voltage of the LF output.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFGenVoltage (ViSession instrSession,
                                         ViReal64 *voltage)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:VOLT?",
                              voltage);
}

/*============================================================================
 * Function: rssmb_SetLFGenFrequency
 * Purpose:  This function sets the frequency of the LF signal for CW mode
 *           (:SOUR::MODE CW mode). In Sweep mode (:SOUR:LFO:FREQ:MODE SWE),
 *           the frequency is linked to the sweep frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFGenFrequency (ViSession instrSession,
                                           ViReal64  frequency)
{
    return SETPARAMREAL64 (instrSession,
                              "LFO:FREQ",
                              frequency,
                              0.1, 1.0e6);
}

/*============================================================================
 * Function: rssmb_GetLFGenFrequency
 * Purpose:  This function returns the frequency of the LF signal of CW mode
 *           (:SOUR::MODE CW mode). In Sweep mode (:SOUR:LFO:FREQ:MODE SWE),
 *           the frequency is linked to the sweep frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFGenFrequency (ViSession instrSession,
                                           ViReal64  *frequency)
{
    return GETPARAMREAL64 (instrSession,
                              "LFO:FREQ?",
                              frequency);
}

/*============================================================================
 * Function: rssmb_SetLFGenShape
 * Purpose:  This function selects the shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLFGenShape (ViSession instrSession, ViInt32 shape)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "LFO:SHAP",
                          shape,
                          shapeArr);
}

/*============================================================================
 * Function: rssmb_GetLFGenShape
 * Purpose:  This function returns the shape of the second LF generator.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLFGenShape (ViSession instrSession,
                                       ViInt32 *shape)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "LFO:SHAP?",
                          shape,
                          shapeArr);
}

/*============================================================================
 * Function: Set All Modulations State
 * Purpose:  This function switches the modulations on and off.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAllModulationsState (ViSession instrSession,
                                                ViBoolean modulationsState)
{
    return SETPARAMBOOL (instrSession,
                               "MOD:STAT",
                               modulationsState);
}

/*============================================================================
 * Function: Set LIST Index
 * Purpose:  This function sets the index for step mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTIndex (ViSession instrSession, ViInt32 index)
{
	return SETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND",
							 index,
							 0,0);
}

/*============================================================================
 * Function: Get LIST Index
 * Purpose:  This function queries the index for step mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTIndex (ViSession instrSession, ViInt32 *index)
{
	return GETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND?",
							 index);
}

/*============================================================================
 * Function: Set LIST Start Index
 * Purpose:  This function sets the start index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTStartIndex (ViSession instrSession,
                                           ViInt32 startIndex)
{
	return SETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND:STAR",
							 startIndex,
							 0, 0);
}

/*============================================================================
 * Function: Get LIST Start Index
 * Purpose:  This function returns the start index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTStartIndex (ViSession instrSession,
                                           ViInt32 *startIndex)
{
	return GETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND:STAR?",
							 startIndex);
}

/*============================================================================
 * Function: Set LIST Stop Index
 * Purpose:  This function sets the stop index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTStopIndex (ViSession instrSession,
                                          ViInt32 stopIndex)
{
	return SETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND:STOP",
							 stopIndex,
							 0, 0);
}

/*============================================================================
 * Function: Get LIST Stop Index
 * Purpose:  This function returns the stop index of the index range which
 *           defines a subgroup of frequency/level value pairs in the current
 *           list. Only the values in the set index range (:LIST:INDex:STARt
 *           ... :LIST:INDex:STOP) are processed in List mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTStopIndex (ViSession instrSession,
                                          ViInt32 *stopIndex)
{
	return GETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:IND:STOP?",
							 stopIndex);
}

/*============================================================================
 * Function: Learn LIST Mode Data
 * Purpose:  This function learns the selected list, i.e. it determines the
 *           hardware setting for the entire list. The data determined in
 *           this way is stored together with the list. When the list is
 *           activated for the first time, these settings are calculated
 *           automatically.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LearnLISTModeData (ViSession instrSession)
{
	return rssmb_sendCmd (instrSession, "SOUR:LIST:LEAR");    
}

/*****************************************************************************
 * Function: Get LIST Free Space
 * Purpose:  This function returns the free storage space for list mode lists.
 *           
 *           Remote-control command(s):
 *           [SOURce:]LIST:FREE?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetLISTFreeSpace(
	ViSession	instrSession,
	ViInt32*	freeSpace
)
{
	return GETPARAMINT32 (instrSession, 
		                     "SOUR:LIST:FREE?",
							 freeSpace);
}

/*===========================================================================
 * Function: Set LIST Import Export List Mode
 * Purpose:  This function sets the dexchange list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportListMode(
    ViSession   instrSession,
    ViInt32 listMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Get LIST Import Export List Mode
 * Purpose:  This function returns the dexchange list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportListMode(
    ViSession   instrSession,
    ViInt32*    listMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Set LIST Import Export List File
 * Purpose:  This function calls the File Manager for selecting the list 
 *           mode list to be exported (source) into an ASCII file or the 
 *           destination for the ASCII file to be imported (destination) in. 
 *           
 *           The list mode files are stored with the fixed file extensions 
 *           *.lsw in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR. A path can also be specified in command 
 *           SOUR:LIST:DEXC:SEL, in which case the files are stored or loaded 
 *           in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportListFile (ViSession instrSession,
                                                     ViString sourceFile)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:LIST:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*===========================================================================
 * Function: Get LIST Import Export List File
 * Purpose:  This function calls the File Manager for querying the list mode 
 *           list to be exported (source) into an ASCII file or the 
 *           destination for the ASCII file to be imported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportListFile (ViSession instrSession,
                                                     ViChar sourceFile[])
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMSTRING (instrSession,
                              "SOUR:LIST:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*===========================================================================
 * Function: Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           
 *           When import is selected, the ASCII file is imported as list 
 *           mode list.
 *           
 *           When export is selected, the list mode list is exported into 
 *           the selected ASCII file.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ImportExportExecute(
    ViSession   instrSession
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return rssmb_sendCmd (instrSession, "SOUR:LIST:DEXC:EXEC");
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII 
 *           file to be imported into a list mode list (source) or the ASCII 
 *           file the list mode list is exported (destination) in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:LIST:DEXC:AFIL:SEL, in which case the files are stored or 
 *           loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportASCIIFile(
    ViSession   instrSession,
    ViString    asciiSourceFile
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:LIST:DEXC:AFIL:SEL",
                                    "'",
                                    asciiSourceFile);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII 
 *           file to be imported into a list mode list (source) or the ASCII 
 *           file the list mode list is exported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportASCIIFile(
    ViSession   instrSession,
    ViChar _VI_FAR asciiSourceFile[]
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMSTRING (instrSession,
                              "SOUR:LIST:DEXC:AFIL:SEL?",
                              asciiSourceFile,
                              128);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32 extension
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Extension
 * Purpose:  This function queries the selected file extension of the ASCII 
 *           file to be imported or exported. Selection TXT (text file) or 
 *           CSV (Excel file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportASCIIExtension(
    ViSession   instrSession,
    ViInt32*    extension
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32 separator
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Column Separator
 * Purpose:  This function queries the selected separator between the 
 *           frequency and level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportASCIIColumnSeparator(
    ViSession   instrSession,
    ViInt32*    separator
    
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Set LIST Import Export ASCII Column Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32 decimalSeparator
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: Get LIST Import Export ASCII Column Decimal Separator
 * Purpose:  This function displays the selected decimal separator used in 
 *           the ASCII data - '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTImportExportASCIIColumnDecimalSeparator(
    ViSession   instrSession,
    ViInt32*    decimalSeparator
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:LIST:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: LIST Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of list mode data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified in command 
 *           SOUR:LIST:DEXC:AFIL:CAT?, in which case the files in the 
 *           specified directory are read.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTImportExportCatalog (ViSession instrSession, 
                            ViUInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return rssmb_getCatalog (instrSession,
                             ":LIST:DEXC:AFIL:CAT?;*OPC?",
                             arraySize,
                             catalogList,"catalogList");

}

/*****************************************************************************
 * Function: Configure Power Sense
 * Purpose:  This function configure the power sense (NRP-Z Power View)
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_confPowerSense(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean   continuous,
    ViInt32 unit,
    ViBoolean   permanentDisplay,
    ViInt32 source,
    ViReal64    frequency,
    ViInt32 filterMode,
    ViBoolean   useSParameters
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
	CHECKERR (rssmb_SetPowerContinuous (instrSession, sense, continuous));
    CHECKERR (rssmb_SetPowerUnit (instrSession, sense, unit));    
    CHECKERR (rssmb_SetPowerDisplayPermanentState (instrSession, sense, permanentDisplay));    
    CHECKERR (rssmb_SetPowerSource (instrSession, sense, source));    
    if (source == RSSMB_SENSE_POWER_SOURCE_USER)
    {
        CHECKERR (rssmb_SetPowerFrequency (instrSession, sense, frequency));
	}
    
    CHECKERR (rssmb_SetPowerSenseFilterMode (instrSession, sense, filterMode));    
    CHECKERR (rssmb_SetPowerUseSParameters (instrSession, sense, useSParameters));
	
	return rssmb_status;
}

/*****************************************************************************
 * Function: Set Power Offset
 * Purpose:  This function sets a level offset which is added to the measured 
 *           level value after activation with command 
 *           SENSe[:POWer]:OFFSet:STATe ON. This allows e.g. an attenuator in 
 *           the signal path to be taken into account.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerOffset(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64    offset
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    CHECKVIREAL64(instrSession, offset, -100.0, 100.0);
    
    sprintf (buffer, "SENS%ld:POW:OFFS %f", sense, offset);

    return rssmb_sendCmd (instrSession, buffer);
}

/*****************************************************************************
 * Function: Get Power Offset
 * Purpose:  This function returns a level offset which is added to the 
 *           measured level value after activation with command 
 *           SENSe[:POWer]:OFFSet:STATe ON. This allows e.g. an attenuator in 
 *           the signal path to be taken into account.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerOffset(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64*   offset
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS?", sense);

    return GETPARAMREAL64 (instrSession, 
                              buffer,
                              offset);
}

/*****************************************************************************
 * Function: Set Power Offset State
 * Purpose:  This function activates/deactivates the addition of  the level 
 *           offset to the measured value. The level offset value is set with 
 *           command SENSe[:POWer]:OFFSet.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerOffsetState(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean   offsetState
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               offsetState);
}

/*****************************************************************************
 * Function: Get Power Offset State
 * Purpose:  This function returns the state of activation the addition of  
 *           the level offset to the measured value. The level offset value is 
 *           set with command SENSe[:POWer]:OFFSet.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerOffsetState(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean*  offsetState
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:OFFS:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               offsetState);
}

/*****************************************************************************
 * Function: Set Power Continuous
 * Purpose:  This function activates/deactivates the continuous power 
 *           measurement by the R&S NRP-Z power sensors. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerContinuous(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean   continuous
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "INIT%ld:POW:CONT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               continuous);
}

/*****************************************************************************
 * Function: Get Power Continuous
 * Purpose:  This function returns activation the continuous power measurement 
 *           by the R&S NRP-Z power sensors. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerContinuous(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean*  continuous
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "INIT%ld:POW:CONT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               continuous);
}

/*****************************************************************************
 * Function: Set Power Use SParameters
 * Purpose:  This function activates/deactivates the use of the s-parameters 
 *           correction data of the connected power sensor. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerUseSParameters(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean   useSParameters
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:CORR:SPD:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               useSParameters);
}

/*****************************************************************************
 * Function: Get Power Use SParameters
 * Purpose:  This function returns the state of usage of the s-parameters 
 *           correction data of the connected power sensor. 
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerUseSParameters(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean*  useSParameters
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:CORR:SPD:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               useSParameters);
}

/*****************************************************************************
 * Function: Set Power Display Permanent State
 * Purpose:  This function switches on and off  the permanent indication of 
 *           the power measurement result in the upper right corner of the 
 *           block diagram. For each sensor, the type of sensor, the connector, 
 *           the measurement source and - if set - the offset is indicated.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerDisplayPermanentState(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean   permanentDisplay
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:STAT", sense);

    return SETPARAMBOOL (instrSession, 
                               buffer,
                               permanentDisplay);
}

/*****************************************************************************
 * Function: Get Power Display Permanent State
 * Purpose:  This function returns on and off state of the permanent 
 *           indication of the power measurement result in the upper right 
 *           corner of the block diagram. For each sensor, the type of sensor, 
 *           the connector, the measurement source and - if set - the offset is 
 *           indicated.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerDisplayPermanentState(
    ViSession   instrSession,
    ViInt32 sense,
    ViBoolean*  permanentDisplay
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:STAT?", sense);

    return GETPARAMBOOL (instrSession, 
                               buffer,
                               permanentDisplay);
}

/*****************************************************************************
 * Function: Set Power Display Permanent Priority
 * Purpose:  This function sets which power measurement result (average or 
 *           peak power) is indicated when permanent display is active.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPowerDisplayPermanentPriority(
	ViSession	instrSession,
	ViInt32	sense,
	ViInt32	priority
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:PRI", sense);

    return SETPARAMENUM (instrSession, 
                          buffer,
                          priority,
						  priorityArr);
}

/*****************************************************************************
 * Function: Get Power Display Permanent Priority
 * Purpose:  This function returns which power measurement result (average or 
 *           peak power) is indicated when permanent display is active.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerDisplayPermanentPriority(
	ViSession	instrSession,
	ViInt32	sense,
	ViInt32*	priority
)
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:DISP:PERM:PRI?", sense);

    return GETPARAMENUM (instrSession, 
                          buffer,
                          priority,
						  priorityArr);
}

/*===========================================================================*/
/* Function: Power Filter
/* Purpose:  This function selects the filter mode. For mode NSR selects 
/*           noise content and noise timeout.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerFilter(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 filterMode,
    ViReal64    noiseContent,
    ViReal64    timeout
)
{
    ViStatus    rssmb_status;


    CHECKERR (rssmb_SetPowerSenseFilterMode (instrSession, sense, filterMode));
    CHECKERR (rssmb_SetPowerFilterFixedNoiseContent (instrSession, sense, noiseContent));
    CHECKERR (rssmb_SetPowerFilterFixedNoiseTimeout (instrSession, sense, timeout));    
    
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Power Filter Mode
/* Purpose:  This function selects the filter mode. The filter length is the 
/*           multiplier for the time window and thus directly influences the 
/*           measurement time. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerSenseFilterMode(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 filterMode
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE", sense);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          filterMode,
                          powerSenseFilterModeArr);
}

/*===========================================================================*/
/* Function: Get Power Filter Mode
/* Purpose:  This function queriess the filter mode for user filter mode. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerSenseFilterMode(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    filterMode
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE?", sense);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          filterMode,
                          powerSenseFilterModeArr);
}

/*===========================================================================*/
/* Function: Set Power Filter Fixed Noise Content
/* Purpose:  This function sets the noise content for fixed noise filter 
/*           mode (:SENSe:POWer:FILTer:TYPE NSRation). This value determines 
/*           the proportion of intrinsic noise in the measured result.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerFilterFixedNoiseContent(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64    noiseContent
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR", sense);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              noiseContent,
                              0.0001, 1.0);
}

/*===========================================================================*/
/* Function: Get Power Filter Fixed Noise Content
/* Purpose:  This function returns the noise content for fixed noise filter 
/*           mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFilterFixedNoiseContent(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64*   noiseContent
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR?", sense);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              noiseContent);
}

/*===========================================================================*/
/* Function: Set Power Filter Fixed Noise Timeout
/* Purpose:  This function sets the timeout for fixed noise filter mode 
/*           (:SENSe:POWer:FILTer:TYPE NSRation). This value ensures limited 
/*           settling times.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerFilterFixedNoiseTimeout(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64    timeout
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR:MTIM", sense);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              timeout,
                              1.0, 999.99);
}

/*===========================================================================*/
/* Function: Get Power Filter Fixed Noise Timeout
/* Purpose:  This function returns the timeout for fixed noise filter mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFilterFixedNoiseTimeout(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64*   timeout
)
{
    ViChar buffer[BUFFER_SIZE];
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:FILT:NSR:MTIM?", sense);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              timeout);
}

/*===========================================================================*/
/* Function: Configure Stereo Modulation
/* Purpose:  This function configure the stereo modulation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_confStereoModulation(
    ViSession   instrSession,
    ViInt32 inputImpedance,
    ViReal64    maxModulationFrequency,
    ViReal64    fmDeviation,
    ViInt32 source,
    ViInt32 audioMode,
    ViInt32 ariType,
    ViBoolean   ariState,
    ViBoolean   pilotState,
    ViBoolean   rdsState,
    ViBoolean   stereoState
)
{
    ViStatus    rssmb_status;


    CHECKERR (rssmb_SetStereoExternalInputImpedance (instrSession, inputImpedance));
    
    CHECKERR (rssmb_SetStereoMaxModulationFrequency (instrSession, maxModulationFrequency));
    CHECKERR (rssmb_SetStereoFMDeviation (instrSession, fmDeviation));
    CHECKERR (rssmb_SetStereoSource (instrSession, source));
    CHECKERR (rssmb_SetStereoAudioMode (instrSession, audioMode));
    CHECKERR (rssmb_SetStereoARIType (instrSession, ariType));    
    CHECKERR (rssmb_SetStereoARIState (instrSession, ariState));
    CHECKERR (rssmb_SetStereoPilotState (instrSession, pilotState));
    CHECKERR (rssmb_SetStereoRDSState (instrSession, rdsState));
    CHECKERR (rssmb_SetStereoState (instrSession, stereoState));
    
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Stereo Direct
/* Purpose:  This function sends a R&S SML&R&S SMV command string to the 
/*           stereo coder. Please see the Rohde & Schwarz download area for 
/*           the manual of the R&S SML-B5 which describes the remote control 
/*           commands of the option. The manual can also be found on the R&S 
/*           SMB100A user documentation CD-ROM which is included in delivery. 
/*           The direct commands offer extended settings possibilities for 
/*           the stereo coder.
/*           
/*           Example:
/*           STER:DIR 'ARI-ID=0' - deactivates the ARI identification
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_StereoDirect(
    ViSession   instrSession,
    ViString    command
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMSTRINGQUOTED (instrSession,
                              "SOUR:STER:DIR",
                              "'",
                              command);
}

/*===========================================================================*/
/* Function: Configure Stereo ARI
/* Purpose:  This function configure Stereo ARI
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_confStereoARI(
    ViSession   instrSession,
    ViInt32 ariType,
    ViReal64    ariDeviation,
    ViInt32 ariBKCode,
    ViBoolean   ariState
)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetStereoARIType (instrSession, ariType));
    CHECKERR (rssmb_SetStereoARIDeviation (instrSession, ariDeviation));
    CHECKERR (rssmb_SetStereoARIBKCode (instrSession, ariBKCode));
    CHECKERR (rssmb_SetStereoARIState (instrSession, ariState));
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Stereo ARI BK Code
/* Purpose:  This function selects the area identification (BK) code of the 
/*           ARI signal. The six letters (six different frequencies) identify 
/*           a specific regions in each country. The code is generated if the 
/*           BK or DK+BK identifier of the ARI signal is activated.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoARIBKCode(
    ViSession   instrSession,
    ViInt32 ariBKCode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:ARI:BK:CODE",
                          ariBKCode,
                          ariBKCodeArr);
}

/*===========================================================================*/
/* Function: Get Stereo ARI BK Code
/* Purpose:  This function returns the area identification (BK) code of the 
/*           ARI signal. The six letters (six different frequencies) identify 
/*           a specific regions in each country. The code is generated if the 
/*           BK or DK+BK identifier of the ARI signal is activated.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoARIBKCode(
    ViSession   instrSession,
    ViInt32*    ariBKCode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:ARI:BK:CODE?",
                          ariBKCode,
                          ariBKCodeArr);
}

/*===========================================================================*/
/* Function: Set Stereo ARI Deviation
/* Purpose:  This function sets the frequency deviation of the ARI 
/*           subcarrier signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoARIDeviation(
    ViSession   instrSession,
    ViReal64    ariDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:ARI:DEV",
                              ariDeviation,
                              0.0, 10.0e3);
}

/*===========================================================================*/
/* Function: Get Stereo ARI Deviation
/* Purpose:  This function returns the frequency deviation of the ARI 
/*           subcarrier signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoARIDeviation(
    ViSession   instrSession,
    ViReal64*   ariDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:ARI:DEV?",
                              ariDeviation);
}

/*===========================================================================*/
/* Function: Set Stereo ARI State
/* Purpose:  This function activates/deactivates the ARI signal generation. 
/*           ARI signals can be generated  simultaneously with MPX and RDS 
/*           signals.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoARIState(
    ViSession   instrSession,
    ViBoolean   ariState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:ARI:STAT",
                               ariState);
}

/*===========================================================================*/
/* Function: Get Stereo ARI State
/* Purpose:  This function returns the ARI signal generation state. ARI 
/*           signals can be generated  simultaneously with MPX and RDS 
/*           signals.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoARIState(
    ViSession   instrSession,
    ViBoolean*  ariState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:ARI:STAT?",
                               ariState);
}

/*===========================================================================*/
/* Function: Set Stereo ARI Type
/* Purpose:  This function selects the generated identifiers of the ARI 
/*           signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoARIType(
    ViSession   instrSession,
    ViInt32 ariType
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:ARI:TYPE",
                          ariType,
                          ariTypeArr);
}

/*===========================================================================*/
/* Function: Get Stereo ARI Type
/* Purpose:  This function returns the generated identifiers of the ARI 
/*           signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoARIType(
    ViSession   instrSession,
    ViInt32*    ariType
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:ARI:TYPE?",
                          ariType,
                          ariTypeArr);
}

/*****************************************************************************
 * Function: Set Stereo ARI Type State
 * Purpose:  This function activates/deactivates the Stereo ARI Identifier.
 *           
 *           Required Option(s):
 *           The enhanced features require option R&S SMB-B5
 *           
 *           Remote-control command(s):
 *           [:SOURce]:STEReo:ARI:TYPE:STATe
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetStereoARITypeState(
	ViSession	instrSession,
	ViBoolean	ariState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                          	   "SOUR:STER:ARI:TYPE:STAT",
                          	   ariState);
}

/*****************************************************************************
 * Function: Get Stereo ARI Type State
 * Purpose:  This function returns the Stereo ARI Identifier.
 *           
 *           Required Option(s):
 *           The enhanced features require option R&S SMB-B5
 *           
 *           Remote-control command(s):
 *           [:SOURce]:STEReo:ARI:TYPE:STATe?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetStereoARITypeState(
	ViSession	instrSession,
	ViBoolean*	ariState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                          	   "SOUR:STER:ARI:TYPE:STAT?",
                          	   ariState);
}

/*===========================================================================*/
/* Function: Configure Stereo Audio
/* Purpose:  This function configure the stereo audio signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_confStereoAudio(
    ViSession   instrSession,
    ViInt32 audioMode,
    ViReal64    audioFrequency,
    ViInt32 audioPreemphasis,
    ViBoolean   audioPreemphasisState
)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetStereoAudioMode (instrSession, audioMode));
    CHECKERR (rssmb_SetStereoAudioFrequency (instrSession, audioFrequency));
    CHECKERR (rssmb_SetStereoAudioPreemphasisState (instrSession, audioPreemphasisState));
    if (audioPreemphasisState == VI_TRUE)
    {
        CHECKERR (rssmb_SetStereoAudioPreemphasis (instrSession, audioPreemphasis));
	}

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Stereo Audio Frequency
/* Purpose:  This function sets the frequency of the LF generator signal. 
/*           The command is an alias to command SOURce:LFOutput:FREQuency.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoAudioFrequency(
    ViSession   instrSession,
    ViReal64    audioFrequency
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:AUD:FREQ",
                              audioFrequency,
                              0.1, 1000000.0);
}

/*===========================================================================*/
/* Function: Get Stereo Audio Frequency
/* Purpose:  This function returns the frequency of the LF generator signal. 
/*           The command is an alias to command SOURce:LFOutput:FREQuency.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoAudioFrequency(
    ViSession   instrSession,
    ViReal64*   audioFrequency
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:AUD:FREQ?",
                              audioFrequency);
}

/*===========================================================================*/
/* Function: Set Stereo Audio Mode
/* Purpose:  This function selects the generated identifiers of the AUDio 
/*           signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoAudioMode(
    ViSession   instrSession,
    ViInt32 audioMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:AUD:MODE",
                          audioMode,
                          audioModeArr);
}

/*===========================================================================*/
/* Function: Get Stereo Audio Mode
/* Purpose:  This function returns the generated identifiers of the AUDio 
/*           signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoAudioMode(
    ViSession   instrSession,
    ViInt32*    audioMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:AUD:MODE?",
                          audioMode,
                          audioModeArr);
}

/*===========================================================================*/
/* Function: Set Stereo Audio Preemphasis
/* Purpose:  This function sets the preemphasis used for signal generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoAudioPreemphasis(
    ViSession   instrSession,
    ViInt32 audioPreemphasis
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:AUD:PRE",
                          audioPreemphasis,
                          audioPreemphasisArr);
}

/*===========================================================================*/
/* Function: Get Stereo Audio Preemphasis
/* Purpose:  This function returns the preemphasis used for signal 
/*           generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoAudioPreemphasis(
    ViSession   instrSession,
    ViInt32*    audioPreemphasis
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:AUD:PRE?",
                          audioPreemphasis,
                          audioPreemphasisArr);
}

/*===========================================================================*/
/* Function: Set Stereo Audio Preemphasis State
/* Purpose:  This function activates the use of preemphasis for signal 
/*           generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoAudioPreemphasisState(
    ViSession   instrSession,
    ViBoolean   audioPreemphasisState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:AUD:PRE:STAT",
                               audioPreemphasisState);
}

/*===========================================================================*/
/* Function: Get Stereo Audio Preemphasis State
/* Purpose:  This function returns the use of preemphasis for signal 
/*           generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoAudioPreemphasisState(
    ViSession   instrSession,
    ViBoolean*  audioPreemphasisState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:AUD:PRE:STAT?",
                               audioPreemphasisState);
}

/*===========================================================================*/
/* Function: Configure Stereo Pilot
/* Purpose:  This function configure the stereo pilot.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_confStereoPilot(
    ViSession   instrSession,
    ViReal64    pilotPhase,
    ViReal64    pilotDeviation,
    ViBoolean   pilotState
)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetStereoPilotPhase (instrSession, pilotPhase));
    CHECKERR (rssmb_SetStereoPilotDeviation (instrSession, pilotDeviation));
    CHECKERR (rssmb_SetStereoPilotState (instrSession, pilotState));
	
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Set Stereo Pilot State
/* Purpose:  This function activates/deactivates the pilot tone generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoPilotState(
    ViSession   instrSession,
    ViBoolean   pilotState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:PIL:STAT",
                               pilotState);
}

/*===========================================================================*/
/* Function: Get Stereo Pilot State
/* Purpose:  This function returns the pilot tone generation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoPilotState(
    ViSession   instrSession,
    ViBoolean*  pilotState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:PIL:STAT?",
                               pilotState);
}

/*===========================================================================*/
/* Function: Set Stereo Pilot Deviation
/* Purpose:  This function sets deviation of the pilot tone. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoPilotDeviation(
    ViSession   instrSession,
    ViReal64    pilotDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:PIL:DEV",
                              pilotDeviation,
                              0.0,10.0e3);
}

/*===========================================================================*/
/* Function: Get Stereo Pilot Deviation
/* Purpose:  This function returns the deviation of the pilot tone. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoPilotDeviation(
    ViSession   instrSession,
    ViReal64*   pilotDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:PIL:DEV?",
                              pilotDeviation);
}

/*===========================================================================*/
/* Function: Set Stereo Pilot Phase
/* Purpose:  This function sets the phase of the pilot tone in relation to 
/*           the 38 kHz carrier signal of the receiver. For a correct 
/*           demodulation, the pilot tone must be in phase with the 38 kHz 
/*           carrier.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoPilotPhase(
    ViSession   instrSession,
    ViReal64    pilotPhase
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:PIL:PHASe",
                              pilotPhase,
                              -4.0,5.0);
}

/*===========================================================================*/
/* Function: Get Stereo Pilot Phase
/* Purpose:  This function returns the phase of the pilot tone in relation 
/*           to the 38 kHz carrier signal of the receiver. For a correct 
/*           demodulation, the pilot tone must be in phase with the 38 kHz 
/*           carrier.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoPilotPhase(
    ViSession   instrSession,
    ViReal64*   pilotPhase
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:PIL:PHASe?",
                              pilotPhase);
}

/*===========================================================================*/
/* Function: Configure Stereo RDS
/* Purpose:  This function configure the stereo RDS.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_confStereoRDS(
    ViSession   instrSession,
    ViInt32 rdsDataSet,
    ViReal64    rdsDeviation,
    ViBoolean   trafficProgramState,
    ViBoolean   trafficAnnouncementState,
    ViBoolean   rdsState
)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetStereoRDSDataSet (instrSession, rdsDataSet));
    CHECKERR (rssmb_SetStereoRDSDeviation (instrSession, rdsDeviation));
    CHECKERR (rssmb_SetStereoRDSTrafficProgramState (instrSession, trafficProgramState));    
    CHECKERR (rssmb_SetStereoRDSTrafficAnnouncementState (instrSession, trafficAnnouncementState));    
    CHECKERR (rssmb_SetStereoRDSState (instrSession, rdsState));
	
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Stereo RDS Program Identification
/* Purpose:  This function queries the RDS program identification.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_StereoRDSPrgIdentification(
    ViSession   instrSession,
    ViChar _VI_FAR  programIdentification[]
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMSTRING (instrSession,
                              "SOUR:STER:RDS:PROG:IDEN?",
                              programIdentification,
                              1024);
}

/*===========================================================================*/
/* Function: Stereo RDS Program Name
/* Purpose:  This function queries the RDS program service name.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_StereoRDSPrgName(
    ViSession   instrSession,
    ViChar _VI_FAR  programName[]
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMSTRING (instrSession,
                              "SOUR:STER:RDS:PROG:NAME?",
                              programName,
                              1024);
}

/*===========================================================================*/
/* Function: Set Stereo RDS Data Set
/* Purpose:  This function selects the one of five data set provided on the 
/*           instrument for use in the RDS signal. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoRDSDataSet(
    ViSession   instrSession,
    ViInt32 rdsDataSet
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:RDS:DAT",
                          rdsDataSet,
                          rdsDataSetArr);
}

/*===========================================================================*/
/* Function: Get Stereo RDS Data Set
/* Purpose:  This function returns one of the five data set provided on the 
/*           instrument for use in the RDS signal. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoRDSDataSet(
    ViSession   instrSession,
    ViInt32*    rdsDataSet
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:RDS:DAT?",
                          rdsDataSet,
                          rdsDataSetArr);
}

/*===========================================================================*/
/* Function: Set Stereo RDS Deviation
/* Purpose:  This function sets deviation of the RDS subcarrier. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoRDSDeviation(
    ViSession   instrSession,
    ViReal64    rdsDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:RDS:DEV",
                              rdsDeviation,
                              0.0, 10000.0);
}

/*===========================================================================*/
/* Function: Get Stereo RDS Deviation
/* Purpose:  This function returns deviation of the RDS subcarrier. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoRDSDeviation(
    ViSession   instrSession,
    ViReal64*   rdsDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:RDS:DEV?",
                              rdsDeviation);
}

/*===========================================================================*/
/* Function: Set Stereo RDS State
/* Purpose:  This function activates/deactivates the RDS signal generation. 
/*           RDS signals can be generated simultaneously with MPX and ARI 
/*           signals.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoRDSState(
    ViSession   instrSession,
    ViBoolean   rdsState
)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:STAT",
                               rdsState);
}

/*===========================================================================*/
/* Function: Get Stereo RDS State
/* Purpose:  This function returns if activated the RDS signal generation. 
/*           RDS signals can be generated simultaneously with MPX and ARI 
/*           signals.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoRDSState(
    ViSession   instrSession,
    ViBoolean*  rdsState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:STAT?",
                               rdsState);
}

/*===========================================================================*/
/* Function: Set Stereo RDS Traffic Announcement State
/* Purpose:  This function sets the RDS traffic announcement. If activated, 
/*           the receiver switches from the current status, e.g. playing a 
/*           CD, to the receive mode and enables the broadcast of a traffic 
/*           announcement. The TP state has to be on.
/*           
/*           Example:
/*            STER:RDS:TRAF:PROG:STAT ON
/*             - activates RDS traffic program.
/*           
/*            STER:RDS:TRAF:ANN:STAT ON
/*             - activates RDS traffic announcement.
/*            
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoRDSTrafficAnnouncementState(
    ViSession   instrSession,
    ViBoolean   trafficAnnouncementState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:TRAF:ANN:STAT",
                               trafficAnnouncementState);
}

/*===========================================================================*/
/* Function: Get Stereo RDS Traffic Announcement State
/* Purpose:  This function returns the RDS traffic announcement. If 
/*           activated, the receiver switches from the current status, e.g. 
/*           playing a CD, to the receive mode and enables the broadcast of a 
/*           traffic announcement. The TP state has to be on.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoRDSTrafficAnnouncementState(
    ViSession   instrSession,
    ViBoolean*  trafficAnnouncementState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:TRAF:ANN:STAT?",
                               trafficAnnouncementState);
}

/*===========================================================================*/
/* Function: Set Stereo RDS Traffic Program State
/* Purpose:  This function sets the RDS traffic program. The receiver can 
/*           recognize a frequency as being capable of traffic information 
/*           only if the TP function is active.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoRDSTrafficProgramState(
    ViSession   instrSession,
    ViBoolean   trafficProgramState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:TRAF:PROG:STAT",
                               trafficProgramState);
}

/*===========================================================================*/
/* Function: Get Stereo RDS Traffic Program State
/* Purpose:  This function returns the RDS traffic program state. The 
/*           receiver can recognize a frequency as being capable of traffic 
/*           information only if the TP function is active.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoRDSTrafficProgramState(
    ViSession   instrSession,
    ViBoolean*  trafficProgramState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:RDS:TRAF:PROG:STAT?",
                               trafficProgramState);
}

/*===========================================================================*/
/* Function: Set Stereo External Input Impedance
/* Purpose:  This function selects the input impedance for the external 
/*           analog audio signal inputs L and R.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoExternalInputImpedance(
    ViSession   instrSession,
    ViInt32 inputImpedance
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:EXT:IMP",
                          inputImpedance,
                          stereoInputImpedanceArr);
}

/*===========================================================================*/
/* Function: Get Stereo External Input Impedance
/* Purpose:  This function returns the input impedance for the external 
/*           analog audio signal inputs L and R.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoExternalInputImpedance(
    ViSession   instrSession,
    ViInt32*    inputImpedance
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:EXT:IMP?",
                          inputImpedance,
                          stereoInputImpedanceArr);
}

/*===========================================================================*/
/* Function: Set Stereo Max Modulation Frequency
/* Purpose:  This function sets the maximum possibly used modulation 
/*           frequency. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoMaxModulationFrequency(
    ViSession   instrSession,
    ViReal64    maxModulationFrequency
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:MMF",
                              maxModulationFrequency,
                              0.0,18000.0);
}

/*===========================================================================*/
/* Function: Get Stereo Max Modulation Frequency
/* Purpose:  This function returns the maximum possibly used modulation 
/*           frequency. 
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoMaxModulationFrequency(
    ViSession   instrSession,
    ViReal64*   maxModulationFrequency
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:MMF?",
                              maxModulationFrequency);
}

/*===========================================================================*/
/* Function: Set Stereo FM Deviation
/* Purpose:  This function sets the MPX (Multiplex stereo signal) deviation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoFMDeviation(
    ViSession   instrSession,
    ViReal64    fmDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "SOUR:STER:DEV",
                              fmDeviation,
                              0.0,80000.0);
}

/*===========================================================================*/
/* Function: Get Stereo FM Deviation
/* Purpose:  This function returns the MPX (Multiplex stereo signal) 
/*           deviation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoFMDeviation(
    ViSession   instrSession,
    ViReal64*   fmDeviation
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "SOUR:STER:DEV?",
                              fmDeviation);
}

/*===========================================================================*/
/* Function: Set Stereo Source
/* Purpose:  This function selects the source for the audio signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoSource(
    ViSession   instrSession,
    ViInt32 source
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:STER:SOUR",
                          source,
                          stereoSourceArr);
}

/*===========================================================================*/
/* Function: Get Stereo Source
/* Purpose:  This function returns the source for the audio signal.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoSource(
    ViSession   instrSession,
    ViInt32*    source
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:STER:SOUR?",
                          source,
                          stereoSourceArr);
}

/*===========================================================================*/
/* Function: Set Stereo State
/* Purpose:  This function activates/deactivates the stereo modulation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetStereoState(
    ViSession   instrSession,
    ViBoolean   stereoState
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "SOUR:STER:STAT",
                               stereoState);
}

/*===========================================================================*/
/* Function: Get Stereo State
/* Purpose:  This function returns the state of stereo modulation.
/*           
/*           Required Option(s):
/*           The enhanced features require option R&S SMB-B5
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetStereoState(
    ViSession   instrSession,
    ViBoolean*  stereoState
)
{
    // workaround = returns ON/OFF instead of 1/0
#if 1   
    return GETPARAMBOOL (instrSession,
                               "SOUR:STER:STAT?",
                               stereoState);
#else
    ViStatus rssmb_status = VI_SUCCESS;
    ViInt32 state = 0;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    rssmb_status = GETPARAMENUM (instrSession,
                                  "SOUR:STER:STAT?",
                                  &state,
                                  modeOnOffArr);
    
    *stereoState = (ViBoolean) state;
    
    return rssmb_status;
#endif
}

/*****************************************************************************
 * Function: Get Stereo Adjustment
 * Purpose:  This function starts all adjustments which affect the analog 
 *           channels of the stereo coder option.
 *           
 *           Required Option(s):
 *           The enhanced features require option R&S SMB-B5
 *           
 *           Remote-control command(s):
 *           [:SOURce]:CALibration:STEReo:ANALog[:MEAS]?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetStereoAdjustment(
	ViSession	instrSession,
	ViInt32*	adjustment
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMINT32 (instrSession,
                             "SOUR:CAL:STER:ANAL?",
                             adjustment);
}

/*============================================================================
 * Function: rssmb_SetAMState
 * Purpose:  This function activates/deactivates amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAMState (ViSession instrSession, 
                                    ViBoolean AMState)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:AM:STAT",
                               AMState);
}

/*============================================================================
 * Function: rssmb_GetAMState
 * Purpose:  This function returns the state of the amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMState (ViSession instrSession, 
                                    ViBoolean *AMState)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:AM:STAT?",
                               AMState);
}

/*============================================================================
 * Function: Set Analog Modulations External Input Impedance
 * Purpose:  This function selects the impedance of external input for analog
 *           modulations AM, FM or PM.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAnalogModulationsExternalInputImpedance (ViSession instrSession,
                                                                    ViInt32 externalInputImpedance)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
	CHECKVIINT32(instrSession, externalInputImpedance, 2, 3);
    return SETPARAMENUM (instrSession,
                          "SOUR:INP:MOD:IMP",
                          externalInputImpedance,
                          externalInputImpedanceArr);
}

/*============================================================================
 * Function: Get Analog Modulations External Input Impedance
 * Purpose:  This function queries the impedance of external input for analog
 *           modulations AM, FM or PM.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAnalogModulationsExternalInputImpedance (ViSession instrSession,
                                                                    ViInt32 *externalInputImpedance)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:INP:MOD:IMP?",
                          externalInputImpedance,
                          externalInputImpedanceArr);
}

/*============================================================================
 * Function: rssmb_configAM
 * Purpose:  This function sets available parameter of amplitude modulation..
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_configAM (ViSession instrSession, ViBoolean AMState,
                                  ViInt32 AMSource, ViReal64 AMDepth,
                                  ViInt32 AMExternalCoupling)
{
    INIT;

    CHECKERR (rssmb_SetAMSource (instrSession, AMSource));
    CHECKERR (rssmb_SetAMDepth (instrSession, AMDepth));
    if (AMSource != RSSMB_AM_SOURCE_INT)
    {
        CHECKERR (rssmb_SetAMExternalCoupling (instrSession, AMExternalCoupling));
	}
    
    return rssmb_SetAMState (instrSession, AMState);
}


/*============================================================================
 * Function: rssmb_SetAMSource
 * Purpose:  This function selects the modulation source for amplitude
 *           modulation. INT is the internal modulation source which is
 *           selected with command AM:SOURce:INTernal and configured in the
 *           SOURce:LFOutput subsystem. The external signal is input at the
 *           AM EXT connector. Internal and external modulation source can be
 *           selected at the same time.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAMSource (ViSession instrSession,
                                     ViInt32   amSource)
{
    return SETPARAMENUM (instrSession,
                          "AM:SOUR",
                          amSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_GetAMSource
 * Purpose:  This function returns the modulation source of the amplitude
 *           modulation. INT is the internal modulation source. The external
 *           signal is input at the AM EXT connector. Internal and external
 *           modulation source can be selected at the same time.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMSource (ViSession instrSession,
                                     ViInt32   *amSource)
{
    return GETPARAMENUM (instrSession,
                          "AM:SOUR?",
                          amSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_SetAMDepth
 * Purpose:  This function sets the modulation depth of the amplitude
 *           modulation in percent.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAMDepth (ViSession instrSession,
                                    ViReal64  amDepth)
{
    return SETPARAMREAL64 (instrSession,
                              "AM:DEPT",
                              amDepth,
                              0.0, 100.0);
}

/*============================================================================
 * Function: rssmb_GetAMDepth
 * Purpose:  This function returns the modulation depth of the amplitude
 *           modulation in percent.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMDepth (ViSession instrSession,
                                    ViReal64  *amDepth)
{
    return GETPARAMREAL64 (instrSession,
                              "AM:DEPT?",
                              amDepth);
}

/*============================================================================
 * Function: rssmb_SetAMExternalCoupling
 * Purpose:  This function selects the coupling mode for the external
 *           modulation input (AM EXT) in the case of amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAMExternalCoupling (ViSession instrSession,
                                               ViInt32   amExtCoupling)
{
    return SETPARAMENUM (instrSession,
                          "AM:EXT:COUP",
                          amExtCoupling,
                          amExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_GetAMExternalCoupling
 * Purpose:  This function returns the coupling mode of the external
 *           modulation input (AM EXT) in the case of amplitude modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMExternalCoupling (ViSession instrSession,
                                     ViInt32   *amExtCoupling)
{
    return GETPARAMENUM (instrSession,
                          "AM:EXT:COUP?",
                          amExtCoupling,
                          amExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_GetAMSensitivity
 * Purpose:  This function queries the input sensitivity of the AM EXT input
 *           in %/V. This function is only effective if the external
 *           modulation source is selected (SOUR:AM:SOUR EXT). The returned
 *           value depends on the modulation depth setting (SOUR:AM:DEPTh).
 *           This value is assigned to the voltage value for full modulation
 *           of the input.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMSensitivity (ViSession instrSession,
                                          ViReal64  *amSensitivity)
{
    return GETPARAMREAL64 (instrSession,
                              "AM:SENS?",
                              amSensitivity);
}

/*============================================================================
 * Function: rssmb_SetAMInternalDepth
 * Purpose:  This function sets the modulation depth for the two LF
 *           generators. The sum of the two values must not exceed the
 *           overall modulation depth set in the upper part of the menu.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAMInternalDepth (ViSession instrSession,
                                            ViReal64  amIntDepth)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              "AM:INT:DEPT",
                              amIntDepth,
                              0.0, 100.0);
}

/*============================================================================
 * Function: rssmb_GetAMInternalDepth
 * Purpose:  This function returns the modulation depth for the two LF
 *           generators.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAMInternalDepth (ViSession instrSession,
                                            ViReal64  *amIntDepth)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                              "AM:INT:DEPT",
                              amIntDepth);
}

/*============================================================================
 * Function: rssmb_confPulm
 * Purpose:  This function sets the parameters of pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confPulm (ViSession instrSession, ViBoolean state,
                                  ViInt32 source, ViInt32 polarity,
                                  ViBoolean pulseGeneratorState,
                                  ViReal64 pulsePeriod, ViReal64 pulseWidth,
                                  ViReal64 pulseDelay,
                                  ViBoolean doublePulseState,
                                  ViReal64 doublePulseWidth,
                                  ViReal64 doublePulseDelay)
{
    ViStatus    rssmb_status;

	if (doublePulseState)
	{
		CHECKERR (rssmb_SetPULMDoubleState (instrSession, doublePulseState));
		CHECKERR (rssmb_SetPULMDoubleDelay (instrSession, doublePulseDelay));
	    CHECKERR (rssmb_SetPULMDoubleWidth (instrSession, doublePulseWidth));

	}
	else
	{
	    CHECKERR (rssmb_SetPULMDelay (instrSession, pulseDelay));
		CHECKERR (rssmb_SetPULMState (instrSession, state));
	}
            
    CHECKERR (rssmb_SetPULMSource (instrSession, source));
    CHECKERR (rssmb_SetPULMPolarity (instrSession, polarity));
    CHECKERR (rssmb_SetPULMPeriod (instrSession, pulsePeriod));
    CHECKERR (rssmb_SetPULMWidth (instrSession, pulseWidth));
    CHECKERR (rssmb_SetPGENState (instrSession, pulseGeneratorState));

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_SetPULMState
 * Purpose:  This function activates/deactivates the pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMState (ViSession instrSession, 
                                      ViBoolean state)
{

    return SETPARAMBOOL (instrSession,
                               "SOUR:PULM:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetPULMState
 * Purpose:  This function returns the pulse modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMState (ViSession instrSession, 
                                      ViBoolean *state)
{

    return GETPARAMBOOL (instrSession,
                               "SOUR:PULM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssmb_SetPULMPolarity
 * Purpose:  This function sets the polarity between modulating and modulated
 *           signal. This function is only effective for an external
 *           modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMPolarity (ViSession instrSession,
                                         ViInt32   polarity)
{
    return SETPARAMENUM (instrSession,
                          "PULM:POL",
                          polarity,
                          polarityArr);
}

/*============================================================================
 * Function: rssmb_GetPULMPolarity
 * Purpose:  This function returns the polarity between modulating and
 *           modulated signal. This function is only effective for an
 *           external modulation signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMPolarity (ViSession instrSession,
                                         ViInt32   *polarity)
{
    return GETPARAMENUM (instrSession,
                          "PULM:POL?",
                          polarity,
                          polarityArr);
}

/*============================================================================
 * Function: rssmb_SetPULMSource
 * Purpose:  This function selects the source for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMSource (ViSession instrSession,
                                       ViInt32   pulmSource)
{
    return SETPARAMENUM (instrSession,
                          "PULM:SOUR",
                          pulmSource,
                          pulmSourceArr);
}

/*============================================================================
 * Function: rssmb_GetPULMSource
 * Purpose:  This function returns the source for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMSource (ViSession instrSession,
                                       ViInt32   *pulmSource)
{
    return GETPARAMENUM (instrSession,
                          "PULM:SOUR?",
                          pulmSource,
                          pulmSourceArr);
}

/*============================================================================
 * Function: rssmb_SetPULMPeriod
 * Purpose:  This function sets the period of the generated pulse. The period
 *           determines the repetition
 *           frequency of the internal signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMPeriod (ViSession instrSession,
                                       ViReal64  period)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:PER",
                              period,
                              0.0,0.0);
}

/*============================================================================
 * Function: rssmb_GetPULMPeriod
 * Purpose:  This function returns the period of the generated pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMPeriod (ViSession instrSession,
                                       ViReal64 *period)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:PER?",
                              period);
}

/*============================================================================
 * Function: rssmb_SetPULMWidth
 * Purpose:  This function sets the width of the generated pulse. The width
 *           determines the pulse length. The pulse width must be at least 20
 *           ns less than the set pulse period.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMWidth (ViSession instrSession,
                                      ViReal64  width)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:WIDT",
                              width,
                              20.0e-9,1.0);
}

/*============================================================================
 * Function: rssmb_GetPULMWidth
 * Purpose:  This function returns the width of the generated pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMWidth (ViSession instrSession,
                                      ViReal64  *period)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:WIDT?",
                              period);
}

/*============================================================================
 * Function: rssmb_SetPULMDelay
 * Purpose:  This function sets the pulse delay. The pulse delay determines
 *           the time that elapses after a trigger event before pulse
 *           modulation starts. The pulse delay is not effective for double
 *           pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMDelay (ViSession instrSession,
                                      ViReal64  delay)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:DEL",
                              delay,
                              0.0,1.0);
}

/*============================================================================
 * Function: rssmb_GetPULMDelay
 * Purpose:  This function returns the pulse delay.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMDelay (ViSession instrSession,
                                      ViReal64  *delay)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:DEL?",
                              delay);
}

/*============================================================================
 * Function: rssmb_SetPULMDoubleState
 * Purpose:  This function enables/disables double pulse generation. The two
 *           pulses are generated in one pulse period.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMDoubleState (ViSession instrSession, 
                                            ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "SOUR:PULM:DOUBLE:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetPULMDoubleState
 * Purpose:  This function returns the double pulse modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMDoubleState (ViSession instrSession, 
                                            ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "SOUR:PULM:DOUBLE:STAT?",
                               state);
}

/*============================================================================
 * Function: rssmb_SetPULMDoubleWidth
 * Purpose:  This function sets the width of the second pulse in case of
 *           double pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMDoubleWidth (ViSession instrSession,
                                            ViReal64  width)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:WIDT",
                              width,
                              20.0e-9,1.0);
}

/*============================================================================
 * Function: rssmb_GetPULMDoubleWidth
 * Purpose:  This function returns the width of the second pulse in case of
 *           double pulse generation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMDoubleWidth (ViSession instrSession,
                                            ViReal64  *width)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:WIDT?",
                              width);
}

/*============================================================================
 * Function: rssmb_SetPULMDoubleDelay
 * Purpose:  This function sets the delay from the start of the first pulse
 *           to the start of the second pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMDoubleDelay (ViSession instrSession,
                                            ViReal64  delay)
{
    return SETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:DEL",
                              delay,
                              20.0e-9,1.0);
}

/*============================================================================
 * Function: rssmb_GetPULMDoubleDelay
 * Purpose:  This function returns the delay from the start of the first
 *           pulse to the start of the second pulse.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMDoubleDelay (ViSession instrSession,
                                            ViReal64  *delay)
{
    return GETPARAMREAL64 (instrSession,
                              "PULM:DOUBLE:DEL?",
                              delay);
}

/*===========================================================================*/
/* Function: Set PULM Train Repetition
/* Purpose:  This function sets the number of repetitions for each
/*           ontime/offtime value pair. The maximum length is 1023 values. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainRepetition (ViSession instrSession,
                                                ViInt32 arraySize,
                                                ViInt32 repetition[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    INIT;
    
    CHECKOPT(K27);
    CHECKNULL (repetition);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":SOUR:PULM:TRA:REP ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%ld,", repetition[i]);

    *--p2buf = '\0';

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Get PULM Train Repetition
/* Purpose:  This function returns the number of repetitions for each
/*           ontime/offtime value pair. The maximum length is 1023 values. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainRepetition (ViSession instrSession,
                                                ViInt32 arraySize,
                                                ViInt32 repetition[])
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32ARRAY (instrSession,
                                   "SOUR:PULM:TRA:REP?",
                                   repetition,
                                   arraySize);
}

/*===========================================================================*/
/* Function: Get Train Repetition Points
/* Purpose:  This function queries the length (in points) of the repetition
/*           component of the selected list. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainRepetitionPoints (ViSession instrSession,
                                                  ViInt32 *number_ofRepetitionPoints)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:REP:POIN?",
                             number_ofRepetitionPoints);    
}

/*****************************************************************************
 * Function: PULM Train Catalog
 * Purpose:  This function requests a list of available pulse train files. The 
 *           individual pulse train files are separated by commas.
 *           
 *           The files are stored with the fixed file extensions *.pulstrn in 
 *           a directory of the user's choice. The directory applicable to the 
 *           commands is defined with the command MMEMory:CDIR.
 *           
 *           Remote-control command(s):
 *           [:SOURce]:PULM:TRAin:CATalog?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_PULMTrainCatalog(
	ViSession	instrSession,
	ViInt32	arraySize,
	ViChar _VI_FAR	catalogList[]
)
{
    INIT;
    CHECKOPT(K27);
	
	return rssmb_getCatalog (instrSession,
                             ":PULM:TRA:CAT?;*OPC?",
                             arraySize,
                             catalogList,"catalogList");

}

/*****************************************************************************
 * Function: PULM Train Select
 * Purpose:  This function selects the specified pulse train file. If a new 
 *           file is to be created, the name can be entered here. The file is 
 *           created if it does not yet exist. The file selected here is
 *           available for the further processing steps (editing) and is used 
 *           in the instrument when the pulse train mode is activated.
 *           
 *           The files are stored with the fixed file extensions *.pulstrn in 
 *           a directory of the user's choice. The directory applicable to the 
 *           command is defined with the command MMEMory:CDIR.
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *           
 *           Remote-control command(s):
 *           [SOURce:]PULM:TRAin:SELect
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_PULMTrainSelect(
	ViSession	instrSession,
	ViString	file
)
{
    INIT;
    CHECKOPT(K27);
	
	return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:SEL",
                                    "'",
                                    file);
}

/*****************************************************************************
 * Function: PULM Train Delete
 * Purpose:  This function deletes the specified pulse train file.
 *           
 *           The files are stored with the fixed file extensions *.pulstrn in 
 *           a directory of the user's choice. The directory applicable to the 
 *           command is defined with the command MMEMory:CDIR. To access the 
 *           files in this directory, only the file name has to be given
 *           without the path and the file extension.
 *           
 *           Remote-control command(s):
 *           [SOURce:]PULM:TRAin:DELete
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_PULMTrainDelete(
	ViSession	instrSession,
	ViString	file
)
{
    INIT;
    CHECKOPT(K27);
	
	return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:DEL",
                                    "'",
                                    file);
}

/*****************************************************************************
 * Function: Set PULM Train Off Time
 * Purpose:  This function sets the Offtime part of the selected file with 
 *           data. 
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPULMTrainOffTime (ViSession instrSession,
                                             ViInt32 arraySize,
                                             ViReal64 offTime[])
{
    INIT;
    CHECKOPT(K27);
    
    return SETPARAMREAL64ARRAY (instrSession,
                              "SOUR:PULM:TRA:OFFT",
                              offTime,
							  arraySize);
}

/*****************************************************************************
 * Function: Get PULM Train Off Time
 * Purpose:  This function returns the Offtime part of the selected file with 
 *           data. 
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMTrainOffTime (ViSession instrSession,
                                             ViInt32 arraySize,
                                             ViReal64 offTime[])
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMREAL64ARRAY (instrSession,
	                              "SOUR:PULM:TRA:OFFT?",
	                              offTime,
								  arraySize);
}

/*****************************************************************************
 * Function: Get PULM Train Off Time Points
 * Purpose:  This function returns the length (in points) of the offtime 
 *           component of the selected list.
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *           
 *           Remote-control command(s):
 *           [:SOURce]:PULM:TRAin:OFFTime:POINts?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMTrainOffTimePoints(
	ViSession	instrSession,
	ViInt32*	offTimePoints
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:OFFT:POIN?",
                             offTimePoints);
}

/*****************************************************************************
 * Function: Set PULM Train On Time
 * Purpose:  This function sets the Ontime part of the selected file with 
 *           data. 
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPULMTrainOnTime (ViSession instrSession,
                                            ViInt32 arraySize, ViReal64 onTime[])
{
    INIT;
    CHECKOPT(K27);
    
    return SETPARAMREAL64ARRAY (instrSession,
                              "SOUR:PULM:TRA:ONT",
                              onTime,
							  arraySize);
}

/*****************************************************************************
 * Function: Get PULM Train On Time
 * Purpose:  This function returns the Ontime part of the selected file with 
 *           data. 
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMTrainOnTime (ViSession instrSession,
                                            ViInt32 arraySize, ViReal64 onTime[])
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMREAL64ARRAY (instrSession,
                              "SOUR:PULM:TRA:ONT?",
                              onTime,
							  arraySize);
}

/*****************************************************************************
 * Function: Get PULM Train On Time Points
 * Purpose:  This function returns the length (in points) of the ontime 
 *           component of the selected list.
 *           
 *           Required Option(s):
 *           R&S SMB-K27
 *           
 *           Remote-control command(s):
 *           [:SOURce]:PULM:TRAin:ONTime:POINts?
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMTrainOnTimePoints(
	ViSession	instrSession,
	ViInt32*	onTimePoints
)
{
    INIT;
    CHECKOPT(K27);
    
    return GETPARAMINT32 (instrSession,
                             "SOUR:PULM:TRA:ONT:POIN?",
                             onTimePoints);
}

/*===========================================================================
 * Function: Set PULM Train Import Export List Mode
 * Purpose:  This function selects if pulse train lists should be imported or 
 *           exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportListMode (ViSession instrSession,
                                                          ViInt32 listMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:MODE",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export List Mode
 * Purpose:  This function returns whether pulse train lists is imported or 
 *           exported.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportListMode (ViSession instrSession,
                                                          ViInt32 *listMode)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:MODE?",
                          listMode,
                          dexcListModeArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export List File
 * Purpose:  This function calls the File Manager for selecting the pulse train 
 *           list to be exported (source) into an ASCII file or the destination 
 *           for the ASCII file to be imported (destination) in. 
 *           
 *           The list mode files are stored with the fixed file extensions 
 *           *.lsw in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR. A path can also be specified, in which case the 
 *           files are stored or loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportListFile (ViSession instrSession,
                                                          ViString sourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:DEXC:SEL",
                                    "'",
                                    sourceFile);
}

/*===========================================================================
 * Function: Get PULM Train Import Export List File
 * Purpose:  This function calls the File Manager for querying the pulse
 *           train list to be exported (source) into an ASCII file or the
 *           destination for the ASCII file to be imported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportListFile (ViSession instrSession,
                                                          ViChar sourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:PULM:TRA:DEXC:SEL?",
                              sourceFile,
                              128);
}

/*===========================================================================
 * Function: PULM Train Import Export Execute
 * Purpose:  This function starts the export or import of the selected file. 
 *           
 *           When import is selected, the ASCII file is imported as pulse 
 *           train list.
 *           
 *           When export is selected, the pulse train list is exported into 
 *           the selected ASCII file.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PULMTrainImportExportExecute (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "SOUR:PULM:TRA:DEXC:EXEC");
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII File
 * Purpose:  This function calls the File Manager for selecting the ASCII 
 *           file to be imported into a pulse train list (source) or the ASCII 
 *           file the pulse train list is exported (destination) in.
 *           
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified, in which case the 
 *           files are stored or loaded in the specified directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportASCIIFile (ViSession instrSession,
                                                           ViString ASCIISourceFile)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SOUR:PULM:TRA:DEXC:AFIL:SEL",
                                    "'",
                                    ASCIISourceFile);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII File
 * Purpose:  This function calls the File Manager for querying the ASCII file
 *           to be imported into a pulse train list (source) or the ASCII
 *           file the pulse train list is exported (destination) in.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportASCIIFile (ViSession instrSession,
                                                           ViChar ASCIISourceFile[])
{
    return GETPARAMSTRING (instrSession,
                              "SOUR:PULM:TRA:DEXC:AFIL:SEL?",
                              ASCIISourceFile,
                              128);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Extension
 * Purpose:  This function selects the file extension of the ASCII file to 
 *           be imported or exported. Selection TXT (text file) or CSV (Excel 
 *           file) is available. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportASCIIExtension (ViSession instrSession,
                                                                ViInt32 extension)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:EXT",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Extension
 * Purpose:  This function queries the selected file extension of the ASCII
 *           file to be imported or exported. Selection TXT (text file) or
 *           CSV (Excel file) is available.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportASCIIExtension (ViSession instrSession,
                                                                ViInt32 *extension)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:EXT?",
                          extension,
                          dexcExtensionArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Column Separator
 * Purpose:  This function selects the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                      ViInt32 separator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:COL",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Column Separator
 * Purpose:  This function returns the separator between the frequency and 
 *           level column of the ASCII table. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportASCIIColumnSeparator (ViSession instrSession,
                                                                      ViInt32 *separator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:COL?",
                          separator,
                          dexcSeparatorArr);
}

/*===========================================================================
 * Function: Set PULM Train Import Export ASCII Decimal Separator
 * Purpose:  This function selects the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTrainImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                       ViInt32 decimalSeparator)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:DEC",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: Get PULM Train Import Export ASCII Decimal Separator
 * Purpose:  This function returns the decimal separator used in the ASCII 
 *           data between '.' (decimal point) and ',' (comma) with 
 *           floating-point numerals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTrainImportExportASCIIDecimalSeparator (ViSession instrSession,
                                                                       ViInt32 *decimalSeparator)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRA:DEXC:AFIL:SEP:DEC?",
                          decimalSeparator,
                          dexcDecimalSeparatorArr);
}

/*===========================================================================
 * Function: PULM Train Import Export Catalog
 * Purpose:  This function requests a list of available ASCII files for 
 *           export/import of pulse train data. The individual files are 
 *           separated by commas.
 *            
 *           The ASCII files are stored with the fixed file extensions *.txt 
 *           or *.csv in a directory of the user's choice. The directory 
 *           applicable to the commands is defined with the command 
 *           MMEMory:CDIR.  A path can also be specified, in which case the 
 *           files in the specified directory are read.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PULMTrainImportExportCatalog (ViSession instrSession,
                                                      ViInt32 arraySize,
                                                      ViChar catalogList[])
{
    return rssmb_getCatalog (instrSession,
                             ":PULM:TRA:DEXC:AFIL:CAT?;*OPC?",
                             arraySize,
                             catalogList,"catalogList");

}

/*****************************************************************************
 * Function: Configure Pulse Modulation Trigger
 * Purpose:  This function configures pulse modulation trigger.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_confPulmTrigger(
    ViSession   instrSession,
    ViInt32 externalInputImpedance,
    ViInt32 triggerMode,
    ViInt32 extTriggerInputSlope,
    ViInt32 gateInputPolarity
)
{
    INIT;
    
    CHECKERR (rssmb_SetPULMTriggerMode (instrSession, triggerMode));    
    CHECKERR (rssmb_SetPULMExtTriggerInputSlope (instrSession, extTriggerInputSlope));    
    CHECKERR (rssmb_SetPULMGateInputPolarity (instrSession, gateInputPolarity));
    if (rssmb_checkInstrument (instrSession, SMB, VI_TRUE) != VI_SUCCESS)
        CHECKERR (rssmb_SetPULMExternalInputImpedance (instrSession, externalInputImpedance));
    
    return rssmb_status;
}

/*****************************************************************************
 * Function: Set PULM External Input Impedance
 * Purpose:  This function selects the impedance for external pulse trigger.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPULMExternalInputImpedance(
    ViSession   instrSession,
    ViInt32 externalInputImpedance
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRIG:EXT:IMP",
                          externalInputImpedance,
                          externalInputImpedanceArr);
}

/*****************************************************************************
 * Function: Get PULM External Input Impedance
 * Purpose:  This function queries the impedance for external pulse trigger.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMExternalInputImpedance(
    ViSession   instrSession,
    ViInt32* externalInputImpedance
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRIG:EXT:IMP?",
                          externalInputImpedance,
                          externalInputImpedanceArr);
}

/*============================================================================
 * Function: rssmb_SetPULMTriggerMode
 * Purpose:  This function selects the trigger mode for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPULMTriggerMode (ViSession instrSession,
                                            ViInt32   pulmTriggerMode)
{
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:MODE",
                          pulmTriggerMode,
                          pulmTriggerModeArr);
}

/*============================================================================
 * Function: rssmb_GetPULMTriggerMode
 * Purpose:  This function returns the trigger mode for pulse modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPULMTriggerMode (ViSession instrSession,
                                            ViInt32   *pulmTriggerMode)
{
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:MODE?",
                          pulmTriggerMode,
                          pulmTriggerModeArr);
}

/*****************************************************************************
 * Function: Set PULM Ext Trigger Input Slope
 * Purpose:  This function sets the active slope of an externally applied 
 *           trigger signal at the TRIGGER input (BNC connector at the rear of 
 *           the instrument).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPULMExtTriggerInputSlope(
    ViSession   instrSession,
    ViInt32 extTriggerInputSlope
)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:PULM:TRIG:EXT:SLOP",
                          extTriggerInputSlope,
                          slopeArr);
}

/*****************************************************************************
 * Function: Get PULM Ext Trigger Input Slope
 * Purpose:  This function queries the active slope of an externally applied 
 *           trigger signal at the TRIGGER input (BNC connector at the rear of 
 *           the instrument).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMExtTriggerInputSlope(
    ViSession   instrSession,
    ViInt32*    extTriggerInputSlope
)
{
   return GETPARAMENUM (instrSession,
                          "SOUR:PULM:TRIG:EXT:SLOP?",
                          extTriggerInputSlope,
                          slopeArr);
}

/*****************************************************************************
 * Function: Set PULM Gate Input Polarity
 * Purpose:  This function selects the active level of the gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetPULMGateInputPolarity(
    ViSession   instrSession,
    ViInt32 polarity
)
{
    return SETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:GATE:POL",
                          polarity,
                          polarityArr);
}

/*****************************************************************************
 * Function: Get PULM Gate Input Polarity
 * Purpose:  This function queries the active level of the gate signal.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPULMGateInputPolarity(
    ViSession   instrSession,
    ViInt32*    polarity
)
{
    return GETPARAMENUM (instrSession,
                          "PULM:TRIG:EXT:GATE:POL?",
                          polarity,
                          polarityArr);
}

/*****************************************************************************
 * Function: Set LFGen Output Impedance
 * Purpose:  This function selects the output impedance of the LF generator.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SetLFGenOutputImpedance (ViSession instrSession,
                                                 ViInt32 outputImpedance)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    if ((outputImpedance != RSSMB_INPUT_IMPEDANCE_LOW) && (outputImpedance != RSSMB_INPUT_IMPEDANCE_G600))
		return rssmb_ParameterError (instrSession, "outputImpedance");

    return SETPARAMENUM (instrSession,
                          "SOUR:LFO:SIMP",
                          outputImpedance,
                          externalInputImpedanceArr);
}

/*****************************************************************************
 * Function: Get LFGen Output Impedance
 * Purpose:  This function queries the output impedance of the LF generator.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetLFGenOutputImpedance (ViSession instrSession,
                                                 ViInt32 *outputImpedance)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));

    return GETPARAMENUM (instrSession,
                          "SOUR:LFO:SIMP?",
                          outputImpedance,
                          externalInputImpedanceArr);
}

/*============================================================================
 * Function: rssmb_SetPGENState
 * Purpose:  This function activates/deactivates the pulse generator. The
 *           pulse generator and the pulse modulation are activated
 *           separately.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPGENState (ViSession instrSession, 
                                      ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "PGEN:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetPGENState
 * Purpose:  This function returns the pulse generator state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPGENState (ViSession instrSession, 
                                      ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "PGEN:STAT?",
                               state);
}

 /*============================================================================
 * Function: rssmb_confFM
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confFM (ViSession instrSession, ViBoolean state,
                                ViInt32 source, ViInt32 mode,
                                ViReal64 FMDeviation, ViReal64 externalDeviation,
                                ViInt32 externalCoupling)
{
    ViStatus    rssmb_status;     

    CHECKERR (rssmb_SetFMSource (instrSession, source));
    if (rssmb_checkInstrument (instrSession, SMB, VI_FALSE) == VI_SUCCESS)
    {
        CHECKERR (rssmb_SetFMMode (instrSession, mode));
	}

    CHECKERR (rssmb_SetFMDeviation (instrSession, FMDeviation));
    if  (source != RSSMB_MODULATION_SOURCE_INT)
    {
        CHECKERR (rssmb_SetFMExternalDeviation (instrSession, externalDeviation));
        CHECKERR (rssmb_SetFMExtCoupling (instrSession, externalCoupling));
	}

    return rssmb_SetFMState (instrSession, state);
}

/*============================================================================
 * Function: rssmb_SetFMState
 * Purpose:  This function activates/deactivates frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMState (ViSession instrSession, 
                                    ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "FM:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetFMState
 * Purpose:  This function returns the frequency modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMState (ViSession instrSession, 
                                    ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "FM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssmb_SetFMSource
 * Purpose:  This function sets the source for the FM signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMSource (ViSession instrSession,
                                     ViInt32   fmSource)
{
    return SETPARAMENUM (instrSession,
                          "FM:SOUR",
                          fmSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_GetFMSource
 * Purpose:  This function retruns the FM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMSource (ViSession instrSession,
                                     ViInt32   *fmSource)
{
    return GETPARAMENUM (instrSession,
                          "FM:SOUR?",
                          fmSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_SetFMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMMode (ViSession instrSession,
                                   ViInt32   fmMode)
{
    INIT;

    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));

    return SETPARAMENUM (instrSession,
                          "FM:MODE",
                          fmMode,
                          fmModeArr);
}

/*============================================================================
 * Function: rssmb_GetFMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMMode (ViSession instrSession,
                                   ViInt32   *fmMode)
{
    INIT;

    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    
    return GETPARAMENUM (instrSession,
                          "FM:MODE?",
                          fmMode,
                          fmModeArr);
}

/*============================================================================
 * Function: rssmb_SetFMDeviation
 * Purpose:  Sets the modulation deviation of the frequency modulation in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMDeviation (ViSession instrSession,
                                        ViReal64  deviation)
{
    return SETPARAMREAL64 (instrSession,
                              "FM:DEV",
                              deviation,
                              0.0,20.0e+6);
}

/*============================================================================
 * Function: rssmb_GetFMDeviation
 * Purpose:  This function returns the the modulation depth in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMDeviation (ViSession instrSession,
                                        ViReal64  *deviation)
{
    return GETPARAMREAL64 (instrSession,
                              "FM:DEV?",
                              deviation);
}

/*============================================================================
 * Function: rssmb_SetFMExtCoupling
 * Purpose:  This function selects the coupling modefor the external
 *           modulation input (FM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMExtCoupling (ViSession instrSession,
                                          ViInt32   fmExtCoupling)
{
    return SETPARAMENUM (instrSession,
                          "FM:EXT:COUP",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_GetFMExtCoupling
 * Purpose:  This function returns the coupling modefor the external
 *           modulation input (FM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMExtCoupling (ViSession instrSession,
                                          ViInt32   *fmExtCoupling)
{
    return GETPARAMENUM (instrSession,
                          "FM:EXT:COUP?",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_SetFMExternalDeviation
 * Purpose:  This function sets the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetFMExternalDeviation (ViSession instrSession,
                                                ViReal64 externalDeviation)
{
    return SETPARAMREAL64 (instrSession,
                              "FM:EXT:DEV",
                              externalDeviation,
                              0.0, 40.0e6);
}

/*============================================================================
 * Function: rssmb_GetFMExternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetFMExternalDeviation (ViSession instrSession,
                                                ViReal64 *externalDeviation)
{
    return GETPARAMREAL64 (instrSession,
                              "FM:EXT:DEV?",
                              externalDeviation);
}

/*****************************************************************************
 * Function: Get FM Sensitivity
 * Purpose:  This function queries the input sensitivity of the EXT MOD input 
 *           in Hz/V. The function is only effective if the external modulation 
 *           source is selected (SOUR:FM:SOUR EXT). The returned value depends 
 *           on the modulation deviation setting (SOUR:FM:DEViation). This 
 *           value is assigned to the voltage value for full modulation of the 
 *           input signal
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetFMSensitivity(
    ViSession   instrSession,
    ViReal64*   sensitivity
)
{
    return GETPARAMREAL64 (instrSession,
                           "FM:SENS?",
                           sensitivity);    
}
 /*============================================================================
 * Function: rssmb_confPM
 * Purpose:  This function returns the step width factor for logarithmic
 *           sweeps.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confPM (ViSession instrSession, ViBoolean state,
                                ViInt32 source, ViInt32 mode,
                                ViReal64 PMDeviation, ViReal64 externalDeviation,
                                ViInt32 externalCoupling)
{
    ViStatus    rssmb_status;
    
    CHECKERR (rssmb_SetPMSource (instrSession, source));
    if (rssmb_checkInstrument (instrSession, SMB, VI_TRUE) != VI_SUCCESS)
    {
        CHECKERR (rssmb_SetPMMode (instrSession, mode));
	}

    CHECKERR (rssmb_SetPMDeviation (instrSession, PMDeviation));
    if (source != RSSMB_MODULATION_SOURCE_INT)
    {
        CHECKERR (rssmb_SetPMExternalDeviation (instrSession, externalDeviation));
		CHECKERR (rssmb_SetPMExtCoupling (instrSession, externalCoupling));
	}
    
    return rssmb_SetPMState (instrSession, state);
}

 /*============================================================================
 * Function: rssmb_SetPMState
 * Purpose:  This function activates/deactivates frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMState (ViSession instrSession, 
                                    ViBoolean state)
{
     return SETPARAMBOOL (instrSession,
                               "PM:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetPMState
 * Purpose:  This function returns the frequency modulation state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMState (ViSession instrSession, 
                                    ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "PM:STAT?",
                               state);
}

/*============================================================================
 * Function: rssmb_SetPMSource
 * Purpose:  This function sets the source for the PM signal.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMSource (ViSession instrSession,
                                     ViInt32   fmSource)
{
    return SETPARAMENUM (instrSession,
                          "PM:SOUR",
                          fmSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_GetPMSource
 * Purpose:  This function retruns the PM source.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMSource (ViSession instrSession,
                                     ViInt32   *fmSource)
{
    return GETPARAMENUM (instrSession,
                          "PM:SOUR?",
                          fmSource,
                          sourceArr);
}

/*============================================================================
 * Function: rssmb_SetPMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMMode (ViSession instrSession,
                                   ViInt32   pmMode)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "PM:MODE",
                          pmMode,
                          pmModeArr);
}

/*============================================================================
 * Function: rssmb_GetPMMode
 * Purpose:  This function selects the mode for the frequency modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMMode (ViSession instrSession,
                                   ViInt32   *pmMode)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMENUM (instrSession,
                          "PM:MODE?",
                          pmMode,
                          pmModeArr);
}

/*============================================================================
 * Function: rssmb_SetPMDeviation
 * Purpose:  Sets the modulation deviation of the frequency modulation in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMDeviation (ViSession instrSession,
                                        ViReal64  deviation)
{
    return SETPARAMREAL64 (instrSession,
                              "PM:DEV",
                              deviation,
                              0.0,20.0);
}

/*============================================================================
 * Function: rssmb_GetPMDeviation
 * Purpose:  This function returns the the modulation depth in Hz.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMDeviation (ViSession instrSession,
                                        ViReal64  *deviation)
{
    return GETPARAMREAL64 (instrSession,
                              "PM:DEV?",
                              deviation);
}

/*============================================================================
 * Function: rssmb_SetPMExtCoupling
 * Purpose:  This function selects the coupling modefor the external
 *           modulation input (PM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMExtCoupling (ViSession instrSession,
                                          ViInt32   fmExtCoupling)
{
    return SETPARAMENUM (instrSession,
                          "PM:EXT:COUP",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_GetPMExtCoupling
 * Purpose:  This function returns the coupling modefor the external
 *           modulation input (PM/PM EXT) in the case of frequency
 *           modulation.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMExtCoupling (ViSession instrSession,
                                          ViInt32   *fmExtCoupling)
{
    return GETPARAMENUM (instrSession,
                          "PM:EXT:COUP?",
                          fmExtCoupling,
                          fmExtCouplingArr);
}

/*============================================================================
 * Function: rssmb_SetPMExternalDeviation
 * Purpose:  This function sets the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPMExternalDeviation (ViSession instrSession,
                                                ViReal64 externalDeviation)
{
    return SETPARAMREAL64 (instrSession,
                              "PM:EXT:DEV",
                              externalDeviation,
                              0.0, 20.0);
}

/*============================================================================
 * Function: rssmb_GetPMExternalDeviation
 * Purpose:  This function returns the deviation for the two frequency
 *           modulation signals.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPMExternalDeviation (ViSession instrSession,
                                                ViReal64 *externalDeviation)
{
    return GETPARAMREAL64 (instrSession,
                              "PM:EXT:DEV?",
                              externalDeviation);
}

/*****************************************************************************
 * Function: Get PM Sensitivity
 * Purpose:  This function queries queries the input sensitivity of the EXT 
 *           MOD input in RAD/V. The command is only effective if the external 
 *           modulation source is selected (SOUR:PM:SOUR EXT). The returned 
 *           value depends on the modulation deviation setting 
 *           (SOUR:PM:DEViation). This value is assigned to the voltage value 
 *           for full modulation of the input.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPMSensitivity (ViSession instrSession,
                                          ViReal64 *sensitivity)
{
    return GETPARAMREAL64 (instrSession,
                             "PM:SENS?",
                              sensitivity);
}

/*============================================================================
 * Function: Get Level Sweep Spacing
 * Purpose:  This function queries the sweep spacing. The sweep spacing for
 *           level sweeps is always linear.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLevelSweepSpacing (ViSession instrSession,
                                              ViInt32 *spacing)
{
    return GETPARAMENUM (instrSession,
                          "SOUR:SWE:POW:SPAC:MODE?",
                          spacing,
                          spacingArr);
}

/*============================================================================
 * Function: Reset Level Sweep
 * Purpose:  This function resets all active sweeps to the starting point.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ResetLevelSweep (ViSession instrSession)
{
 	return rssmb_sendCmd (instrSession, "SOUR:SWE:RES:ALL");   
}

/*============================================================================
 * Function: rssmb_confList
 * Purpose:  This function sets all parameters of selected list. If list with
 *           specified name does not exists it will be created.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_confList (ViSession instrSession,
                                  ViString listName, ViInt32 listMode,
                                  ViInt32 arraySize, ViReal64 frequencyValues[],
                                  ViReal64 powerValues[], ViReal64 dwellTime)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_SetLISTSelectList (instrSession, listName));

    CHECKERR (rssmb_SetLISTFrequencyValues(instrSession, arraySize, frequencyValues));
    CHECKERR (rssmb_SetLISTPowerValues(instrSession, arraySize, powerValues));
    CHECKERR (rssmb_SetLISTDwellTime(instrSession, dwellTime));
    CHECKERR (rssmb_SetLISTMode(instrSession, listMode));
    RETURN;
}

/*============================================================================
 * Function: rssmb_LISTCatalog
 * Purpose:  This function requests a list of available lists. The individual
 *           lists are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTCatalog (ViSession instrSession, ViString directoryPath,
                            ViInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    ViStatus    rssmb_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    if (!directoryPath)
        sprintf (buffer, ":LIST:CAT?;*OPC?");
    else
        sprintf (buffer, ":LIST:CAT? '%s';*OPC?", directoryPath);

    rssmb_status = rssmb_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList,"catalogList");

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_LISTDeleteList
 * Purpose:  This function deletes the specified list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTDeleteList (ViSession instrSession, ViString listName)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMSTRINGQUOTED (instrSession,
                              ":LIST:DEL",
                              "'",
                              listName);
}

/*============================================================================
 * Function: rssmb_LISTDeleteAll
 * Purpose:  This function deletes all lists in the set directory. The List
 *           mode must be deactivated beforehand to ensure that no lists are
 *           selected when this function is called (:SOUR:FREQ:MODE CW or SWE).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTDeleteAll (ViSession instrSession)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return rssmb_sendCmd (instrSession, ":LIST:DEL:ALL");
}

/*============================================================================
 * Function: rssmb_LISTResetList
 * Purpose:  This function resets the list to the starting point.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTResetList (ViSession instrSession)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return rssmb_sendCmd (instrSession, ":SOUR:LIST:RES");
}

/*============================================================================
 * Function: rssmb_LISTTriggerExecute
 * Purpose:  This function immediately starts the processing of a list in list
 *           mode. It corresponds to the manual-control command Execute Single.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_LISTTriggerExecute (ViSession instrSession)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return rssmb_sendCmd (instrSession, "SOUR:LIST:TRIG:EXEC");
}

/*============================================================================
 * Function: rssmb_SetLISTMode
 * Purpose:  This function sets the list mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTMode (ViSession instrSession,
                                     ViInt32 listMode)
{
    ViStatus    rssmb_status;

    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));

    switch (listMode)
    {
    case RSSMB_SWEEP_MODE_AUTO:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                              RSSMB_LIST_TRIGGER_SOURCE_AUTO));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        break;
    case RSSMB_SWEEP_MODE_SINGLE:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                                        RSSMB_LIST_TRIGGER_SOURCE_SINGLE));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        CHECKERR (rssmb_LISTTriggerExecute (instrSession));
        break;
    case RSSMB_SWEEP_MODE_STEP:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                                        RSSMB_LIST_TRIGGER_SOURCE_SINGLE));
        break;
    case RSSMB_SWEEP_MODE_EXT_SINGLE:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_AUTO));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                              RSSMB_LIST_TRIGGER_SOURCE_EXTERNAL));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        break;
    case RSSMB_SWEEP_MODE_EXT_STEP:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                              RSSMB_LIST_TRIGGER_SOURCE_EXTERNAL));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        break;
    case RSSMB_SWEEP_MODE_EXT_HOP:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                              RSSMB_LIST_TRIGGER_SOURCE_HOP));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        break;
    case RSSMB_SWEEP_MODE_EXT_DHOP:
        CHECKERR (rssmb_SetModeLIST (instrSession, RSSMB_LIST_SWEEP_MODE_STEP));
        CHECKERR (rssmb_SetLISTTriggerSource (instrSession,
                                              RSSMB_LIST_TRIGGER_SOURCE_DHOP));
        CHECKERR (rssmb_SetFrequencyMode (instrSession, RSSMB_FREQUENCY_MODE_LIST));
        break;
    default:
        rssmb_status = rssmb_ParameterError (instrSession, "mode");
    }

    RETURN;
}

/*============================================================================
 * Function: rssmb_SetLISTDwellTime
 * Purpose:  This function sets the time for which the instrument retains a
 *           setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTDwellTime (ViSession instrSession,
                                 ViReal64 dwellTime)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMREAL64 (instrSession,
                              ":SOUR:LIST:DWEL",
                              dwellTime,
                              1.0e-3, 1.0);
}

/*============================================================================
 * Function: rssmb_GetLISTDwellTime
 * Purpose:  This function Returns the time for which the instrument retains a
 *           setting.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTDwellTime (ViSession instrSession, ViReal64 *dwellTime)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64 (instrSession,
                                  ":SOUR:LIST:DWEL?",
                                  dwellTime);
}

/*============================================================================
 * Function: rssmb_SetLISTSelectList
 * Purpose:  This function selects the specified list. If a new list is to be
 *           created, the name can be entered here. The list is created if it
 *           does not yet exist. The list selected here is available for the
 *           further processing steps (editing) and is used in the instrument
 *           when the List mode is activated.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTSelectList (ViSession instrSession,
                                            ViString listName)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMSTRINGQUOTED (instrSession,
                                    ":SOUR:LIST:SEL",
                                    "'",
                                    listName);
}

/*============================================================================
 * Function: rssmb_GetLISTSelectList
 * Purpose:  This function queries name of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTSelectList (ViSession instrSession,
                                           ViChar _VI_FAR listName[])
{
    ViStatus    rssmb_status;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    rssmb_status = GETPARAMSTRING (instrSession,
                                  "SOUR:LIST:SEL?",
                                  listName,
                                  128);

    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*============================================================================
 * Function: rssmb_SetLISTFrequencyValues
 * Purpose:  This function fills the FREQuency part of the selected list with
 *           data. The data can be given either as a list of numbers (list can
 *           be of any length and list entries must be separated by commas) or
 *           as binary block data. When block data is transferred, 8 (4) bytes
 *           are always interpreted as a floating-point number with double
 *           accuracy (see the command FORMat:DATA).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTFrequencyValues (ViSession instrSession,
                                       ViInt32 arraySize,
                                       ViReal64 _VI_FAR frequencyValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus    rssmb_status;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));

    CHECKNULL (frequencyValues);

    p2buf = pbuffer = malloc((size_t)  arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":FORM ASC;:SOUR:LIST:FREQ ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%f,", frequencyValues[i]);

    *--p2buf = '\0';

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_GetLISTFrequencyValues
 * Purpose:  This function queries the FREQuency part of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTFrequencyValues (ViSession instrSession,
                                       ViInt32 arraySize,
                                       ViReal64 _VI_FAR frequencyValues[])
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64ARRAY (instrSession,
                                   ":FORM ASC;:LIST:FREQ?",
                                   frequencyValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssmb_GetLISTFrequencyPoints
 * Purpose:  This function queries the length (in points) of the FREQuency
 *           component of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTFrequencyPoints (ViSession instrSession,
                                       ViInt32 *number_ofFrequencyPoints)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMINT32 (instrSession,
                             ":LIST:FREQ:POIN?",
                             number_ofFrequencyPoints);
}

/*============================================================================
 * Function: rssmb_SetLISTPowerValues
 * Purpose:  This function fills the Level part of the selected list with
 *           data. The data can be given either as a list of numbers (list can
 *           be of any length and list entries must be separated by commas) or
 *           as binary block data. When block data is transferred, 8 (4) bytes
 *           are always interpreted as a floating-point number with double
 *           accuracy (see the command FORMat:DATA).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetLISTPowerValues (ViSession instrSession, 
                                            ViInt32 arraySize,
                                            ViReal64 _VI_FAR powerValues[])
{
    ViChar* pbuffer;
    ViChar* p2buf;
    ViInt32 i;
    ViStatus    rssmb_status;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));

    CHECKNULL (powerValues);
    
    p2buf = pbuffer = malloc((size_t)arraySize*20 + 30);
    p2buf += sprintf(pbuffer, ":FORM ASC;:SOUR:LIST:POW ");

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf (p2buf, "%fdBm,", powerValues[i]);

    *--p2buf = '\0';

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);
    free (pbuffer);
    
    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_GetLISTPowerValues
 * Purpose:  This function queries the Level part of the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTPowerValues (ViSession instrSession, ViInt32 arraySize,
                                   ViReal64 _VI_FAR powerValues[])
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMREAL64ARRAY (instrSession,
                                   ":FORM ASC;:LIST:POW?",
                                   powerValues,
                                   arraySize);
}

/*============================================================================
 * Function: rssmb_GetLISTPowerPoints
 * Purpose:  This function queries the length (in points) of the LEVel part of
 *           the selected list.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetLISTPowerPoints (ViSession instrSession,
                                   ViInt32 *number_ofPowerPoints)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMINT32 (instrSession,
                             ":LIST:POW:POIN?",
                             number_ofPowerPoints);
}

/*============================================================================
 * Function: rssmb_SetModeLIST
 * Purpose:  This function specifies how the list is to be processed (similar
 *           to SOURce:SWEep:MODE).
 *===========================================================================*/
ViStatus rssmb_SetModeLIST (ViSession instrSession,
                            ViInt32 listMode)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:MODE",
                          listMode,
                          listModeArr);
}

/*============================================================================
 * Function: rssmb_SetLISTTriggerSource
 * Purpose:  This function sets the trigger source for the LIST mode.
 *===========================================================================*/
ViStatus rssmb_SetLISTTriggerSource (ViSession instrSession,
                                     ViInt32 triggerSource)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMENUM (instrSession,
                          "SOUR:LIST:TRIG:SOUR",
                          triggerSource,
                          listTriggerSourceArr);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address
 * Purpose:  This function sets the ip address.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkIPAddress(
    ViSession   instrSession,
    ViString    ipAddress
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD",
                                    "'",
                                    ipAddress);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address
 * Purpose:  This function returns the IP address.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkIPAddress(
    ViSession   instrSession,
    ViUInt32     arraySize,
    ViChar _VI_FAR  ipAddress[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD?",
                              ipAddress,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Subnet Mask
 * Purpose:  This function sets the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkIPAddressSubnetMask(
    ViSession   instrSession,
    ViString    ipAddressSubnetMask
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD:SUBN:MASK",
                                    "'",
                                    ipAddressSubnetMask);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Subnet Mask
 * Purpose:  This function returns the IP address subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkIPAddressSubnetMask(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  ipAddressSubnetMask[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD:SUBN:MASK?",
                              ipAddressSubnetMask,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Gateway
 * Purpose:  This function sets the ip adress gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkIPAddressGateway(
    ViSession   instrSession,
    ViString    ipAddressGateway
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:IPAD:GAT",
                                    "'",
                                    ipAddressGateway);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Gateway
 * Purpose:  This function returns the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkIPAddressGateway(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  ipAddressGateway[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:IPAD:GAT?",
                              ipAddressGateway,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network IP Address Mode
 * Purpose:  This function sets the IP address mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkIPAddressMode(
    ViSession   instrSession,
    ViInt32 ipAddressMode
)
{
    return SETPARAMENUM (instrSession,
                          "SYST:COMM:NETW:IPAD:MODE",
                          ipAddressMode,
                          ipAddressModeArr);
}

/*===========================================================================
 * Function: Get SYSTem Network IP Address Mode
 * Purpose:  This function returns the IP address mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkIPAddressMode(
    ViSession   instrSession,
    ViInt32*    ipAddressMode
)
{
    return GETPARAMENUM (instrSession,
                          "SYST:COMM:NETW:IPAD:MODE?",
                          ipAddressMode,
                          ipAddressModeArr);
}

/*===========================================================================
 * Function: SYSTem Network IP Address Execute
 * Purpose:  This function execute the IP address settings.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemNetworkIPAddressExecute(
    ViSession   instrSession
)
{
    return rssmb_sendCmd (instrSession, "SYST:COMM:NETW:IPAD:SET");
}

/*===========================================================================
 * Function: Set SYSTem Network Common Workgroup
 * Purpose:  This function sets the common workgroup.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkCommonWorkgroup(
    ViSession   instrSession,
    ViString    commonWorkgroup
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:COMM:WORK",
                                    "'",
                                    commonWorkgroup);
}

/*===========================================================================
 * Function: Get SYSTem Network Common Workgroup
 * Purpose:  This function returns the common workgroup.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkCommonWorkgroup(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  commonWorkgroup[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:COMM:WORK?",
                              commonWorkgroup,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Common Domain
 * Purpose:  This function sets the common domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkCommonDomain(
    ViSession   instrSession,
    ViString    commonDomain
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:COMM:DOM",
                                    "'",
                                    commonDomain);
}

/*===========================================================================
 * Function: Get SYSTem Network Common Domain
 * Purpose:  This function returns the common domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkCommonDomain(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  commonDomain[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:COMM:DOM?",
                              commonDomain,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Common Hostname
 * Purpose:  This function sets the individual computer name of the R&S SMA.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkCommonHostname (ViSession instrSession,
                                                        ViString commonHostname)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:COMM:HOST",
                                    "'",
                                    commonHostname);    
}

/*===========================================================================
 * Function: Get SYSTem Network Common Hostname
 * Purpose:  This function returns the individual computer name of the R&S SMA.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkCommonHostname (ViSession instrSession,
                                                        ViUInt32 arraySize,
                                                        ViChar commonHostname[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:COMM:HOST?",
                              commonHostname,
                              arraySize);    
}

/*===========================================================================
 * Function: Set SYSTem Network Domain
 * Purpose:  This function sets the domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkDomain(
    ViSession   instrSession,
    ViString    domain
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:DOM",
                                    "'",
                                    domain);
}

/*===========================================================================
 * Function: Get SYSTem Network Domain
 * Purpose:  This function returns the domain.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkDomain(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  domain[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:DOM?",
                              domain,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Gateway
 * Purpose:  This function sets the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkGateway(
    ViSession   instrSession,
    ViString    gateway
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:GAT",
                                    "'",
                                    gateway);
}

/*===========================================================================
 * Function: Get SYSTem Network Gateway
 * Purpose:  This function returns the gateway.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkGateway(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  gateway[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:GAT?",
                              gateway,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network Resource
 * Purpose:  This function returns the resource string.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkResource(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  resource[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:RES?",
                              resource,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Subnet Mask
 * Purpose:  This function sets the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkSubnetMask(
    ViSession   instrSession,
    ViString    subnetMask
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:SUBN:MASK",
                                    "'",
                                    subnetMask);
}

/*===========================================================================
 * Function: Get SYSTem Network Subnet Mask
 * Purpose:  This function returns the subnet mask.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkSubnetMask(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  subnetMask[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:SUBN:MASK?",
                              subnetMask,
                              arraySize);
}

/*===========================================================================
 * Function: Set SYSTem Network Workgroup
 * Purpose:  This function sets the workgroup.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemNetworkWorkgroup(
    ViSession   instrSession,
    ViString    workgroup
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                                    "SYST:COMM:NETW:WORK",
                                    "'",
                                    workgroup);
}

/*===========================================================================
 * Function: Get SYSTem Network Workgroup
 * Purpose:  This function returns the workgroup.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkWorkgroup(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  workgroup[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:WORK?",
                              workgroup,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network Mac Address
 * Purpose:  This function returns the MAC Address.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetworkMACAddress(
    ViSession   instrSession,
    ViUInt32 arraySize,
    ViChar _VI_FAR  macAddress[]
)
{
    return GETPARAMSTRING (instrSession,
                              "SYST:COMM:NETW:MACaddress?",
                              macAddress,
                              arraySize);
}

/*===========================================================================
 * Function: Get SYSTem Network
 * Purpose:  This function retrieves the current network settings. They can
 *           be queried with the corresponding functions.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemNetwork(
    ViSession   instrSession
)
{
    return rssmb_sendCmd (instrSession,"SYST:COMM:NETW:GET");
}

/*============================================================================
 * Function: rssmb_SetInputTriggerSlope
 * Purpose:  This function sets the active slope of an externally applied
 *           trigger signal at the INST TRIG inputs (BNC connector at the rear
 *           of the instrument for Path A, pin at the AUX I/O intf for
 *           Path B). The setting is effective for both inputs at the same
 *           time.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetInstrumentTriggerInputSlope (ViSession instrSession,
                                                        ViInt32 inputTriggerSlope)
{
    return SETPARAMENUM (instrSession,
                          "INP:TRIG:SLOP",
                          inputTriggerSlope,
                          slopeArr);
}

/*============================================================================
 * Function: rssmb_GetInputTriggerSlope
 * Purpose:  This function returns the active slope of an externally applied
 *           trigger signal at the INST TRIG inputs (BNC connector at the rear
 *           of the instrument for Path A, pin at the AUX I/O intf for
 *           Path B). The setting is effective for both inputs at the same
 *           time.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetInstrumentTriggerInputSlope (ViSession instrSession,
                                                        ViInt32 *inputTriggerSlope)
{
    return GETPARAMENUM (instrSession,
                          "INP:TRIG:SLOP?",
                          inputTriggerSlope,
                          slopeArr);
}

/*============================================================================
 * Function: rssmb_TRIGgerAllSweeps
 * Purpose:  This function starts all sweeps which are activated for the
 *           respective path. The sweep to be executed depends on the
 *           respective MODE setting (:SOURce<1|2>:SWEep:POWer|FREQuency:MODE
 *           STEP). This function corresponds to the manual-control command
 *           Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerAllSweeps (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "TRIG");
}

/*============================================================================
 * Function: rssmb_TRIGgerRFFrequencySweep
 * Purpose:  This function immediately starts an RF frequency sweep. The sweep
 *           to be executed depends on the respective MODE setting
 *           (:SOURce:SWEep:FREQuency:MODE SING). This function corresponds to
 *           the manual-control command Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerRFFrequencySweep (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "TRIG:FSW");
}

/*============================================================================
 * Function: rssmb_TRIGgerRFLevelSweep
 * Purpose:  This function immediately starts an RF level sweep. The sweep to
 *           be executed depends on the set sweep mode
 *           (:SOURce<1|2>:SWEep:POWer:MODE). This function corresponds to the
 *           manual-control command Execute Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerRFLevelSweep (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "TRIG:PSW");
}

/*============================================================================
 * Function: rssmb_TRIGgerListMode
 * Purpose:  This function immediately starts the processing of a list in LIST
 *           mode. It corresponds to the manual-control command Execute
 *           Trigger.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerListMode (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "TRIG:LIST");
}

/*===========================================================================*/
/* Function: Set Angle Units
/* Purpose:  This function sets the default unit for angles. It is valid for 
/*           all commands which determine angle values. It does not influence 
/*           the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAngleUnits(
    ViSession   instrSession,
    ViInt32 angleUnits
)
{
    return SETPARAMENUM (instrSession,
                          "UNIT:ANG",
                          angleUnits,
                          angleUnitsArr);
}

/*===========================================================================*/
/* Function: Get Angle Units
/* Purpose:  This function returns the unit for angles. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAngleUnits(
    ViSession   instrSession,
    ViInt32*    angleUnits
)
{
    return GETPARAMENUM (instrSession,
                          "UNIT:ANG?",
                          angleUnits,
                          angleUnitsArr);
}

/*===========================================================================*/
/* Function: Set Power Units
/* Purpose:  This function sets the default unit for power. It is valid for 
/*           all commands which determine power values. It does not influence 
/*           the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerUnits(
    ViSession   instrSession,
    ViInt32 powerUnits
)
{
    return SETPARAMENUM (instrSession,
                          "UNIT:POW",
                          powerUnits,
                          powerUnitsArr);
}

/*===========================================================================*/
/* Function: Get Power Units
/* Purpose:  This function returns the unit for power.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerUnits(
    ViSession   instrSession,
    ViInt32*    powerUnits
)
{
    return GETPARAMENUM (instrSession,
                          "UNIT:POW?",
                          powerUnits,
                          powerUnitsArr);
}

/*===========================================================================*/
/* Function: Set Velocity Units
/* Purpose:  This function sets the default unit for velocity. It is valid 
/*           for all commands which determine velocity values. It does not 
/*           influence the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetVelocityUnits(
    ViSession   instrSession,
    ViInt32 velocityUnits
)
{
    return SETPARAMENUM (instrSession,
                          "UNIT:VEL",
                          velocityUnits,
                          velocityUnitsArr);
}

/*===========================================================================*/
/* Function: Get Velocity Units
/* Purpose:  This function returns the default unit for velocity. It is 
/*           valid for all commands which determine velocity values. It does 
/*           not influence the manual control parameter unit and the display.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetVelocityUnits(
    ViSession   instrSession,
    ViInt32*    velocityUnits
)
{
    return GETPARAMENUM (instrSession,
                          "UNIT:VEL?",
                          velocityUnits,
                          velocityUnitsArr);
}

/*===========================================================================
 * Function: Power Read Power
 * Purpose:  This function retrieves the power measurement 
 *           result of the selected power sensor. The value is
 *           provided with the unit set with command 
 *           SENSe<[1]...3>:UNIT[:POWer]. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerReadPower(
    ViSession   instrSession,
    ViInt32 read,
    ViReal64*   power
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "READ%ld:POW?", read);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              power);
}

/*===========================================================================
 * Function: Set Power Unit
 * Purpose:  The command selects the unit used for result query with command 
 *           READ. The power sensor provides the measured value in Watt. In 
 *           which unit the measured value is returned is selected here
 *           and might be either Watt, dBm or dBuV.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerUnit(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 unit
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:UNIT:POW", sense);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          unit,
                          powerUnitArr);
}

/*===========================================================================
 * Function: Get Power Unit
 * Purpose:  This function queries the unit used for result query with 
 *           command READ.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerUnit(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    unit
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:UNIT:POW?", sense);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          unit,
                          powerUnitArr);
}

/*===========================================================================
 * Function: Set Power Frequency
 * Purpose:  The command sets the RF frequency of the source if the user 
 *           source is selected (SENSe<[1]...3>[:POWer]:SOURce USER).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerFrequency(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64    frequency
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FREQ", sense);
    
    return SETPARAMREAL64 (instrSession,
                              buffer,
                              frequency,
                              0.0,0.0);
}

/*===========================================================================
 * Function: Get Power Frequency
 * Purpose:  This function queries the RF frequency. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFrequency(
    ViSession   instrSession,
    ViInt32 sense,
    ViReal64*   frequency
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FREQ?", sense);
    
    return GETPARAMREAL64 (instrSession,
                              buffer,
                              frequency);
}

/*===========================================================================
 * Function: Set Power Source
 * Purpose:  The command selects the signal source for the measurement. The 
 *           RF signal of path A or path B or a user defined source can be 
 *           selected.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerSource(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 source
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:SOUR", sense);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          source,
                          powerSourceArr);
}

/*===========================================================================
 * Function: Get Power Source
 * Purpose:  The command queries the signal source for the measurement.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerSource(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    source
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:SOUR?", sense);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          source,
                          powerSourceArr);
}

/*****************************************************************************
 * Function: Power Sensor Version
 * Purpose:  This function queries the software version of the connected NRP 
 *           power sensor.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_GetPowerSensorVersion (ViSession instrSession,
                                               ViInt32 sense, ViInt32 arraySize,
                                               ViChar version[])
{
    ViChar buffer[BUFFER_SIZE];
    INIT;
    
    CHECKVIINT32(instrSession, sense, 1, 3);
    
    sprintf (buffer, "SENS%ld:POW:SVER?", sense);

    return GETPARAMSTRING (instrSession, 
                              buffer,
                              version,
                              arraySize);
}

/*===========================================================================
 * Function: Power Sensor Serial Number
 * Purpose:  This function queries the serial number of the sensor. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerSensorSerialNumber(
    ViSession   instrSession,
    ViInt32 sense,
    ViChar _VI_FAR  serialNumber[]
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:SNUM?", sense);
    
    return GETPARAMSTRING (instrSession,
                              buffer,
                              serialNumber,
                              1024);
}

/*===========================================================================
 * Function: Power Sensor Device
 * Purpose:  This function queries if a sensor is connected at the selected 
 *           port to the signal generator. The sensor is selected by suffix 
 *           1, 2 or 3 in key word SENSe or READ of the command header. 
 *           Suffix 1 denotes the sensor connected to the SENSOR connector, 
 *           suffix 2 the sensor connected first to one of the USB interfaces 
 *           and suffix 3 the sensor connected second to one of the USB 
 *           interfaces.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerSensorDevice(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    device
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:STAT:DEV?", sense);
    
    return GETPARAMINT32 (instrSession,
                             buffer,
                             device);
}

/*===========================================================================
 * Function: Power Sensor Type
 * Purpose:  This function queries the type of sensor connected at the 
 *           selected port. The type is automatically detected.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerSensorType(
    ViSession   instrSession,
    ViInt32 sense,
    ViChar _VI_FAR  type[]
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:TYPE?", sense);
    
    return GETPARAMSTRING (instrSession,
                              buffer,
                              type,
                              1024);
}

/*===========================================================================
 * Function: Power Zero
 * Purpose:  This function activates the autozero function. Zeroing is 
 *           required in regular interval (at least once a day)and if the 
 *           temperature has varied more than about 5 °C, if the sensor has 
 *           been replaced or if measurements of signals with very low power 
 *           are to be performed. The sensor must be disconnected from all 
 *           power sources before starting the autozero function.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerZero(
    ViSession   instrSession,
    ViInt32 sense
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:ZERO", sense);
    
    return rssmb_sendCmd (instrSession, buffer);
}

/*===========================================================================
 * Function: Set Power Filter Length
 * Purpose:  This function selects the filter length for user filter mode
 *           (:SENSe<[1]...3>:POWer:FILTer:TYPE USER). As the filter length 
 *           works as a multiplier for the time window, a constant filter 
 *           length results in a constant measurement time. Values 1 and 2^n 
 *           are setable. In remote control, when using command READ to 
 *           retrieve the measurement result, the time window is fixed to 20 
 *           ms. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerFilterLength(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 filterLength
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG", sense);
    
    return SETPARAMINT32 (instrSession,
                             buffer,
                             filterLength,
                             1,65536);
}

/*===========================================================================
 * Function: Get Power Filter Length
 * Purpose:  This function selects the filter length for user filter mode. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFilterLength(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    filterLength
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG?", sense);
    
    return GETPARAMINT32 (instrSession,
                             buffer,
                             filterLength);
}

/*===========================================================================
 * Function: Set Power Filter Mode
 * Purpose:  This function selects the filter mode. The filter length is the 
 *           multiplier for the time window and thus directly influences the 
 *           measurement time. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetPowerFilterMode(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32 filterMode
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE", sense);
    
    return SETPARAMENUM (instrSession,
                          buffer,
                          filterMode,
                          powerFilterModeArr);
}

/*===========================================================================
 * Function: Get Power Filter Mode
 * Purpose:  This function queriess the filter mode for user filter mode. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFilterMode(
    ViSession   instrSession,
    ViInt32 sense,
    ViInt32*    filterMode
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:TYPE?", sense);
    
    return GETPARAMENUM (instrSession,
                          buffer,
                          filterMode,
                          powerFilterModeArr);
}

/*===========================================================================
 * Function: Get Power Filter Auto
 * Purpose:  This function returns the filter length automatically set value.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetPowerFilterLengthAuto (ViSession instrSession,
                                                  ViInt32 sense,
                                                  ViInt32 *filterLengthAuto)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:LENG:AUTO?", sense);
    
    return GETPARAMINT32 (instrSession,
                             buffer,
                             filterLengthAuto);    
}

/*===========================================================================
 * Function: Power Filter Search Once
 * Purpose:  This function activates the search for the optimum filter 
 *           length for the current measurement conditions. The found filter 
 *           length can be retrieved with command 
 *           SENSe<[1]...3>:POWer:FILTer:LENGth?. 
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerFilterSearchOnce(
    ViSession   instrSession,
    ViInt32 sense
)
{
    ViChar buffer[BUFFER_SIZE];
    
    sprintf (buffer, "SENS%ld:POW:FILT:SONC", sense);
    
    return rssmb_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: Set Image Size
 * Purpose:  This function sets the size of the hardcopy in number of pixels. 
 *           The first value of the size setting defines the width, the second 
 *           value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetImageSize (ViSession instrSession,
                                      ViInt32 imageSize)
{
    return SETPARAMENUM (instrSession,
                          "HCOP:IMAG:SIZE",
                          imageSize,
                          hcopImageSizeArr);     
}

/*============================================================================
 * Function: Get Image Size
 * Purpose:  This function returns the size of the hardcopy in number of pixels. 
 *           The first value of the size setting defines the width, the second 
 *           value the height of the image.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetImageSize (ViSession instrSession,
                                      ViInt32 *imageSize)
{
    return GETPARAMENUM (instrSession,
                          "HCOP:IMAG:SIZE?",
                          imageSize,
                          hcopImageSizeArr);         
}

/*============================================================================
 * Function: Set Output Device
 * Purpose:  This function sets the output device.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetOutputDevice (ViSession instrSession,
                                         ViInt32 outputDevice)
{
    return SETPARAMENUM (instrSession,
                          "HCOP:DEV",
                          outputDevice,
                          hcopDeviceArr);        
}

/*============================================================================
 * Function: Get Output Device
 * Purpose:  This function returns the output device.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetOutputDevice (ViSession instrSession,
                                         ViInt32 *outputDevice)
{
    return GETPARAMENUM (instrSession,
                          "HCOP:DEV?",
                          outputDevice,
                          hcopDeviceArr);               
}

/*============================================================================
 * Function: Save Hard Copy
 * Purpose:  This function triggers the generation of a hardcopy of 
 *           the display.. The data is written into the file selected/created 
 *           with the HCOP:FILE commands.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SaveHardCopy (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "HCOP:EXEC");
}

/*============================================================================
 * Function: Set Automatic Naming State
 * Purpose:  This function activates/deactivates automatic naming of 
 *           the hardcopy files.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAutomaticNamingState (ViSession instrSession,
                                                 ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:STAT",
                               state);
}

/*============================================================================
 * Function: Get Automatic Naming State
 * Purpose:  This function returns state of automatic naming of the hardcopy 
 *           files.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticNamingState (ViSession instrSession,
                                                 ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:STAT?",
                               state);    
}

/*============================================================================
 * Function: Get Automatic File Name
 * Purpose:  This function returns the file name generated using 
 *           the automatic naming settings. As default the automatically 
 *           generated file name is composed of:
 *           "<Path>/<Prefix><YYYY><MM><DD><Number>.<Format>". Each component 
 *           can be deactivated/activated separately to individually design 
 *           the file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFileName (ViSession instrSession,
                                              ViChar fileName[])
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:FILE?",
                              fileName,
                              1024);   
}

/*============================================================================
 * Function: Get Automatic File Path
 * Purpose:  This function returns the file name generated using 
 *           the automatic naming settings. As default the automatically 
 *           generated file name is composed of:
 *           "<Path>/<Prefix><YYYY><MM><DD><Number>.<Format>". Each component 
 *           can be deactivated/activated separately to individually design 
 *           the file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFilePath (ViSession instrSession,
                                              ViInt32 arraySize,
                                              ViChar filePath[])
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO?",
                              filePath,
                              arraySize);   
}

/*============================================================================
 * Function: Set Automatic Directory
 * Purpose:  This function defines the directory into which the hardcopy files 
 *           will be stored if auto naming is activated 
 *           (HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAutomaticDirectory (ViSession instrSession,
                                               ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:AUTO:DIR",
                              "'",
                              directoryName);     
}

/*============================================================================
 * Function: Get Automatic Directory
 * Purpose:  This function returns the directory into which the hardcopy files 
 *           will be stored if auto naming is activated 
 *           (HCOP:FILE:AUTO:STAT ON).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticDirectory (ViSession instrSession,
                                               ViChar directory[])
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:DIR?",
                              directory,
                              1024);      
}

/*============================================================================
 * Function: Delete Image Files
 * Purpose:  This function deletes all files with extensions "bmp", "img", 
 *           "png" and "xpm" in the directory set for automatic naming.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_DeleteImageFiles (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "HCOP:FILE:AUTO:DIR:CLE");    
}

/*============================================================================
 * Function: Set Automatic File Date State
 * Purpose:  This function activates/deactivates the usage of the day/month/year 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAutomaticFileDateState (ViSession instrSession,
                                                   ViInt32 fileNamePart,
                                                   ViBoolean state)
{
    INIT;
    
    CHECKVIINT32(instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMB_HCOPY_AUTO_FILE_DAY:
            rssmb_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:DAY:STAT",
                               state);        
        break;
        case RSSMB_HCOPY_AUTO_FILE_MONTH:
            rssmb_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:MONT:STAT",
                               state);
        break;                         
        case RSSMB_HCOPY_AUTO_FILE_YEAR:
            rssmb_status = SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:YEAR:STAT",
                               state);
        break;
    }
    return rssmb_status;
}

/*============================================================================
 * Function: Get Automatic File Date State
 * Purpose:  This function returns the state of usage of the day/month/year in 
 *           the automatic file name
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFileDateState (ViSession instrSession,
                                                   ViInt32 fileNamePart,
                                                   ViBoolean *state)
{
    INIT;
    
    CHECKVIINT32(instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMB_HCOPY_AUTO_FILE_DAY:
            rssmb_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:DAY:STAT?",
                               state);        
        break;
        case RSSMB_HCOPY_AUTO_FILE_MONTH:
            rssmb_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:MONT:STAT?",
                               state);
        break;                         
        case RSSMB_HCOPY_AUTO_FILE_YEAR:
            rssmb_status = GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:YEAR:STAT?",
                               state);
        break;
    }
    return rssmb_status;     
}

/*============================================================================
 * Function: Set Automatic File Date State
 * Purpose:  This function queries the day/month/year part in the automatic 
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFileDate (ViSession instrSession,
                                              ViInt32 fileNamePart,
                                              ViChar part[])
{
    INIT;
    
    CHECKVIINT32(instrSession, fileNamePart, 0, 2);
    
    switch (fileNamePart){
        case RSSMB_HCOPY_AUTO_FILE_DAY:
            rssmb_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:DAY?",
                              part,
                              1024);         
        break;
        case RSSMB_HCOPY_AUTO_FILE_MONTH:
            rssmb_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:MONT?",
                              part,
                              1024);  
        break;                         
        case RSSMB_HCOPY_AUTO_FILE_YEAR:
            rssmb_status = GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:YEAR?",
                              part,
                              1024);  
        break;
    } 
    return rssmb_status; 
}

/*============================================================================
 * Function: Set Automatic File Prefix State
 * Purpose:  This function activates/deactivates the usage of the prefix 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAutomaticFilePrefixState (ViSession instrSession,
                                                     ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:PREF:STAT",
                               state);
}

/*============================================================================
 * Function: Get Automatic File Prefix State
 * Purpose:  This function returns the state of the usage of the prefix 
 *           in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFilePrefixState (ViSession instrSession,
                                                     ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "HCOP:FILE:AUTO:PREF:STAT?",
                               state);    
}

/*============================================================================
 * Function: Set Automatic File Prefix
 * Purpose:  This function defines the prefix part in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetAutomaticFilePrefix (ViSession instrSession,
                                                ViString prefix)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:AUTO:PREF",
                              "'",
                              prefix);     
}

/*============================================================================
 * Function: Get Automatic File Prefix
 * Purpose:  This function returns the prefix part in the automatic file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFilePrefix (ViSession instrSession,
                                                ViChar fileName[])
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:AUTO:PREF?",
                              fileName,
                              1024);     
}

/*============================================================================
 * Function: Get Automatic File Number
 * Purpose:  This function returns queries the generated number in the automatic
 *           file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetAutomaticFileNumber (ViSession instrSession,
                                                ViInt32 *number)
{
    return GETPARAMINT32 (instrSession,
                              "HCOP:FILE:AUTO:NUMB?",
                              number);        
}

/*============================================================================
 * Function: rssmb_ClearStatus
 * Purpose:  CLEAR STATUS sets the status byte (STB), the standard event
 *           register (ESR), and the EVENt part of the QUEStionable and
 *           OPERation register to zero. The function does not change the
 *           enable and transition parts of the registers. It deletes the
 *           output buffer.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ClearStatus (ViSession instrSession)
{
    return rssmb_sendCmd ( instrSession, "*CLS" );
}

/*============================================================================
 * Function: rssmb_EventStatusEnable
 * Purpose:  EVENT STATUS ENABLE sets the event status enable register to the
 *           specified value. The *ESE? query command returns the content of
 *           the event status enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_EventStatusEnable (ViSession instrSession,
                                           ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*ESE",
                             registerValue,
                             0, 255);
}

/*============================================================================
 * Function: rssmb_EventStatusQuery
 * Purpose:  STANDARD EVENT STATUS QUERY returns the content of the event
 *           status register in decimal form (0...255) and then sets the
 *           register to zero.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_EventStatusQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*ESR?",
                             registerValue);
}

/*============================================================================
 * Function: rssmb_IdentificationQuery
 * Purpose:  IDENTIFICATION QUERY queries the instrument identification. The
 *           instrument type, serial number and firmware version are returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_IdentificationQuery (ViSession instrSession,
                                             ViChar _VI_FAR instrumentIdentification[])
{
    ViStatus rssmb_status = VI_SUCCESS;

    rssmb_status = GETPARAMSTRING (instrSession,
                              "*IDN?",
                              instrumentIdentification,
                              256);

    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*============================================================================
 * Function: rssmb_IndividualStatusQuery
 * Purpose:  INDIVIDUAL STATUS QUERY returns the content of the IST flag in
 *           decimal form (0 | 1). The IST flag is the status bit which is sent
 *           during a parallel poll.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_IndividualStatusQuery (ViSession instrSession,
                                      ViInt32 *ISTFlag)
{
    return GETPARAMINT32 (instrSession,
                             "*IST?",
                             ISTFlag);
}

/*============================================================================
 * Function: rssmb_OperationComplete
 * Purpose:  OPERATION COMPLETE sets bit 0 in the event status register when
 *           all previous commands have been processed. This bit can be used to
 *           trigger a service request.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_OperationComplete (ViSession instrSession)
{
    ViStatus            rssmb_status;
    rssmb_instrRange    instrPtr = NULL;
    ViBoolean           err_check;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    CHECKNULL (instrPtr);

    err_check = instrPtr -> errorChecking;
    
    CHECKERR (rssmb_errorCheckState (instrSession, VI_FALSE));
    rssmb_status = rssmb_sendCmd (instrSession, "*OPC");
    (void) rssmb_errorCheckState (instrSession, err_check);
    
    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_OperationCompleteQuery
 * Purpose:  OPERATION COMPLETE QUERY returns a 1 when all previous commands
 *           have been processed. It is important to ensure that the timeout
 *           set at the IEC/IEEE bus is long enough.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_OperationCompleteQuery (ViSession instrSession,
                                       ViInt32 *OPCState)
{
    return GETPARAMINT32 (instrSession,
                             "*OPC?",
                             OPCState);
}

/*============================================================================
 * Function: rssmb_OptionIdentificationQuery
 * Purpose:  OPTION IDENTIFICATION QUERY queries the options configured in the
 *           instrument and returns a list of the installed options. The
 *           options are separated by commas.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_OptionIdentificationQuery (ViSession instrSession,
                                          ViChar _VI_FAR optionIdentification[])
{
    return GETPARAMSTRING (instrSession,
                              "*OPT?",
                              optionIdentification,
                              1024);
}

/*============================================================================
 * Function: rssmb_ParallelPollRegisterEnable
 * Purpose:  PARALLEL POLL REGISTER ENABLE sets the parallel poll enable
 *           register to the specified value. The *PRE? query command returns
 *           the content of the parallel poll enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ParallelPollRegisterEnable (ViSession instrSession,
                                           ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*PRE",
                             registerValue,
                             0, 655);
}

/*============================================================================
 * Function: rssmb_ParallelPollRegisterQuery
 * Purpose:  PARALLEL POLL REGISTER QUERY returns the content of the parallel
 *           poll enable register in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ParallelPollRegisterQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*PRE?",
                             registerValue);
}

/*============================================================================
 * Function: rssmb_PowerOnStatusClear
 * Purpose:  POWER ON STATUS CLEAR determines whether the content of the
 *           ENABle registers are retained or reset at power on.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerOnStatusClear (ViSession instrSession,
                                   ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*PSC",
                             registerValue,
                             0, 1);
}

/*============================================================================
 * Function: rssmb_PowerOnStatusClearQuery
 * Purpose:  POWER ON STATUS CLEAR QUERY retrieves the contents of the
 *           Power-on-Status-Clear flag. The answer is either 0 or 1.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_PowerOnStatusClearQuery (ViSession instrSession,
                                        ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*PSC?",
                             registerValue);
}

/*============================================================================
 * Function: rssmb_Recall
 * Purpose:  RECALL calls up the instrument status which was stored under the
 *           specified number using the *SAV command or using the
 *           MMEMory:STORe:STATe command. "0" corresponds to the power-on
 *           state, unless a new instrument setting was stored under this
 *           number in the meantime using the MMEMory:STORe:STATe command. This
 *           function can also be used to call up the 3 intermediate instrument
 *           states which are stored with *SAV.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_Recall (ViSession instrSession, ViInt32 instrumentStatus)
{
    return SETPARAMINT32 (instrSession,
                             "*RCL",
                             instrumentStatus,
                             0, 10);
}

/*============================================================================
 * Function: rssmb_Save
 * Purpose:  SAVE stores the current device state under the specified number
 *           (see also *RCL). This function is used to store the 3 intermediate
 *           instrument states and to store the current instrument state
 *           internally when the instrument is switched off. This state is then
 *           restored at power on.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_Save (ViSession instrSession, ViInt32 instrumentStatus)
{
    return SETPARAMINT32 (instrSession,
                             "*SAV",
                             instrumentStatus,
                             0, 10);
}

/*============================================================================
 * Function: rssmb_ServiceRequestEnable
 * Purpose:  SERVICE REQUEST ENABLE sets the service request enable register
 *           to the specified value. Bit 6 (MSS enable bit) remains 0. This
 *           function determines the conditions under which a service request
 *           is triggered. The *SRE? query command reads out the content of the
 *           service request enable register in decimal form. Bit 6 is always
 *           0.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ServiceRequestEnable (ViSession instrSession,
                                     ViInt32 registerValue)
{
    return SETPARAMINT32 (instrSession,
                             "*SRE",
                             registerValue,
                             0, 255);
}

/*============================================================================
 * Function: rssmb_ServiceRequestEnableQuery
 * Purpose:  SERVICE REQUEST ENABLE reads out the content of the service
 *           request enable register in decimal form. Bit 6 is always 0.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_ServiceRequestEnableQuery (ViSession instrSession,
                                          ViInt32 *registerValue)
{
    return GETPARAMINT32 (instrSession,
                             "*SRE?",
                             registerValue);
}

/*============================================================================
 * Function: rssmb_StatusByteQuery
 * Purpose:  READ STATUS BYTE QUERY reads out the content of the status byte
 *           in decimal form.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_StatusByteQuery (ViSession instrSession, ViInt32 *statusByte)
{
    return GETPARAMINT32 (instrSession,
                             "*STB?",
                             statusByte);
}

/*============================================================================
 * Function: rssmb_Trigger
 * Purpose:  TRIGGER triggers all actions which are waiting for a trigger
 *           event. Specific trigger events can be triggered by means of the
 *           "TRIGger" command system (sweep and lists) or the SOURce
 *           subsystems (baseband).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_Trigger (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "*TRG");
}

/*===========================================================================*/
/* Function: Send Trigger and Wait for OPC                                   */
/* Purpose:  This function triggers all actions waiting for a trigger event  */
/*           in the currently active window and waits for operation completed*/
/*           (OPC) before returning the status code.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_TriggerWaitOPC (ViSession instrSession, ViInt32 timeout)
{
    ViStatus rssmb_status = VI_SUCCESS;
    ViChar buffer[BUFFER_SIZE] = "";

    CHECKVIINT32(instrSession, timeout, 0, 600000);

    CHECKERR (rssmb_waitOPC (instrSession, "*TRG;*OPC?", buffer, timeout));

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_SelfTestQuery
 * Purpose:  SELF TEST QUERY triggers all self tests of the instrument and
 *           outputs an rssmb_status code in decimal form (see Service Manual supplied
 *           with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SelfTestQuery (ViSession instrSession, ViInt32 *errorCode)
{
    return GETPARAMINT32 (instrSession,
                             "*TST?",
                             errorCode);
}

/*============================================================================
 * Function: rssmb_WaittoContinue
 * Purpose:  WAIT-to-CONTINUE does not allow subsequent commands to be
 *           processed until all previous commands have been executed and all
 *           signals are in their transient condition.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_WaittoContinue (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "*WAI");
}

/*===========================================================================*/
/* Function: Preset Signal Path
/* Purpose:  This function presets all parameters which are related to the 
/*           signal path.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_PresetSignalPath(
    ViSession   instrSession
)
{
    return rssmb_sendCmd (instrSession, "SOUR:PRES");
}

/*============================================================================
 * Function: rssmb_CALibrationAll
 * Purpose:  This function starts all internal adjustments for which no
 *           external measuring equipment is needed. With two-path instruments,
 *           adjustment is performed for both paths.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationAll (ViSession instrSession, ViInt32 timeout,
                                        ViInt32 *response)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:ALL?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: CALibration Clock Synthesis
/* Purpose:  This function performs all adjustments which affect the clock 
/*           synthesis.
/*           
/*           Note:
/*           
/*           If the calibration fails, this function returns an error code.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationClockSynthesis(
    ViSession   instrSession,
    ViInt32 timeout,
    ViInt32*    response
)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:CSYN:MEAS?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_CALibrationFMOffset
 * Purpose:  This function starts all adjustment for the FM/PhiM modulator.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationFMOffset (ViSession instrSession, ViInt32 timeout,
                                                   ViInt32   *response)
{
    ViChar      buffer[BUFFER_SIZE];

    INIT;
    
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:FMOF?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_CALibrationFrequency
 * Purpose:  This function starts all adjustments which affect the frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationFrequency (ViSession instrSession,
                                              ViInt32 timeout,
                                              ViInt32 *response)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:FREQ:MEAS?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_CALibrationModulationGenerators
 * Purpose:  This function performs all adjustments which affect the internal
 *           modulation generators.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationModulationGenerators (ViSession instrSession,
                                                         ViInt32 timeout,
                                                         ViInt32 *response)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:LFO?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_CALibrationModulationGenerators
 * Purpose:  This function performs all adjustments which affect the analog
 *           channels of the stereo coder.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationStereoCoder (ViSession instrSession,
                                       ViInt32 timeout, ViInt32 *response)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:STER:ANAL?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_CALibrationLevel
 * Purpose:  This function starts all adjustments which affect the level.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_CALibrationLevel (ViSession instrSession,
                                          ViInt32 timeout,
                                          ViInt32 *response)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE];

    CHECKVIINT32 (instrSession, timeout, 0, 600000);
    CHECKNULL(response);

    sprintf (buffer, "CAL:LEV:MEAS?;*OPC?");

    CHECKERR (rssmb_waitOPC (instrSession, buffer, buffer, timeout));

    if (sscanf (buffer, "%ld", response) != 1)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    if (*response == 1) /* Calibration failed. */
        rssmb_status = RSSMB_ERROR_CALIBRATION_FAILED;

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_SetCALibrationLevelState
 * Purpose:  This function switches on or off use of external level
 *           correction data. This switch can be used in case of corrupt
 *           external level correction data.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetCALibrationLevelState(ViSession instrSession,
                                                 ViBoolean state)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "CAL:LEV:STAT",
                               state);
}

/*============================================================================
 * Function: rssmb_GetCALibrationLevelState
 * Purpose:  This function returns state of external level correction data.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetCALibrationLevelState(ViSession   instrSession,
                                                 ViBoolean*  state)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "CAL:LEV:STAT?",
                               state);
}

/*============================================================================
 * Function: Set CALibration Level Adjustment Data
 * Purpose:  This function selects the data used for external level correction. 
 *           By default the instrument uses correction data obtained in 
 *           the factory before delivery. In addition, customer data can be 
 *           used for external level correction. The customer data is obtained 
 *           using an R&S NRP power sensor.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetCALibrationLevelAdjustmentData (ViSession instrSession,
                                                           ViInt32 adjustmentData)
{
    INIT;
    
    return SETPARAMENUM (instrSession,
                          "CAL:LEV:EXT:DATA",
                          adjustmentData,
                          calLevelAdjDataArr);    
}

/*============================================================================
 * Function: Get CALibration Level Adjustment Data (*N)
 * Purpose:  This function returns the data used for external level correction.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetCALibrationLevelAdjustmentData (ViSession instrSession,
                                                           ViInt32 *adjustmentData)
{
    INIT;
    
    return GETPARAMENUM (instrSession,
                          "CAL:LEV:EXT:DATA?",
                          adjustmentData,
                          calLevelAdjDataArr);    
}

/*===========================================================================*/
/* Function: Set Image File Name
/* Purpose:  This function creates/selects a file into which the hardcopy 
/*           will be stored after the HCOP:EXEC command is sent. The 
/*           directory is either defined with the command MMEMory:CDIR or the 
/*           path is specified together with the file name. Access via remote 
/*           control is possible using the commands of the MMEM-Subsystem.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetImageFileName(
    ViSession   instrSession,
    ViString    fileName
)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "HCOP:FILE:NAME",
                              "'",
                              fileName);
}

/*===========================================================================*/
/* Function: Get Image File Name
/* Purpose:  This function returns a file name where the hardcopy will be 
/*           stored after the HCOP:EXEC command is sent. The directory is 
/*           either defined with the command MMEMory:CDIR or the path is 
/*           specified together with the file name. Access via remote control 
/*           is possible using the commands of the MMEM-Subsystem.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetImageFileName(
    ViSession   instrSession,
    ViChar _VI_FAR  fileName[]
)
{
    return GETPARAMSTRING (instrSession,
                              "HCOP:FILE:NAME?",
                              fileName,
                              1024);
}

/*===========================================================================*/
/* Function: Set Image Format
/* Purpose:  This function sets the image format in which the hard copy 
/*           should be stored.
/*           
/*           Example(s):
/*           HCOP:DEV:LANG BMP
/*             - 'selects bmp as imge format.
/*           HCOP:FILE 'C:/TEMP/snapshot001'
/*             - 'defines directory, path and file for storing the hard 
/*           copy.
/*           HCOP
/*             - 'executees the hard copy procedure.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_SetImageFormat(
    ViSession   instrSession,
    ViInt32 imageFormat
)
{
    return SETPARAMENUM (instrSession,
                          "HCOP:DEV:LANG",
                          imageFormat,
                          imageFormatArr);
}

/*===========================================================================*/
/* Function: Get Image Format
/* Purpose:  This function returns the image format in which the hard copy 
/*           should be stored.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetImageFormat(
    ViSession   instrSession,
    ViInt32*    imageFormat
)
{
    return GETPARAMENUM (instrSession,
                          "HCOP:DEV:LANG?",
                          imageFormat,
                          imageFormatArr);
}

/*===========================================================================*/
/* Function: Get Image Snapshot
/* Purpose:  This function transfers the hardcopy (snapshot) as a NByte 
/*           stream to the remote client. 
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetImageSnapshot (ViSession instrSession,
                                          ViString destination)
{
    return rssmb_readDataToFile (instrSession, "HCOP:DATA?", destination); 
}

/*============================================================================
 * Function: rssmb_DIAGnosticModules
 * Purpose:  This function checks the assemblies available in the instrument
 *           using the variant and revision state.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_DIAGnosticModules (ViSession instrSession, 
                                           ViString moduleName, 
                                           ViInt32 arraySize,
                                           ViChar _VI_FAR moduleConfiguration[])
{
    ViChar      buffer [BUFFER_SIZE];

    CHECKNULL (moduleConfiguration);

    if (!moduleName)
        sprintf (buffer, "DIAG:BGIN?;*OPC?");
    else
        sprintf (buffer, "DIAG:BGIN? '%s';*OPC?", moduleName);

    return rssmb_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             moduleConfiguration,"moduleConfiguration");
}

/*============================================================================
 * Function: rssmb_DIAGnosticModuleNames
 * Purpose:  This function queries the names of the assemblies available in
 *           the instrument.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_DIAGnosticModuleNames (ViSession instrSession, 
                                               ViInt32 arraySize, 
                                               ViChar _VI_FAR moduleNames[])
{
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "DIAG:BGIN:CAT?;*OPC?");

    return rssmb_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             moduleNames,"moduleNames");
}

/*============================================================================
 * Function: rssmb_DIAGnosticTestPoints
 * Purpose:  This function queries the test points available in the
 *           instrument. A detailed description of the test points can be found
 *           in Chapter 3 of the Service Manual (supplied with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_DIAGnosticTestPoints (ViSession instrSession, 
                                              ViInt32 arraySize, 
                                              ViChar _VI_FAR testPoints[])
{
    ViChar  buffer [BUFFER_SIZE];

    sprintf (buffer, "DIAG:POIN:CAT?;*OPC?");

    return rssmb_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             testPoints,"testPoints");
}

/*============================================================================
 * Function: rssmb_DIAGnosticTestPointVoltages
 * Purpose:  This function triggers voltage measurement at the specified test
 *           point and returns the measured voltage. A detailed description of
 *           the test points can be found in Chapter 3 of the Service Manual
 *           (supplied with the instrument).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_DIAGnosticTestPointVoltages (ViSession instrSession,
                                                     ViString testPointName,
                                                     ViReal64 *measuredVoltage)
{
    ViChar      buffer [BUFFER_SIZE];

    CHECKNULL (testPointName);

    sprintf (buffer, "DIAG:POIN? '%s'", testPointName);

    return GETPARAMREAL64 (instrSession,
                              buffer,
                              measuredVoltage);
}

/*****************************************************************************
 * Function: DIAGnostic Operation Time
 * Purpose:  This function queries the number of operation hours.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_DIAGnosticOperationTime(
    ViSession   instrSession,
    ViUInt32*   operationTime
)
{
    INIT;

    return GETPARAMUINT32 (instrSession,
                              "DIAG:INFO:OTIM?",
                              operationTime);
}

/*****************************************************************************
 * Function: DIAGnostic Power On Events
 * Purpose:  This function queries the number of power-on events.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_DIAGnosticPowerOnEvents(
    ViSession   instrSession,
    ViUInt32*   powerOnEvents
)
{
    INIT;

    return GETPARAMUINT32 (instrSession,
                              "DIAG:INFO:POC?",
                              powerOnEvents);
}

/*============================================================================
 * Function: rssmb_TESTDirect
 * Purpose:  The respective hardware assembly responds directly to this
 *           function; any safety mechanisms are bypassed.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TESTDirect (ViSession instrSession, 
                                    ViInt32 command, 
                                    ViChar _VI_FAR assemblyResponse[])
{
    ViChar  buffer [BUFFER_SIZE];

    CHECKVIINT32(instrSession, command, 0, 7);

    sprintf (buffer, "TEST:DIR? '%s',0", testCommandsArr[command]);

    return GETPARAMSTRING (instrSession,
                              buffer,
                              assemblyResponse,
                              256);
}

/*****************************************************************************
 * Function: TEST All Start
 * Purpose:  This function performs a selftest on all installed hardware 
 *           options.
 *           
 *           Remote-control command(s):
 *           TEST<hw>:ALL:STARt
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_TESTAllStart(
	ViSession	instrSession
)
{
    return rssmb_sendCmd (instrSession, "TEST:ALL:STAR");
}

/*============================================================================
 * Function: rssmb_TESTAllResult
 * Purpose:  This function performs a selftest on all installed hardware
 *			 options.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_TESTAllResult (ViSession instrSession,
									   ViChar _VI_FAR result[])
{
    INIT;

    return GETPARAMSTRING (instrSession, 
							  "TEST:ALL:RES?",
							  result,
							  512);
}

/*===========================================================================
 * Function: Set SYSTem Keyboard Layout
 * Purpose:  This function selects the keyboard layout.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetKeyboardLayout(
    ViSession   instrSession,
    ViInt32 keyboardLayout
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = SETPARAMENUM (instrSession,
                                  "KBO:LAY",
                                  keyboardLayout,
                                  keyboardArr);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
    
}

/*===========================================================================
 * Function: Get SYSTem Keyboard Layout
 * Purpose:  This function displays the keyboard layout.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetKeyboardLayout(
    ViSession   instrSession,
    ViInt32*    keyboardLayout
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = GETPARAMENUM (instrSession,
                                  "KBO:LAY?",
                                  keyboardLayout,
                                  keyboardArr);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
    
}

/*===========================================================================
 * Function: Set SYSTem Keyboard Language
 * Purpose:  This function selects the keyboard language.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetKeyboardLanguage(
    ViSession   instrSession,
    ViInt32 keyboardLanguage
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = SETPARAMENUM (instrSession,
                                  "KBO:LANG",
                                  keyboardLanguage,
                                  keyboardArr);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
    
}

/*===========================================================================
 * Function: Get SYSTem Keyboard Language
 * Purpose:  This function displays the keyboard language.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetKeyboardLanguage(
    ViSession   instrSession,
    ViInt32*    keyboardLanguage
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = GETPARAMENUM (instrSession,
                                  "KBO:LANG?",
                                  keyboardLanguage,
                                  keyboardArr);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
    
}

/*===========================================================================
 * Function: Set Display Power Save State
 * Purpose:  This function sets the display power save state. With the 
 *           screen-save mode activated the display including backlight is 
 *           completely switched off after the elapse of the wait time (see 
 *           command DISPlay:PSAVe:HOLDoff) when no entries via front panel, 
 *           external mouse or external keyboard are made.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetDisplayPowerSaveState(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = SETPARAMBOOL (instrSession,
                                       "DISP:PSAV:STAT",
                                       state);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Get Display Power Save State
 * Purpose:  This function returns the state of display power save.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetDisplayPowerSaveState(
    ViSession   instrSession,
    ViBoolean*  state
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = GETPARAMBOOL (instrSession,
                                       "DISP:PSAV:STAT?",
                                       state);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Set Display Power Save Holdoff
 * Purpose:  This function sets the display power save holdoff.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetDisplayPowerSaveHoldoff(
    ViSession   instrSession,
    ViInt32 holdoff
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = SETPARAMINT32 (instrSession,
                                     "DISP:PSAV:HOLD",
                                     holdoff,
                                     1, 60);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Get Display Power Save Holdoff
 * Purpose:  This function returns the state of display power save holdoff.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetDisplayPowerSaveHoldoff(
    ViSession   instrSession,
    ViInt32*    holdoff
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    rssmb_status = GETPARAMINT32 (instrSession,
                                     "DISP:PSAV:HOLD?",
                                     holdoff);
    
    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*===========================================================================
 * Function: Set Display Update State
 * Purpose:  This function switches the update of the display on/off
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetDisplayUpdateState (ViSession instrSession,
                                               ViBoolean state)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:DISP:UPD",
                               state);
}

/*===========================================================================
 * Function: Get Display Update State
 * Purpose:  This function queries the state of update of the display on/off.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetDisplayUpdateState (ViSession instrSession,
                                               ViBoolean *state)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:DISP:UPD?",
                               state);
}

/*===========================================================================
 * Function: Set Display Blank Mode
 * Purpose:  This function sets the display blank mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetDisplayBlankMode(
    ViSession   instrSession,
    ViBoolean blankMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return SETPARAMBOOL (instrSession,
                               "OUTP:BLAN:LIST:STAT",
                                  blankMode);
}

/*===========================================================================
 * Function: Get Display Blank Mode
 * Purpose:  This function returns the display blank mode.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetDisplayBlankMode(
    ViSession   instrSession,
    ViBoolean*    blankMode
)
{
    ViStatus rssmb_status = VI_SUCCESS;
    
    CHECKERR (rssmb_checkInstrument (instrSession, SMB, VI_FALSE));
    return GETPARAMBOOL (instrSession,
                               "OUTP:BLAN:LIST:STAT?",
                                  blankMode);
}

/*============================================================================
 * Function: rssmb_MMEMoryCatalog
 * Purpose:  This function reads out the subdirectories and files in the
 *           specified directory. If no directory is specified, the default
 *           directory selected with the MMEM:CDIR command is read out on the
 *           default drive selected with the MMEM:MSIS command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryCatalog (ViSession instrSession,
                               ViString directoryPath, ViInt32 arraySize,
                               ViChar _VI_FAR catalogList[])
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:CAT?;*OPC?");
    else
        sprintf (buffer, "MMEM:CAT? '%s';*OPC?", directoryPath);

    return rssmb_getCatalog (instrSession,
                             buffer,
                             (ViUInt32) arraySize,
                             catalogList,"catalogList");
}

/*============================================================================
 * Function: rssmb_MMEMoryCatalogLength
 * Purpose:  This function reads out the number of files in the specified
 *           directory. If no directory is specified, the default directory
 *           selected with the MMEM:CDIR command is read out on the default
 *           drive selected with the MMEM:MSIS command.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryCatalogLength (ViSession instrSession,
                                     ViString directoryPath,
                                     ViInt32 *catalogLength)
{
    ViChar  buffer [BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:CAT:LENG?");
    else
        sprintf (buffer, "MMEM:CAT:LENG? '%s'", directoryPath);

    return GETPARAMINT32 (instrSession,
                             buffer,
                             catalogLength);
}

/*============================================================================
 * Function: rssmb_MMEMoryChangeDirectory
 * Purpose:  This function changes the default directory. This directory is
 *           used for all subsequent MMEM commands if no path is specified with
 *           them. It is also possible to change to a higher directory using
 *           two dots '..'.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryChangeDirectory (ViSession instrSession,
                                       ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                            "MMEM:CDIR",
                            "'",
                            directoryName);
}

/*============================================================================
 * Function: rssmb_MMEMoryCopy
 * Purpose:  This function copies the first specified file to the second
 *           specified file. Instead of just a file, this function can also be
 *           used to copy a complete directory together with all its files.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryCopy (ViSession instrSession, ViString sourcePath,
                            ViString destinationPath)
{
    ViStatus    rssmb_status;
    ViChar      buffer[BUFFER_SIZE];

    CHECKNULL (sourcePath);

    if (destinationPath)
        sprintf (buffer, "MMEM:COPY '%s','%s'", sourcePath, destinationPath);
    else
        sprintf (buffer, "MMEM:COPY '%s'", sourcePath);

    rssmb_status = rssmb_sendCmd (instrSession, buffer);

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_MMEMoryBlockDataWrite
 * Purpose:  This function writes the block data <binary block data> to the
 *           file identified by <file_name>.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryBlockDataWrite (ViSession instrSession,
                                      ViString fileName,
                                      ViInt32 binaryBlockLength,
                                      ViChar _VI_FAR binaryBlockData[])
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViChar*     pbuffer;
    ViChar*     p2buf;

    CHECKNULL (fileName);
    CHECKNULL (binaryBlockData);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    p2buf = pbuffer = malloc (strlen (fileName) + binaryBlockLength + 1000);

    p2buf += sprintf (pbuffer, "MMEM:DATA '%s',#%ld%ld",
                               fileName,
                               (ViInt32)(floor(log10((double)binaryBlockLength)) + 1),
                               binaryBlockLength);

    memcpy (p2buf, binaryBlockData, (size_t) binaryBlockLength);
//    p2buf[binaryBlockLength] = '\n';

    rssmb_status = viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) ((p2buf - pbuffer) + binaryBlockLength), VI_NULL);

    if (pbuffer)
        free (pbuffer);

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL));

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_MMEMoryBlockDataRead
 * Purpose:  This function transfers the specified file from the R&S SMA to
 *           the IEC/IEEE bus and then on to the control computer. It is
 *           important to ensure that the intermediate memory on the control
 *           computer is large enough to take the file. In this case, the
 *           setting for the IEC/IEEE-bus terminator is irrelevant.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryBlockDataRead (ViSession instrSession,
                                              ViString fileName,
                                              ViInt32 binaryBlockLength,
                                              ViChar _VI_FAR binaryBlockData[])
{
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;

    CHECKNULL (fileName);

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, "MMEM:DATA? '%s'\n", fileName);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssmb_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssmb_status = viWrite (instrSession, (ViBuf) "SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssmb_status;
            }

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssmb_status;
        }

    if (count > (ViUInt32) binaryBlockLength)
        {
        rssmb_status = RSSMB_WARNING_SMALL_BUFFER;
        count = (ViUInt32) binaryBlockLength;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
        }
    else
        {
        rssmb_status = VI_SUCCESS;
        memcpy (binaryBlockData, p2buffer, (size_t) count);
        }

    free (pbuffer);

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_MMEMoryDirectoryCatalog
 * Purpose:  This function reads out the subdirectories of the specified
 *           directory. If no directory is specified, the default directory
 *           selected with the MMEM:CDIR command is read out. The directories
 *           are output in a list (the list entries are separated by commas).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryDirectoryCatalog (ViSession instrSession,
                                        ViString directoryPath,
                                        ViInt32 arraySize, ViChar _VI_FAR catalogList[])
{
    ViStatus    rssmb_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:DCAT?;*OPC?");
    else
        sprintf (buffer, "MMEM:DCAT? '%s';*OPC?", directoryPath);

    rssmb_status = rssmb_getCatalog (instrSession,
                                     buffer,
                                     (ViUInt32) arraySize,
                                     catalogList,"catalogList");

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_MMEMoryDirectoryCatalogLength
 * Purpose:  This function reads out the number of subdirectories in the
 *           specified directory. If no directory is specified, the directory
 *           selected with the MMEM:CDIR command is read out.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryDirectoryCatalogLength (ViSession instrSession,
                                              ViString directoryPath,
                                              ViInt32 *catalogLength)
{
    ViStatus    rssmb_status;
    ViChar      buffer[BUFFER_SIZE];

    if (!directoryPath)
        sprintf (buffer, "MMEM:DCAT:LENG?");
    else
        sprintf (buffer, "MMEM:DCAT:LENG? '%s'", directoryPath);

    rssmb_status = GETPARAMINT32 (instrSession,
                                     buffer,
                                     catalogLength);

    return rssmb_status;
}

/*============================================================================
 * Function: rssmb_MMEMoryDelete
 * Purpose:  This function deletes the specified file.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryDelete (ViSession instrSession, ViString fileName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:DEL",
                              "'",
                              fileName);
}

/*============================================================================
 * Function: rssmb_MMEMoryLoad
 * Purpose:  This function loads the specified file stored under the specified
 *           name in an internal memory. If 0 is used, the instrument setting
 *           of the selected file is set directly in the R&S SMA. The *RCL
 *           command is used to load the intermediate instrument setting (see
 *           the section Common Functions (Commands)).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryLoad (ViSession instrSession,
                            ViInt32 instrumentSetting, ViString fileName)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKVIINT32(instrSession, instrumentSetting, 0, 10);
    CHECKNULL (fileName);

    sprintf (buffer, "MMEM:LOAD:STAT %ld,'%s'", instrumentSetting, fileName);

    return rssmb_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssmb_MMEMoryMakeDirectory
 * Purpose:  This function creates a new subdirectory in the specified
 *           directory. If no directory is specified, a subdirectory is created
 *           in the default directory. This function can also be used to create
 *           a directory tree.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryMakeDirectory (ViSession instrSession,
                                     ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:MDIR",
                              "'",
                              directoryName);
}

/*============================================================================
 * Function: rssmb_MMEMoryMove
 * Purpose:  This function renames an existing file ('File Source') if no path
 *           is specified for 'File Destination'. Otherwise the file is moved
 *           to the specified path and stored under the original file name or,
 *           if specified, a new file name.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryMove (ViSession instrSession, ViString fileSource,
                            ViString fileDestination)
{
    ViStatus    rssmb_status;
    ViChar*     pbuffer;

    CHECKNULL (fileSource);
    CHECKNULL (fileDestination);

    pbuffer = malloc (strlen (fileSource) + strlen (fileDestination) + 20);

    sprintf (pbuffer, "MMEM:MOVE '%s','%s'", fileSource, fileDestination);

    rssmb_status = rssmb_sendCmd (instrSession, pbuffer);

    free (pbuffer);

    RETURN;
}

/*============================================================================
 * Function: rssmb_MMEMoryRemoveDirectory
 * Purpose:  This function deletes the specified subdirectory in the specified
 *           directory. If no directory is specified, the subdirectory with the
 *           specified name is deleted in the default directory.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryRemoveDirectory (ViSession instrSession,
                                       ViString directoryName)
{
    return SETPARAMSTRINGQUOTED (instrSession,
                              "MMEM:RDIR",
                              "'",
                              directoryName);
}

/*============================================================================
 * Function: rssmb_MMEMoryStore
 * Purpose:  This function stores the current instrument setting in the
 *           specified file. If 0 is specified, the current instrument setting
 *           is stored directly in the specified file. The intermediate
 *           instrument settings can be stored using the *SAV command (see the
 *           section Common Functions (Commands)).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_MMEMoryStore (ViSession instrSession,
                             ViInt32 instrumentSetting, ViString fileName)
{
    ViChar buffer[BUFFER_SIZE];

    CHECKVIINT32(instrSession, instrumentSetting, 0, 10);
    CHECKNULL (fileName);

    sprintf (buffer, "MMEM:STOR:STAT %ld,'%s'", instrumentSetting, fileName);

    return rssmb_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssmb_setRegister
 * Purpose:  This function sets the value of selected status register
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_setRegister (ViSession instrSession, ViInt32 registerGroup,
                            ViInt32 registerType, ViInt32 value)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32(instrSession, registerGroup, 0, 1);
    CHECKVIINT32(instrSession, registerType, 2, 4);

    sprintf (buffer, "STAT:%s:%s", regGroupArr[registerGroup], regTypeArr[registerType]);

    return SETPARAMINT32 (instrSession,
                             buffer,
                             value,
                             0, 0);
}

/*============================================================================
 * Function: rssmb_getRegister
 * Purpose:  This function returns the value of selected status register
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_getRegister (ViSession instrSession, ViInt32 registerGroup,
                            ViInt32 registerType, ViInt32* value)
{
    ViChar  buffer[BUFFER_SIZE];

    CHECKVIINT32(instrSession, registerGroup, 0, 1);
    CHECKVIINT32(instrSession, registerType, 0, 4);

    sprintf (buffer, "STAT:%s:%s?", regGroupArr[registerGroup], regTypeArr[registerType]);

    return GETPARAMINT32 (instrSession,
                             buffer,
                             value);
}

/*============================================================================
 * Function: rssmb_STATusPRESet
 * Purpose:  This function resets the status registers. All PTRansition parts
 *           are set to FFFFh (32767), i.e. all transitions from 0 to 1 are
 *           detected. All NTRansition parts are set to 0, i.e. a transition
 *           from 1 to 0 in a CONDition bit is not detected. The ENABle parts
 *           of STATus:OPERation and STATus:QUEStionable are set to 0, i.e. all
 *           events in these registers are not passed on.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_STATusPRESet (ViSession instrSession)
{
    return rssmb_sendCmd (instrSession, "STAT:PRES");
}

/*============================================================================
 * Function: rssmb_SetSYSTemBeeper
 * Purpose:  This function switches the key beep on or off.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemBeeper (ViSession instrSession,
                                ViBoolean systemBeeper)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:BEEP:STAT",
                               systemBeeper);
}

/*============================================================================
 * Function: rssmb_GetSYSTemBeeper
 * Purpose:  This function returns status of the key beep on or off.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemBeeper (ViSession instrSession,
                                ViBoolean *systemBeeper)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:BEEP:STAT?",
                               systemBeeper);
}

/*============================================================================
 * Function: rssmb_SYSTemErrorQueueAll
 * Purpose:  This function queries all entries in the rssmb_status queue and then
 *           deletes them. Positive error numbers denote instrument-specific
 *           errors, and negative rssmb_status numbers denote rssmb_status messages defined
 *           by SCPI (see Chapter 9). If the error queue is empty, 0 ("No
 *           error") is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemErrorQueueAll (ViSession instrSession,
                                    ViInt32 arraySize, ViChar _VI_FAR errorQueue[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:ERR:ALL?",
                              errorQueue,
                              (ViUInt32) arraySize);
}

/*============================================================================
 * Function: rssmb_SYSTemErrorQueueCodeAll
 * Purpose:  This function queries all entries in the error queue and then
 *           deletes them. Only the error numbers are returned and not the
 *           entire error text. Positive error numbers denote
 *           instrument-specific errors, and negative error numbers denote
 *           error messages defined by SCPI (see Chapter 9). If the error queue
 *           is empty, 0 ("No Error") is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemErrorQueueCodeAll (ViSession instrSession,
                                        ViInt32 arraySize,
                                        ViChar _VI_FAR errorQueueCode[])
{
    return GETPARAMSTRING (instrSession,
                              "SYST:ERR:CODE:ALL?",
                              errorQueueCode,
                              (ViUInt32) arraySize);
}

/*============================================================================
 * Function: rssmb_SYSTemErrorQueueCodeNext
 * Purpose:  This function queries the oldest entry in the error queue and
 *           then deletes it. Only the error number is returned and not the
 *           entire error text. Positive error numbers denote
 *           instrument-specific errors, and negative error numbers denote
 *           error messages defined by SCPI (see Chapter 9). If the error queue
 *           is empty, 0 is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemErrorQueueCodeNext (ViSession instrSession,
                                         ViInt32 *errorQueueCodeNext)
{
    return GETPARAMINT32 (instrSession,
                             "SYST:ERR:CODE?",
                             errorQueueCodeNext);
}

/*============================================================================
 * Function: rssmb_SYSTemErrorQueueCount
 * Purpose:  This function queries the number of entries in the error queue.
 *           If the error queue is empty, '0' is returned.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemErrorQueueCount (ViSession instrSession,
                                      ViInt32 *errorQueueCount)
{
    return GETPARAMINT32 (instrSession,
                             "SYST:ERR:COUN?",
                             errorQueueCount);
}

/*===========================================================================
 * Function: SYSTem USB Resource
 * Purpose:  This function queries the USB resource.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SYSTemUSBResource (ViSession instrSession,
                                           ViUInt32 arraySize,
                                           ViChar USBResource[])
{
    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:USB:RES?",
                              USBResource,
                              arraySize);
}

/*****************************************************************************
 * Function: SYSTem GPIB Resource
 * Purpose:  This function queries the visa resource string for the GPIB
 *			 remote control interface. This string is used for remote control
 *			 of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SYSTemGPIBResource(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar    gpibResource[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:GPIB:RES?",
                              gpibResource,
                              (ViUInt32) arraySize);
}

/*****************************************************************************
 * Function: SYSTem Serial Resource
 * Purpose:  This function queries the visa resource string for the serial
 *			 remote control interface. This string is used for remote control
 *			 of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SYSTemSerialResource(
    ViSession   instrSession,
    ViInt32 arraySize,
    ViChar    serialResource[]
)
{
    INIT;

    return GETPARAMSTRING (instrSession,
                              ":SYST:COMM:SER:RES?",
                              serialResource,
                              (ViUInt32) arraySize);
}

/*****************************************************************************
 * Function: SYSTem Option User Defined
 * Purpose:  This function defines the OPT string for selection user defined
 *			 (SYST:IDEN USER).
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SYSTemOptionUserDefined(
    ViSession   instrSession,
    ViString    userDefinedIdentification
)
{
    INIT;

    return SETPARAMSTRING (instrSession,
                              ":SYST:ORES",
                              userDefinedIdentification);
}

/*****************************************************************************
 * Function: SYSTem Factory Preset
 * Purpose:  This function triggers an instrument reset to the original state 
 *           of delivery.
 *           
 *           Since Factory Preset resets the Remote Channel and network 
 *           settings to the default values, executing Factory Preset via 
 *           remote control terminates the connection to the instrument, if 
 *           these settings had been configured to values different to the 
 *           default ones!
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_SYSTemFactoryPreset(
	ViSession	instrSession
)
{
    INIT;

    return rssmb_sendCmd (instrSession, ":SYST:FPR");
}

/*============================================================================
 * Function: rssmb_SetSYSTemKeyboardLock
 * Purpose:  This function (Keyboard LOCk) disables the front panel keyboard
 *           of the R&S SMA including the  LOCAL  key, or enables it again
 *           (OFF).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemKeyboardLock (ViSession instrSession,
                                      ViBoolean keyboardLock)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:KLOC",
                               keyboardLock);
}

/*============================================================================
 * Function: rssmb_GetSYSTemKeyboardLock
 * Purpose:  This function returns if the front panel keyboard of the R&S SMA
 *           including the LOCAL key is disabled or enabled.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemKeyboardLock (ViSession instrSession,
                                      ViBoolean *keyboardLock)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:KLOC?",
                               keyboardLock);
}

/*============================================================================
 * Function: rssmb_SetSYSTemDisplayLock
 * Purpose:  This function (Display LOCk) disables the manual operation via
 *           the display, or enables it again (OFF).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemDisplayLock (ViSession instrSession,
                                              ViBoolean displayLock)
{
    return SETPARAMBOOL (instrSession,
                               "SYST:DLOC",
                               displayLock);
}

/*============================================================================
 * Function: rssmb_GetSYSTemDisplayLock
 * Purpose:  This function returns if the front panel keyboard of the R&S
 *           Signal Generator including the LOCAL key is disabled or enabled.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemDisplayLock (ViSession instrSession,
                                              ViBoolean *displayLock)
{
    return GETPARAMBOOL (instrSession,
                               "SYST:DLOC?",
                               displayLock);
}

/*============================================================================
 * Function: rssmb_SetSYSTemProtect
 * Purpose:  This function activates and deactivates the specified protection
 *           level. There are several protection levels which disable specific
 *           service functions (authorized personnel of R&S Service Deparments
 *           only).
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_SetSYSTemProtect (ViSession instrSession,
                                 ViInt32 protectionLevel, ViBoolean protection,
                                 ViString password)
{
    ViChar  buffer [BUFFER_SIZE];

    CHECKVIINT32(instrSession, protectionLevel, 1, 5);
    CHECKNULL (password);

    if (protection == VI_FALSE)
        sprintf (buffer, "SYST:PROT%ld OFF,%s", protectionLevel, password);
    else
        sprintf (buffer, "SYST:PROT%ld ON", protectionLevel);

    return rssmb_sendCmd (instrSession, buffer);
}

/*============================================================================
 * Function: rssmb_GetSYSTemVersion
 * Purpose:  This function queries the SCPI version with which the instrument
 *           complies.
 *===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemVersion (ViSession instrSession,
                                 ViChar _VI_FAR SCPIVersion[])
{
    ViStatus rssmb_status = VI_SUCCESS;

    rssmb_status = GETPARAMSTRING (instrSession,
                              "SYST:VERS?",
                              SCPIVersion,
                              32);

    return (rssmb_status < VI_SUCCESS) ? rssmb_status : VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get SYSTem Startup Status
/* Purpose:  This function returns if the startup of the instrument is 
/*           completed.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_GetSYSTemStartupStatus(
    ViSession   instrSession,
    ViInt32*    startupComplete
)
{
    return GETPARAMINT32 (instrSession,
                             "SYST:STAR:COMP?",
                             startupComplete);
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_readToFile (ViSession instrSession, ViString source,
                           ViString destination)
{
#if 0
    ViStatus    rssmb_status    = VI_SUCCESS;
    ViUInt16    intfType        = VI_INTF_GPIB;
    ViUInt32    count           = 0,
                lengthOfLength  = 0;
    ViChar      buffer[BUFFER_SIZE],
                format[32],
                *pbuffer = NULL, *p2buffer = NULL;
    FILE        *targetFile;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfType));

    if (intfType == VI_INTF_GPIB)
        CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL));

    sprintf (buffer, ":MMEM:DATA? '%s'\n", source);

    CHECKERR (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen(buffer), VI_NULL));
    CHECKERR (rssmb_readDataUnknownLength (instrSession, &pbuffer, &count));

    p2buffer = pbuffer;

    if (sscanf (p2buffer, "#%1lu", &lengthOfLength) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    sprintf (format, "%%%lud", lengthOfLength);
    p2buffer += 2;

    if (sscanf (p2buffer, format, &count) != 1)
        {
        free (pbuffer);
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }

    p2buffer += lengthOfLength;

    if (intfType == VI_INTF_GPIB)
        if ((rssmb_status = viWrite (instrSession, (ViBuf) ":SYST:COMM:GPIB:LTER STAN\n", 26, VI_NULL)) < VI_SUCCESS)
            {
            free (pbuffer);
            return rssmb_status;
            }

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < VI_SUCCESS)
        {
        free (pbuffer);
        return rssmb_status;
        }

    /* Store data from instrument to file on host computer */
    if ((targetFile = fopen (destination, "wb")) == NULL)
        {
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != (ViUInt32) fwrite (p2buffer, sizeof (ViByte), (size_t)(count), targetFile))
        {
        (void) fclose (targetFile);
        if (pbuffer)
            free (pbuffer);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (pbuffer) /* If not NULL pointer, then deallocate memory. */
        free (pbuffer);

    if ((rssmb_status = rssmb_checkStatus (instrSession)) < 0)
        return rssmb_status;

    return rssmb_status;
#endif
    ViChar      buffer[BUFFER_SIZE];
    
    sprintf (buffer, ":MMEM:DATA? '%s'\n", source);
    
    return rssmb_readDataToFile (instrSession, buffer, destination);
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_writeFromFile (ViSession instrSession, ViString source,
                              ViString destination)
{
    ViStatus    rssmb_status;
    FILE        *source_file;
    ViInt32     file_size;
    ViByte      *data;
    
    
    if ((source_file = fopen (source, "rb")) == NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    (void) fseek (source_file, 0, SEEK_END);
    file_size = ftell (source_file);
    (void) fseek (source_file, 0, SEEK_SET);

    data=malloc(sizeof(char)*file_size);
    (void) fread (data, sizeof (ViByte), (size_t) file_size, source_file);
    (void) fclose (source_file);

    rssmb_status = rssmb_MMEMoryBlockDataWrite (instrSession, destination, file_size,
                                           (ViChar*) data);

    free (data);
    return rssmb_status;
}

/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_errorCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    rssmb_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    CHECKNULL (instrPtr);
    
    instrPtr -> errorChecking = state;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Option Checking                                                 */
/* Purpose:  This function switches ON/OFF option checking of the instrument */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_optionCheckState (ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    rssmb_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL)
        instrPtr -> optionChecking = state;

    return rssmb_status;
    
}

/*****************************************************************************
 * Function: Range Checking
 * Purpose:  This function switches range checking of the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssmb_setCheckRange(
    ViSession   instrSession,
    ViBoolean   rangeChecking
)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    rssmb_instrRange instrPtr;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> rangeChecking = rangeChecking;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rssmb_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_readInstrData (ViSession instrSession,
                                        ViInt32 numberBytesToRead,
                                        ViChar _VI_FAR readBuffer[],
                                        ViPInt32 numBytesRead)
{
    ViStatus    rssmb_status = VI_SUCCESS;
    ViUInt32    tmpCnt = 0;

    if (numBytesRead != NULL)
        *numBytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, (ViUInt32) numberBytesToRead, &tmpCnt));

    if (numBytesRead)
        *numBytesRead = (ViInt32)tmpCnt;

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Device Preset
/* Purpose:  This function presets all parameters which are not related to 
/*           the signal path. This includes presetting the LF generator.
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_DevicePreset(
    ViSession   instrSession
)
{
    return rssmb_sendCmd (instrSession, "DEV:PRES");
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_reset (ViSession instrSession)
{
    ViStatus rssmb_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST;*WAI;*CLS\n"));
    CHECKERR (rssmb_defaultInstrSetup (instrSession));

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. If the self test function is not supported by the   */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_SELF_TEST.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_self_test (ViSession instrSession,
                                    ViPInt16 testResult,
                                    ViChar _VI_FAR testMessage[])
{
    ViStatus rssmb_status = VI_SUCCESS;

    CHECKERR( viQueryf (instrSession, "*TST?\n", "%hd", testResult));

    sprintf (testMessage, "%s", (*testResult == 0) ? "Self-Test Passed" : "Self-Test Failed");

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result. If the error query function is not supported by the */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_ERROR_QUERY.                                       */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_error_query (ViSession instrSession,
                                      ViPInt32 errorCode,
                                      ViChar _VI_FAR errorMessage[])
{
    ViStatus    rssmb_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
	ViUInt32    retCnt = 0;

	p2buf = (char *)malloc (1);

    *p2buf='\0';
        
    do
	{
		CHECKERR (viWrite (instrSession, (ViBuf) ":SYST:ERR?\n", 11, VI_NULL));
		CHECKERR (viRead (instrSession, (ViBuf)buffer, BUFFER_SIZE, &retCnt));

	    if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
        {
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        }
        else
        {
            if (oldCode != 0)
            {
                *errorCode = oldCode;
                p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
                strcat (p2buf, oldMessage);
                strcat (p2buf, "\n");
                i++;
            }
            else if (i == 0)
            {
                *errorCode = oldCode;
                strncpy (errorMessage, oldMessage, 1024);
            }
        }

    } while (oldCode != 0);
    
    if (i > 0)
    {
        p2buf[strlen (p2buf) - 1] = '\0'; 
        strncpy (errorMessage, p2buf, 1024);
    }
   
	if (p2buf) free (p2buf);

    return rssmb_status;    
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_error_message (ViSession instrSession,
                                        ViStatus statusCode,
                                        ViChar _VI_FAR errMessage[])
{
    ViStatus rssmb_status = VI_SUCCESS;
    ViInt16 i;

    static rssmb_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,                 "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,                    "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,                "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,              "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,                "WARNING: Revision Query not supported"},
        {VI_ERROR_FAIL_ID_QUERY,                "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,                 "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,              "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,             "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE,  "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER,              "ERROR: Parameter out of range"},

        /* --- Instrument specific error codes --- */

        {RSSMB_ERROR_INSTRUMENT_ERROR,          "ERROR: Instrument status error"},
        {RSSMB_ERROR_INVALID_CONFIGURATION,     "ERROR: Instrument configuration error"},
        {RSSMB_ERROR_INSTRUMENT_OPTION,         "ERROR: Option not installed"},
        {RSSMB_ERROR_INSTRUMENT_NSUP_MODEL,     "ERROR: Required Instrument model is not available."},
        {RSSMB_ERROR_SETTINGS_CONFLICT,         "ERROR: Instrument settings conflict."},
        {RSSMB_ERROR_CALIBRATION_FAILED,        "ERROR: Calibration failed."},
        {RSSMB_ERROR_INSTRUMENT_DATA_NOT_AVAILABLE, "ERROR: Data not available."},

        /* --- Instrument specific warnings --- */

        {RSSMB_WARNING_SMALL_BUFFER,            "WARNING: Buffer too small"},

        {VI_NULL, VI_NULL}
    };

    if (statusCode == VI_ERROR_INSTR_PARAMETER)
	{
	    rssmb_instrRange   instrPtr;
	    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
		sprintf (errMessage, "ERROR: Invalid parameter '%s'", instrPtr->parameterErrorMessage);
	}
	else
	{
		rssmb_status = viStatusDesc (instrSession, statusCode, errMessage);
	    if ((rssmb_status != VI_SUCCESS) || (strstr (errMessage, "Unknown status value") != NULL))
	        {
	        for (i = 0; statusDescArray[i].stringName; i++)
	            {
	            if (statusDescArray[i].stringVal == statusCode)
	                {
	                sprintf (errMessage, "%s", statusDescArray[i].stringName);
	                return (VI_SUCCESS);
	                }
	            }
	        sprintf (errMessage, "Unknown Error 0x%X", statusCode);
	        return (VI_WARN_UNKNOWN_STATUS);
	        }
	}

    rssmb_status = VI_SUCCESS;
    return rssmb_status;
}
/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_revision_query (ViSession instrSession,
                    ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
    ViStatus rssmb_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    //CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", instrRev));
    CHECKERR (viScanf (instrSession, "%*[^-]-%[^ (\r\n]", instrRev));

    sprintf (driverRev, "%s", RSSMB_REVISION);

    return rssmb_status;
}

/*===========================================================================*/
/* Function: Delay                                                           */
/* Purpose : With a call to rssmb_delay, the current program is suspended    */
/*           from execution for the number of seconds specified by the       */
/*           argument seconds. rssmb_delay is accurate to a millisecond.     */
/*           This function is platform independent.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_delay (/*@unused@*/ViSession instrSession, ViReal64 seconds)
{
  clock_t StartTime = clock();
  instrSession;
  
  while ((((double)(clock() - StartTime))/CLOCKS_PER_SEC) < (seconds)){
  #ifdef _WIN32
    #ifdef _CVI_
          ProcessSystemEvents ();
    #else
          Sleep(0);
    #endif
  #endif
  }
  return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssmb_close (ViSession instrSession)
{
    rssmb_instrRange instrPtr = NULL;
    ViSession rmSession = 0;
    ViStatus rssmb_status = VI_SUCCESS;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL)
        free (instrPtr);

    rssmb_status = viClose (instrSession);
    (void) viClose (rmSession);

    return rssmb_status;
}

/*===========================================================================*/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*===========================================================================*/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rssmb_reset operation and by the     */
/*           rssmb_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rssmb_defaultInstrSetup (ViSession instrSession)
{
    ViChar              buffer[BUFFER_SIZE] = "";

    ViStatus            rssmb_status = VI_SUCCESS;
    rssmb_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    CHECKNULL (instrPtr);
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = VI_TRUE;
    instrPtr -> optionChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSSMB_REVISION);

    /* Check for instrument version */
    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]", buffer));
    
    /* Check for instruments options */
    if ((rssmb_status = rssmb_OptionIdentificationQuery (instrSession, buffer)) < VI_SUCCESS)
        return rssmb_status;
    else
        rssmb_status = VI_SUCCESS;

    instrPtr->instrOptions.B1 = (strstr (buffer, "SMB-B1c") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.B106 = (strstr (buffer, "SMB-B106") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K22 = (strstr (buffer, "SMB-K24") != VI_NULL) ? 1 : 0;
    instrPtr->instrOptions.K23 = (strstr (buffer, "SMB-K23") != VI_NULL) ? 1 : 0;
	instrPtr->instrOptions.K27 = (strstr (buffer, "SMB-K27") != VI_NULL) ? 1 : 0;
	instrPtr->instrOptions.K28 = (strstr (buffer, "SMB-K28") != VI_NULL) ? 1 : 0;
    
    return rssmb_status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssmb_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssmb_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssmb_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rssmb_instrRange instrPtr = NULL;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    (void) viClose (*openInstrSession);
    (void) viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*============================================================================
 * Function: rssmb_SetSweepMode
 * Purpose:  This function sets the sweep mode.
 *===========================================================================*/
ViStatus rssmb_SetSweepMode (ViSession instrSession, 
                             ViInt32   mode)
{
    
    return SETPARAMENUM (instrSession,
                          "SOUR:SWE:MODE",
                          mode,
                          lfSweepModeArr);
}

/*============================================================================
 * Function: rssmb_SetFrequencySweepTriggerSource
 * Purpose:  This function sets the trigger source for the RF frequency sweep.
 *===========================================================================*/
ViStatus rssmb_SetFrequencySweepTriggerSource (ViSession instrSession,
                                               ViInt32 triggerSourceRFFreqSweep)
{
    return SETPARAMENUM (instrSession,
                          "TRIG:FSW:SOUR",
                          triggerSourceRFFreqSweep,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: rssmb_SetFrequencyMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output frequency.
 *===========================================================================*/
ViStatus rssmb_SetFrequencyMode (ViSession instrSession,
                                 ViInt32 frequencyMode)
{
    return SETPARAMENUM (instrSession,
                          "SOUR:FREQ:MODE",
                          frequencyMode,
                          frequencyModeArr);
}

/*============================================================================
 * Function: rssmb_SetLFMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output frequency.
 *===========================================================================*/
ViStatus rssmb_SetLFMode (ViSession instrSession, ViInt32 LFOutputMode)
{
    return SETPARAMENUM (instrSession,
                          "LFO:FREQ:MODE",
                          LFOutputMode,
                          lfOutputModeArr);
}

/*============================================================================
 * Function: rssmb_SetLFOutputLFSweepMode
 * Purpose:  This function sets the cycle mode of the LF sweep.
 *===========================================================================*/
ViStatus rssmb_SetSweepLFMode(ViSession    instrSession,
                              ViInt32  lfSweepMode)
{
    return SETPARAMENUM (instrSession,
                          "LFO:SWE:MODE",
                          lfSweepMode,
                          lfSweepModeArr);
}

/*============================================================================
 * Function: rssmb_SetLFSweepTriggerSource
 * Purpose:  This function sets the trigger source for the LF sweep. The
 *           trigger is triggered by the command
 *           :SOURce:LFOutput:SWEep[:FREQuency]EXECute.
 *===========================================================================*/
ViStatus rssmb_SetLFSweepTriggerSource (ViSession instrSession,
                                        ViInt32 triggerSourceLFSweep)
{
    return SETPARAMENUM (instrSession,
                          "TRIG0:SWE:SOUR",
                          triggerSourceLFSweep,
                          triggerRFSourceArr);
}

/*============================================================================
 * Function: rssmb_SetLevelSweepTriggerSource
 * Purpose:  This function sets the trigger source for the RF level sweep.
 *===========================================================================*/
ViStatus rssmb_SetLevelSweepTriggerSource (ViSession instrSession,
                                             ViInt32 triggerSourceRFLevelSweep)
{
    return SETPARAMENUM (instrSession,
                              "TRIG:PSW:SOUR",
                              triggerSourceRFLevelSweep,
                              triggerRFSourceArr);
}

/*============================================================================
 * Function: rssmb_SetSweepLevelMode
 * Purpose:  This function sets the cycle mode of the level sweep.
 *===========================================================================*/
ViStatus rssmb_SetSweepLevelMode (ViSession instrSession,
                                  ViInt32 sweepLevelMode)
{
    return SETPARAMENUM (instrSession,
                              "SOUR:SWE:POW:MODE",
                              sweepLevelMode,
                              lfSweepModeArr);
}

/*============================================================================
 * Function: rssmb_SetLevelMode
 * Purpose:  This function sets the instrument operating mode and therefore
 *           also the functions used to set the output level.
 *===========================================================================*/
ViStatus rssmb_SetLevelMode (ViSession instrSession,
                                    ViInt32 RFLevelMode)
{
    return SETPARAMENUM (instrSession,
                              "SOUR:POW:MODE",
                              RFLevelMode,
                              frequencyModeArr);
}
